import type { Classnames, DefaultCombinatorName, DefaultCombinatorNameExtended, DefaultOperatorName, MatchMode, Path, StringUnionToFullOptionArray } from "./types/index.noReact.mjs";
/**
* @group Defaults
*/
export declare const defaultPlaceholderName = "~";
/**
* @group Defaults
*/
export declare const defaultPlaceholderLabel = "------";
/**
* Default `name` for placeholder option in the `fields` array.
*
* @group Defaults
*/
export declare const defaultPlaceholderFieldName: typeof defaultPlaceholderName;
/**
* Default `label` for placeholder option in the `fields` array.
*
* @group Defaults
*/
export declare const defaultPlaceholderFieldLabel: typeof defaultPlaceholderLabel;
/**
* Default `label` for placeholder option group in the `fields` array.
*
* @group Defaults
*/
export declare const defaultPlaceholderFieldGroupLabel: typeof defaultPlaceholderLabel;
/**
* Default `name` for placeholder option in the `operators` array.
*
* @group Defaults
*/
export declare const defaultPlaceholderOperatorName: typeof defaultPlaceholderName;
/**
* Default `label` for placeholder option in the `operators` array.
*
* @group Defaults
*/
export declare const defaultPlaceholderOperatorLabel: typeof defaultPlaceholderLabel;
/**
* Default `label` for placeholder option group in the `operators` array.
*
* @group Defaults
*/
export declare const defaultPlaceholderOperatorGroupLabel: typeof defaultPlaceholderLabel;
/**
* Default `name` for placeholder option in the `values` array.
*
* @group Defaults
*/
export declare const defaultPlaceholderValueName: typeof defaultPlaceholderName;
/**
* Default `label` for placeholder option in the `values` array.
*
* @group Defaults
*/
export declare const defaultPlaceholderValueLabel: typeof defaultPlaceholderLabel;
/**
* Default `label` for placeholder option group in the `values` array.
*
* @group Defaults
*/
export declare const defaultPlaceholderValueGroupLabel: typeof defaultPlaceholderLabel;
/**
* Default character used to `.join` and `.split` arrays.
*
* @group Defaults
*/
export declare const defaultJoinChar = ",";
export type DefaultOperators = StringUnionToFullOptionArray<DefaultOperatorName>;
export declare const defaultOperatorLabelMap: Record<DefaultOperatorName, string>;
export declare const defaultCombinatorLabelMap: Record<DefaultCombinatorNameExtended, string>;
/**
* Default operator list.
*
* @group Defaults
*/
export declare const defaultOperators: DefaultOperators;
/**
* Map of default operators to their respective opposite/negating operators.
*
* @group Defaults
*/
export declare const defaultOperatorNegationMap: Record<DefaultOperatorName, DefaultOperatorName>;
export type DefaultCombinators = StringUnionToFullOptionArray<DefaultCombinatorName>;
/**
* Default combinator list.
*
* @group Defaults
*/
export declare const defaultCombinators: DefaultCombinators;
export type DefaultCombinatorsExtended = StringUnionToFullOptionArray<DefaultCombinatorNameExtended>;
/**
* Default combinator list, with `XOR` added.
*
* @group Defaults
*/
export declare const defaultCombinatorsExtended: DefaultCombinatorsExtended;
export type DefaultMatchModes = StringUnionToFullOptionArray<MatchMode>;
/**
* Default match modes.
*
* @group Defaults
*/
export declare const defaultMatchModes: DefaultMatchModes;
/**
* Standard classnames applied to each component.
*
* @group Defaults
*/
export declare const standardClassnames: {
	readonly queryBuilder: "queryBuilder";
	readonly ruleGroup: "ruleGroup";
	readonly header: "ruleGroup-header";
	readonly body: "ruleGroup-body";
	readonly combinators: "ruleGroup-combinators";
	readonly addRule: "ruleGroup-addRule";
	readonly addGroup: "ruleGroup-addGroup";
	readonly cloneRule: "rule-cloneRule";
	readonly cloneGroup: "ruleGroup-cloneGroup";
	readonly removeGroup: "ruleGroup-remove";
	readonly notToggle: "ruleGroup-notToggle";
	readonly rule: "rule";
	readonly fields: "rule-fields";
	readonly matchMode: "rule-matchMode";
	readonly matchThreshold: "rule-matchThreshold";
	readonly operators: "rule-operators";
	readonly value: "rule-value";
	readonly removeRule: "rule-remove";
	readonly betweenRules: "betweenRules";
	readonly valid: "queryBuilder-valid";
	readonly invalid: "queryBuilder-invalid";
	readonly shiftActions: "shiftActions";
	readonly dndDragging: "dndDragging";
	readonly dndOver: "dndOver";
	readonly dndCopy: "dndCopy";
	readonly dndGroup: "dndGroup";
	readonly dragHandle: "queryBuilder-dragHandle";
	readonly disabled: "queryBuilder-disabled";
	readonly lockRule: "rule-lock";
	readonly lockGroup: "ruleGroup-lock";
	readonly valueSource: "rule-valueSource";
	readonly valueListItem: "rule-value-list-item";
	readonly branches: "queryBuilder-branches";
	readonly justified: "queryBuilder-justified";
	readonly hasSubQuery: "rule-hasSubQuery";
};
/**
* Default classnames for each component.
*
* @group Defaults
*/
export declare const defaultControlClassnames: Classnames;
/**
* Default reason codes for a group being invalid.
*
* @group Defaults
*/
export declare const groupInvalidReasons: {
	readonly empty: "empty";
	readonly invalidCombinator: "invalid combinator";
	readonly invalidIndependentCombinators: "invalid independent combinators";
};
/**
* Component identifiers for testing.
*
* @group Defaults
*/
export declare const TestID: {
	readonly rule: "rule";
	readonly ruleGroup: "rule-group";
	readonly inlineCombinator: "inline-combinator";
	readonly addGroup: "add-group";
	readonly removeGroup: "remove-group";
	readonly cloneGroup: "clone-group";
	readonly cloneRule: "clone-rule";
	readonly addRule: "add-rule";
	readonly removeRule: "remove-rule";
	readonly combinators: "combinators";
	readonly fields: "fields";
	readonly operators: "operators";
	readonly valueEditor: "value-editor";
	readonly notToggle: "not-toggle";
	readonly shiftActions: "shift-actions";
	readonly dragHandle: "drag-handle";
	readonly lockRule: "lock-rule";
	readonly lockGroup: "lock-group";
	readonly valueSourceSelector: "value-source-selector";
	readonly matchModeEditor: "match-mode-editor";
};
export declare const LogType: {
	readonly parentPathDisabled: "action aborted: parent path disabled";
	readonly pathDisabled: "action aborted: path is disabled";
	readonly queryUpdate: "query updated";
	readonly onAddRuleFalse: "onAddRule callback returned false";
	readonly onAddGroupFalse: "onAddGroup callback returned false";
	readonly onGroupRuleFalse: "onGroupRule callback returned false";
	readonly onGroupGroupFalse: "onGroupGroup callback returned false";
	readonly onMoveRuleFalse: "onMoveRule callback returned false";
	readonly onMoveGroupFalse: "onMoveGroup callback returned false";
	readonly onRemoveFalse: "onRemove callback returned false";
	readonly add: "rule or group added";
	readonly remove: "rule or group removed";
	readonly update: "rule or group updated";
	readonly move: "rule or group moved";
	readonly group: "rule or group grouped with another";
};
/**
* The {@link Path} of the root group.
*
* @group Defaults
*/
export declare const rootPath: Path;
