import type { TypedUseSelectorHook } from "react-redux";
import type { RuleGroupTypeAny } from "../types/index.noReact.mjs";
import type { RqbState } from "./types.mjs";
/**
* A Redux `useSelector` hook for RQB's internal store. See also {@link getQuerySelectorById}.
*
* **TIP:** Prefer {@link useQueryBuilderQuery} if you only need to access the query object
* for the nearest ancestor {@link QueryBuilder} component.
*
* @group Hooks
*/
export declare const useQueryBuilderSelector: TypedUseSelectorHook<RqbState>;
/**
* Retrieves the full, latest query object for the nearest ancestor {@link QueryBuilder}
* component.
*
* The optional parameter should only be used when retrieving a query object from a different
* {@link QueryBuilder} than the nearest ancestor. It can be a full props object as passed
* to a custom component or any object matching the interface `{ schema: { qbId: string } }`.
*
* Must follow React's [Rules of Hooks](https://react.dev/warnings/invalid-hook-call-warning).
*
* @group Hooks
*/
export declare const useQueryBuilderQuery: (props?: {
	schema: {
		qbId: string;
	};
}) => RuleGroupTypeAny;
