// src/context/QueryBuilderContext.ts
import { createContext } from "react";
var QueryBuilderContext = /* @__PURE__ */ createContext({});

// src/components/ActionElement.tsx
import { jsx } from "react/jsx-runtime";
var ActionElement = (props) => /* @__PURE__ */ jsx("button", { type: "button", "data-testid": props.testID, disabled: props.disabled && !props.disabledTranslation, className: props.className, title: props.disabledTranslation && props.disabled ? props.disabledTranslation.title : props.title, onClick: (e) => props.handleOnClick(e), children: props.disabledTranslation && props.disabled ? props.disabledTranslation.label : props.label });

// src/components/DragHandle.tsx
import { jsx as jsx2 } from "react/jsx-runtime";
import { forwardRef } from "react";
var DragHandle = /* @__PURE__ */ forwardRef((props, dragRef) => /* @__PURE__ */ jsx2("span", { "data-testid": props.testID, ref: dragRef, className: props.className, title: props.title, children: props.label }));

// src/defaults.ts
var defaultPlaceholderName = "~";
var defaultPlaceholderLabel = "------";
var defaultPlaceholderFieldName = defaultPlaceholderName;
var defaultPlaceholderFieldLabel = defaultPlaceholderLabel;
var defaultPlaceholderFieldGroupLabel = defaultPlaceholderLabel;
var defaultPlaceholderOperatorName = defaultPlaceholderName;
var defaultPlaceholderOperatorLabel = defaultPlaceholderLabel;
var defaultPlaceholderOperatorGroupLabel = defaultPlaceholderLabel;
var defaultPlaceholderValueName = defaultPlaceholderName;
var defaultPlaceholderValueLabel = defaultPlaceholderLabel;
var defaultPlaceholderValueGroupLabel = defaultPlaceholderLabel;
var defaultJoinChar = ",";
var defaultOperatorLabelMap = { "=": "=", "!=": "!=", "<": "<", ">": ">", "<=": "<=", ">=": ">=", contains: "contains", beginsWith: "begins with", endsWith: "ends with", doesNotContain: "does not contain", doesNotBeginWith: "does not begin with", doesNotEndWith: "does not end with", null: "is null", notNull: "is not null", in: "in", notIn: "not in", between: "between", notBetween: "not between" };
var defaultCombinatorLabelMap = { and: "AND", or: "OR", xor: "XOR" };
var defaultOperators = [{ name: "=", value: "=", label: "=" }, { name: "!=", value: "!=", label: "!=" }, { name: "<", value: "<", label: "<" }, { name: ">", value: ">", label: ">" }, { name: "<=", value: "<=", label: "<=" }, { name: ">=", value: ">=", label: ">=" }, { name: "contains", value: "contains", label: "contains" }, { name: "beginsWith", value: "beginsWith", label: "begins with" }, { name: "endsWith", value: "endsWith", label: "ends with" }, { name: "doesNotContain", value: "doesNotContain", label: "does not contain" }, { name: "doesNotBeginWith", value: "doesNotBeginWith", label: "does not begin with" }, { name: "doesNotEndWith", value: "doesNotEndWith", label: "does not end with" }, { name: "null", value: "null", label: "is null" }, { name: "notNull", value: "notNull", label: "is not null" }, { name: "in", value: "in", label: "in" }, { name: "notIn", value: "notIn", label: "not in" }, { name: "between", value: "between", label: "between" }, { name: "notBetween", value: "notBetween", label: "not between" }];
var defaultOperatorNegationMap = { "=": "!=", "!=": "=", "<": ">=", "<=": ">", ">": "<=", ">=": "<", beginsWith: "doesNotBeginWith", doesNotBeginWith: "beginsWith", endsWith: "doesNotEndWith", doesNotEndWith: "endsWith", contains: "doesNotContain", doesNotContain: "contains", between: "notBetween", notBetween: "between", in: "notIn", notIn: "in", notNull: "null", null: "notNull" };
var defaultCombinators = [{ name: "and", value: "and", label: "AND" }, { name: "or", value: "or", label: "OR" }];
var defaultCombinatorsExtended = [...defaultCombinators, { name: "xor", value: "xor", label: "XOR" }];
var defaultMatchModes = [{ name: "all", value: "all", label: "all" }, { name: "some", value: "some", label: "some" }, { name: "none", value: "none", label: "none" }, { name: "atLeast", value: "atLeast", label: "at least" }, { name: "atMost", value: "atMost", label: "at most" }, { name: "exactly", value: "exactly", label: "exactly" }];
var standardClassnames = { queryBuilder: "queryBuilder", ruleGroup: "ruleGroup", header: "ruleGroup-header", body: "ruleGroup-body", combinators: "ruleGroup-combinators", addRule: "ruleGroup-addRule", addGroup: "ruleGroup-addGroup", cloneRule: "rule-cloneRule", cloneGroup: "ruleGroup-cloneGroup", removeGroup: "ruleGroup-remove", notToggle: "ruleGroup-notToggle", rule: "rule", fields: "rule-fields", matchMode: "rule-matchMode", matchThreshold: "rule-matchThreshold", operators: "rule-operators", value: "rule-value", removeRule: "rule-remove", betweenRules: "betweenRules", valid: "queryBuilder-valid", invalid: "queryBuilder-invalid", shiftActions: "shiftActions", dndDragging: "dndDragging", dndOver: "dndOver", dndCopy: "dndCopy", dndGroup: "dndGroup", dragHandle: "queryBuilder-dragHandle", disabled: "queryBuilder-disabled", lockRule: "rule-lock", lockGroup: "ruleGroup-lock", valueSource: "rule-valueSource", valueListItem: "rule-value-list-item", branches: "queryBuilder-branches", justified: "queryBuilder-justified", hasSubQuery: "rule-hasSubQuery" };
var defaultControlClassnames = { queryBuilder: "", ruleGroup: "", header: "", body: "", combinators: "", addRule: "", addGroup: "", cloneRule: "", cloneGroup: "", removeGroup: "", notToggle: "", rule: "", fields: "", matchMode: "", matchThreshold: "", operators: "", value: "", removeRule: "", shiftActions: "", dragHandle: "", lockRule: "", lockGroup: "", valueSource: "", actionElement: "", valueSelector: "", betweenRules: "", valid: "", invalid: "", dndDragging: "", dndOver: "", dndGroup: "", dndCopy: "", disabled: "", valueListItem: "", branches: "", hasSubQuery: "" };
var groupInvalidReasons = { empty: "empty", invalidCombinator: "invalid combinator", invalidIndependentCombinators: "invalid independent combinators" };
var TestID = { rule: "rule", ruleGroup: "rule-group", inlineCombinator: "inline-combinator", addGroup: "add-group", removeGroup: "remove-group", cloneGroup: "clone-group", cloneRule: "clone-rule", addRule: "add-rule", removeRule: "remove-rule", combinators: "combinators", fields: "fields", operators: "operators", valueEditor: "value-editor", notToggle: "not-toggle", shiftActions: "shift-actions", dragHandle: "drag-handle", lockRule: "lock-rule", lockGroup: "lock-group", valueSourceSelector: "value-source-selector", matchModeEditor: "match-mode-editor" };
var LogType = { parentPathDisabled: "action aborted: parent path disabled", pathDisabled: "action aborted: path is disabled", queryUpdate: "query updated", onAddRuleFalse: "onAddRule callback returned false", onAddGroupFalse: "onAddGroup callback returned false", onGroupRuleFalse: "onGroupRule callback returned false", onGroupGroupFalse: "onGroupGroup callback returned false", onMoveRuleFalse: "onMoveRule callback returned false", onMoveGroupFalse: "onMoveGroup callback returned false", onRemoveFalse: "onRemove callback returned false", add: "rule or group added", remove: "rule or group removed", update: "rule or group updated", move: "rule or group moved", group: "rule or group grouped with another" };
var rootPath = [];

// src/components/InlineCombinator.tsx
import { jsx as jsx3 } from "react/jsx-runtime";

// src/utils/clsx.ts
function toVal(mix) {
  let k;
  let y;
  let str = "";
  if (typeof mix === "string" || typeof mix === "number") {
    str += mix;
  } else if (typeof mix === "object") {
    if (Array.isArray(mix)) {
      const len = mix.length;
      for (k = 0; k < len; k++) {
        if (mix[k] && (y = toVal(mix[k]))) {
          str && (str += " ");
          str += y;
        }
      }
    } else {
      for (y in mix) {
        if (mix[y]) {
          str && (str += " ");
          str += y;
        }
      }
    }
  }
  return str;
}
function clsx() {
  let i = 0;
  let tmp;
  let x;
  let str = "";
  const len = arguments.length;
  for (; i < len; i++) {
    if ((tmp = i < 0 || arguments.length <= i ? void 0 : arguments[i]) && (x = toVal(tmp))) {
      str && (str += " ");
      str += x;
    }
  }
  return str;
}
var clsx_default = clsx;

// src/components/InlineCombinator.tsx
var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp.call(b, prop)) __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols) for (var prop of __getOwnPropSymbols(b)) {
    if (__propIsEnum.call(b, prop)) __defNormalProp(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __objRest = (source, exclude) => {
  var target = {};
  for (var prop in source) if (__hasOwnProp.call(source, prop) && exclude.indexOf(prop) < 0) target[prop] = source[prop];
  if (source != null && __getOwnPropSymbols) for (var prop of __getOwnPropSymbols(source)) {
    if (exclude.indexOf(prop) < 0 && __propIsEnum.call(source, prop)) target[prop] = source[prop];
  }
  return target;
};
var InlineCombinator = (allProps) => {
  const _a = allProps, { component: CombinatorSelectorComponent } = _a, props = __objRest(_a, ["component"]);
  const className = clsx(props.schema.suppressStandardClassnames || standardClassnames.betweenRules, props.schema.classNames.betweenRules);
  return /* @__PURE__ */ jsx3("div", { className, "data-testid": TestID.inlineCombinator, children: /* @__PURE__ */ jsx3(CombinatorSelectorComponent, __spreadProps(__spreadValues({}, props), { testID: TestID.combinators })) });
};

// src/utils/arrayUtils.ts
var splitBy = function(str) {
  let splitChar = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : defaultJoinChar;
  return typeof str === "string" ? str.split(`\\${splitChar}`).map((c2) => c2.split(splitChar)).reduce((prev, curr, idx) => {
    if (idx === 0) {
      return curr;
    }
    return [...prev.slice(0, -1), `${prev.at(-1)}${splitChar}${curr[0]}`, ...curr.slice(1)];
  }, []) : [];
};
var joinWith = function(strArr) {
  let joinChar = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : defaultJoinChar;
  return strArr.map((str) => `${str != null ? str : ""}`.replaceAll(joinChar[0], `\\${joinChar[0]}`)).join(joinChar);
};
var trimIfString = (val) => typeof val === "string" ? val.trim() : val;
var toArray = function(v) {
  let { retainEmptyStrings } = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  return Array.isArray(v) ? v.map((v2) => trimIfString(v2)) : typeof v === "string" ? splitBy(v, defaultJoinChar).filter(retainEmptyStrings ? () => true : (s) => !/^\s*$/.test(s)).map((s) => s.trim()) : typeof v === "number" ? [v] : [];
};
var nullFreeArray = (arr) => arr.every((el) => el === false || (el != null ? el : false) !== false);

// src/utils/misc.ts
import { numericRegex as numericQuantityRegex } from "numeric-quantity";
var lc = (v) => typeof v === "string" ? v.toLowerCase() : v;
var numericRegex = new RegExp(numericQuantityRegex.source.replace(/^\^/, String.raw`^\s*`).replace(/\$$/, String.raw`\s*$`));
var isPojo = (obj) => obj === null || typeof obj !== "object" ? false : Object.getPrototypeOf(obj) === Object.prototype;
var nullOrUndefinedOrEmpty = (value) => value === null || value === void 0 || value === "";

// src/utils/isRuleGroup.ts
var isRuleGroup = (rg) => isPojo(rg) && Array.isArray(rg.rules);
var isRuleGroupType = (rg) => isRuleGroup(rg) && typeof rg.combinator === "string";
var isRuleGroupTypeIC = (rg) => isRuleGroup(rg) && rg.combinator === void 0;

// src/utils/convertQuery.ts
import { produce } from "immer";
var __defProp2 = Object.defineProperty;
var __defProps2 = Object.defineProperties;
var __getOwnPropDescs2 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols2 = Object.getOwnPropertySymbols;
var __hasOwnProp2 = Object.prototype.hasOwnProperty;
var __propIsEnum2 = Object.prototype.propertyIsEnumerable;
var __defNormalProp2 = (obj, key, value) => key in obj ? __defProp2(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues2 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp2.call(b, prop)) __defNormalProp2(a, prop, b[prop]);
  if (__getOwnPropSymbols2) for (var prop of __getOwnPropSymbols2(b)) {
    if (__propIsEnum2.call(b, prop)) __defNormalProp2(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps2 = (a, b) => __defProps2(a, __getOwnPropDescs2(b));
var __objRest2 = (source, exclude) => {
  var target = {};
  for (var prop in source) if (__hasOwnProp2.call(source, prop) && exclude.indexOf(prop) < 0) target[prop] = source[prop];
  if (source != null && __getOwnPropSymbols2) for (var prop of __getOwnPropSymbols2(source)) {
    if (exclude.indexOf(prop) < 0 && __propIsEnum2.call(source, prop)) target[prop] = source[prop];
  }
  return target;
};
var combinatorLevels = ["or", "xor", "and"];
var isSameString = (a, b) => lc(a) === b;
var generateRuleGroupICWithConsistentCombinators = function(rg) {
  let baseCombinatorLevel = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : 0;
  const baseCombinator = combinatorLevels[baseCombinatorLevel];
  if (!rg.rules.includes(baseCombinator)) {
    return baseCombinatorLevel < combinatorLevels.length - 2 ? generateRuleGroupICWithConsistentCombinators(rg, baseCombinatorLevel + 1) : rg;
  }
  return produce(rg, (draft) => {
    let cursor = 0;
    while (cursor < draft.rules.length - 2) {
      if (isSameString(draft.rules[cursor + 1], baseCombinator)) {
        cursor += 2;
        continue;
      }
      const nextBaseCombinatorIndex = draft.rules.findIndex((r, i) => i > cursor && typeof r === "string" && lc(r) === baseCombinator);
      if (nextBaseCombinatorIndex === -1) {
        draft.rules.splice(cursor, draft.rules.length, generateRuleGroupICWithConsistentCombinators(
          // oxlint-disable-next-line typescript/no-explicit-any
          { rules: draft.rules.slice(cursor) },
          baseCombinatorLevel + 1
        ));
        break;
      } else {
        draft.rules.splice(cursor, nextBaseCombinatorIndex - cursor, generateRuleGroupICWithConsistentCombinators(
          // oxlint-disable-next-line typescript/no-explicit-any
          { rules: draft.rules.slice(cursor, nextBaseCombinatorIndex) },
          baseCombinatorLevel + 1
        ));
      }
    }
  });
};
var convertFromIC = (rg) => {
  if (isRuleGroupType(rg)) {
    return rg;
  }
  const processedRG = generateRuleGroupICWithConsistentCombinators(rg);
  const rulesAsMixedList = processedRG.rules.map((r) => typeof r === "string" || !isRuleGroup(r) ? r : convertFromIC(r));
  const combinator = rulesAsMixedList.length < 2 ? "and" : rulesAsMixedList[1];
  const rules = rulesAsMixedList.filter((r) => typeof r !== "string");
  return __spreadProps2(__spreadValues2({}, processedRG), { combinator, rules });
};
var convertToIC = (rg) => {
  if (isRuleGroupTypeIC(rg)) {
    return rg;
  }
  const _a = rg, { combinator } = _a, queryWithoutCombinator = __objRest2(_a, ["combinator"]);
  const rules = [];
  const { length } = rg.rules;
  for (const [idx, r] of rg.rules.entries()) {
    if (isRuleGroup(r)) {
      rules.push(convertToIC(r));
    } else {
      rules.push(r);
    }
    if (combinator && idx < length - 1) {
      rules.push(combinator);
    }
  }
  return __spreadProps2(__spreadValues2({}, queryWithoutCombinator), { rules });
};
function convertQuery(query) {
  return isRuleGroupTypeIC(query) ? convertFromIC(query) : convertToIC(query);
}

// src/utils/defaultValidator.ts
var defaultValidator = (query) => {
  const result = {};
  const validateRule = (_rule) => {
  };
  const validateGroup = (rg) => {
    const reasons = [];
    if (rg.rules.length === 0) {
      reasons.push(groupInvalidReasons.empty);
    } else if (!isRuleGroupType(rg)) {
      let invalidICs = false;
      for (let i = 0; i < rg.rules.length && !invalidICs; i++) {
        if (i % 2 === 0 && typeof rg.rules[i] === "string" || i % 2 === 1 && typeof rg.rules[i] !== "string" || i % 2 === 1 && typeof rg.rules[i] === "string" && !defaultCombinators.map((c2) => c2.name).includes(rg.rules[i])) {
          invalidICs = true;
        }
      }
      if (invalidICs) {
        reasons.push(groupInvalidReasons.invalidIndependentCombinators);
      }
    }
    if (isRuleGroupType(rg) && !defaultCombinators.map((c2) => c2.name).includes(rg.combinator) && rg.rules.length > 1) {
      reasons.push(groupInvalidReasons.invalidCombinator);
    }
    if (rg.id) {
      result[rg.id] = reasons.length > 0 ? { valid: false, reasons } : true;
    }
    for (const r of rg.rules) {
      if (typeof r === "string") {
      } else if (isRuleGroup(r)) {
        validateGroup(r);
      } else {
        validateRule(r);
      }
    }
  };
  validateGroup(query);
  return result;
};

// src/utils/optGroupUtils.ts
import { produce as produce2 } from "immer";
var __defProp3 = Object.defineProperty;
var __defProps3 = Object.defineProperties;
var __getOwnPropDescs3 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols3 = Object.getOwnPropertySymbols;
var __hasOwnProp3 = Object.prototype.hasOwnProperty;
var __propIsEnum3 = Object.prototype.propertyIsEnumerable;
var __defNormalProp3 = (obj, key, value) => key in obj ? __defProp3(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues3 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp3.call(b, prop)) __defNormalProp3(a, prop, b[prop]);
  if (__getOwnPropSymbols3) for (var prop of __getOwnPropSymbols3(b)) {
    if (__propIsEnum3.call(b, prop)) __defNormalProp3(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps3 = (a, b) => __defProps3(a, __getOwnPropDescs3(b));
var isOptionWithName = (opt) => isPojo(opt) && "name" in opt && typeof opt.name === "string";
var isOptionWithValue = (opt) => isPojo(opt) && "value" in opt && typeof opt.value === "string";
function toFullOption(opt, baseProperties, labelMap) {
  const recipe = produce2((draft) => {
    var _a;
    const idObj = {};
    let needsUpdating = !!baseProperties;
    if (typeof draft === "string") {
      return __spreadProps3(__spreadValues3({}, baseProperties), { name: draft, value: draft, label: (_a = labelMap == null ? void 0 : labelMap[draft]) != null ? _a : draft });
    }
    if (isOptionWithName(draft) && !isOptionWithValue(draft)) {
      idObj.value = draft.name;
      needsUpdating = true;
    } else if (!isOptionWithName(draft) && isOptionWithValue(draft)) {
      idObj.name = draft.value;
      needsUpdating = true;
    }
    if (needsUpdating) {
      return Object.assign({}, baseProperties, draft, idObj);
    }
  });
  return recipe(opt);
}
function toFullOptionList(optList, baseProperties, labelMap) {
  if (!Array.isArray(optList)) {
    return [];
  }
  const recipe = produce2((draft) => {
    if (isFlexibleOptionGroupArray(draft)) {
      for (const optGroup of draft) {
        for (const [idx, opt] of optGroup.options.entries()) optGroup.options[idx] = toFullOption(opt, baseProperties, labelMap);
      }
    } else {
      for (const [idx, opt] of draft.entries()) draft[idx] = toFullOption(opt, baseProperties, labelMap);
    }
  });
  return recipe(optList);
}
function toFullOptionMap(optMap, baseProperties) {
  return Object.fromEntries(Object.entries(optMap).map((_ref) => {
    let [k, v] = _ref;
    return [k, toFullOption(v, baseProperties)];
  }));
}
var uniqByName = (originalArray) => uniqByIdentifier(originalArray);
var uniqByIdentifier = (originalArray) => {
  var _a, _b;
  const names = /* @__PURE__ */ new Set();
  const newArray = [];
  for (const el of originalArray) {
    if (!names.has((_a = el.value) != null ? _a : el.name)) {
      names.add((_b = el.value) != null ? _b : el.name);
      newArray.push(el);
    }
  }
  return originalArray.length === newArray.length ? originalArray : newArray;
};
var isOptionGroupArray = (arr) => Array.isArray(arr) && arr.length > 0 && isPojo(arr[0]) && "options" in arr[0] && Array.isArray(arr[0].options);
var isFlexibleOptionArray = (arr) => {
  let isFOA = false;
  if (Array.isArray(arr)) {
    for (const o of arr) {
      if (isOptionWithName(o) || isOptionWithValue(o)) {
        isFOA = true;
      } else {
        return false;
      }
    }
  }
  return isFOA;
};
var isFullOptionArray = (arr) => {
  let isFOA = false;
  if (Array.isArray(arr)) {
    for (const o of arr) {
      if (isOptionWithName(o) && isOptionWithValue(o)) {
        isFOA = true;
      } else {
        return false;
      }
    }
  }
  return isFOA;
};
var isFlexibleOptionGroupArray = function(arr) {
  let { allowEmpty = false } = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  let isFOGA = false;
  if (Array.isArray(arr)) {
    for (const og of arr) {
      if (isPojo(og) && "options" in og && (isFlexibleOptionArray(og.options) || allowEmpty && Array.isArray(og.options) && og.options.length === 0)) {
        isFOGA = true;
      } else {
        return false;
      }
    }
  }
  return isFOGA;
};
var isFullOptionGroupArray = function(arr) {
  let { allowEmpty = false } = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  let isFOGA = false;
  if (Array.isArray(arr)) {
    for (const og of arr) {
      if (isPojo(og) && "options" in og && (isFullOptionArray(og.options) || allowEmpty && Array.isArray(og.options) && og.options.length === 0)) {
        isFOGA = true;
      } else {
        return false;
      }
    }
  }
  return isFOGA;
};
var getOption = (arr, name) => (isFlexibleOptionGroupArray(arr, { allowEmpty: true }) ? arr.flatMap((og) => og.options) : arr).find((op) => op.value === name || op.name === name);
var getFirstOption = (arr) => {
  var _a, _b;
  if (!Array.isArray(arr) || arr.length === 0) {
    return null;
  } else if (isFlexibleOptionGroupArray(arr, { allowEmpty: true })) {
    for (const og of arr) {
      if (og.options.length > 0) {
        return (_a = og.options[0].value) != null ? _a : og.options[0].name;
      }
    }
    return null;
  }
  return (_b = arr[0].value) != null ? _b : arr[0].name;
};
var toFlatOptionArray = (arr) => uniqByIdentifier(isOptionGroupArray(arr) ? arr.flatMap((og) => og.options) : arr);
var uniqOptGroups = (originalArray) => {
  var _a, _b;
  const labels = /* @__PURE__ */ new Set();
  const names = /* @__PURE__ */ new Set();
  const newArray = [];
  for (const el of originalArray) {
    if (!labels.has(el.label)) {
      labels.add(el.label);
      const optionsForThisGroup = [];
      for (const opt of el.options) {
        if (!names.has((_a = opt.value) != null ? _a : opt.name)) {
          names.add((_b = opt.value) != null ? _b : opt.name);
          optionsForThisGroup.push(toFullOption(opt));
        }
      }
      newArray.push(__spreadProps3(__spreadValues3({}, el), { options: optionsForThisGroup }));
    }
  }
  return newArray;
};
var uniqOptList = (originalArray) => {
  if (isFlexibleOptionGroupArray(originalArray)) {
    return uniqOptGroups(originalArray);
  }
  return uniqByIdentifier(originalArray.map((o) => toFullOption(o)));
};

// src/utils/filterFieldsByComparator.ts
var __defProp4 = Object.defineProperty;
var __defProps4 = Object.defineProperties;
var __getOwnPropDescs4 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols4 = Object.getOwnPropertySymbols;
var __hasOwnProp4 = Object.prototype.hasOwnProperty;
var __propIsEnum4 = Object.prototype.propertyIsEnumerable;
var __defNormalProp4 = (obj, key, value) => key in obj ? __defProp4(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues4 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp4.call(b, prop)) __defNormalProp4(a, prop, b[prop]);
  if (__getOwnPropSymbols4) for (var prop of __getOwnPropSymbols4(b)) {
    if (__propIsEnum4.call(b, prop)) __defNormalProp4(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps4 = (a, b) => __defProps4(a, __getOwnPropDescs4(b));
var filterByComparator = (field, operator, fieldToCompare) => {
  var _a, _b;
  const fullField = toFullOption(field);
  const fullFieldToCompare = toFullOption(fieldToCompare);
  if (fullField.value === fullFieldToCompare.value) {
    return false;
  }
  if (typeof fullField.comparator === "string") {
    return fullField[fullField.comparator] === fullFieldToCompare[fullField.comparator];
  }
  return (_b = (_a = fullField.comparator) == null ? void 0 : _a.call(fullField, fullFieldToCompare, operator)) != null ? _b : (
    /* istanbul ignore next */
    false
  );
};
var filterFieldsByComparator = (field, fields, operator) => {
  if (!field.comparator) {
    const filterOutSameField = (f) => {
      var _a, _b;
      return ((_a = f.value) != null ? _a : (
        /* istanbul ignore next */
        f.name
      )) !== ((_b = field.value) != null ? _b : (
        /* istanbul ignore next */
        field.name
      ));
    };
    if (isFlexibleOptionGroupArray(fields)) {
      return fields.map((og) => __spreadProps4(__spreadValues4({}, og), { options: og.options.filter((v) => filterOutSameField(v)) }));
    }
    return fields.filter((v) => filterOutSameField(v));
  }
  if (isFlexibleOptionGroupArray(fields)) {
    return fields.map((og) => __spreadProps4(__spreadValues4({}, og), { options: og.options.filter((f) => filterByComparator(field, operator, f)) })).filter((og) => og.options.length > 0);
  }
  return fields.filter((f) => filterByComparator(field, operator, f));
};

// src/utils/parseNumber.ts
import { numericQuantity } from "numeric-quantity";
var parseNumber = function(val) {
  let { parseNumbers, bigIntOnOverflow } = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  if (!parseNumbers || typeof val === "bigint" || typeof val === "number") {
    return val;
  }
  if (parseNumbers === "native") {
    return Number.parseFloat(val);
  }
  const valAsNum = (
    // TODO: Should these options be configurable?
    numericQuantity(val, { allowTrailingInvalid: parseNumbers === "enhanced", bigIntOnOverflow, romanNumerals: false, round: false })
  );
  return typeof valAsNum === "bigint" || !Number.isNaN(valAsNum) ? valAsNum : val;
};

// src/utils/transformQuery.ts
import { produce as produce3 } from "immer";
var __defProp5 = Object.defineProperty;
var __getOwnPropSymbols5 = Object.getOwnPropertySymbols;
var __hasOwnProp5 = Object.prototype.hasOwnProperty;
var __propIsEnum5 = Object.prototype.propertyIsEnumerable;
var __defNormalProp5 = (obj, key, value) => key in obj ? __defProp5(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues5 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp5.call(b, prop)) __defNormalProp5(a, prop, b[prop]);
  if (__getOwnPropSymbols5) for (var prop of __getOwnPropSymbols5(b)) {
    if (__propIsEnum5.call(b, prop)) __defNormalProp5(a, prop, b[prop]);
  }
  return a;
};
var remapProperties = (obj, propertyMap, deleteRemappedProperties) => produce3(obj, (draft) => {
  for (const [k, v] of Object.entries(propertyMap)) {
    if (v === false) {
      delete draft[k];
    } else if (!!v && k !== v && k in draft) {
      draft[v] = draft[k];
      if (deleteRemappedProperties) {
        delete draft[k];
      }
    }
  }
});
function transformQuery(query) {
  let options = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  const { ruleProcessor = (r) => r, ruleGroupProcessor = (rg) => rg, propertyMap = {}, combinatorMap = {}, operatorMap = {}, omitPath = false, deleteRemappedProperties = true } = options;
  const processGroup = (rg) => {
    var _a, _b;
    return __spreadValues5(__spreadValues5({}, ruleGroupProcessor(remapProperties(__spreadValues5(__spreadValues5({}, rg), isRuleGroupType(rg) ? { combinator: (_a = combinatorMap[rg.combinator]) != null ? _a : rg.combinator } : {}), propertyMap, deleteRemappedProperties))), propertyMap["rules"] === false ? null : {
      // oxlint-disable-next-line typescript/no-explicit-any
      [(_b = propertyMap["rules"]) != null ? _b : "rules"]: rg.rules.map((r, idx) => {
        var _a2, _b2;
        const pathObject = omitPath ? null : { path: [...rg.path, idx] };
        if (typeof r === "string") {
          return (_a2 = combinatorMap[r]) != null ? _a2 : r;
        } else if (isRuleGroup(r)) {
          return processGroup(__spreadValues5(__spreadValues5({}, r), pathObject));
        }
        return ruleProcessor(remapProperties(__spreadValues5(__spreadValues5(__spreadValues5({}, r), pathObject), "operator" in r ? { operator: (_b2 = operatorMap[r.operator]) != null ? _b2 : r.operator } : {}), propertyMap, deleteRemappedProperties));
      })
    });
  };
  return processGroup(__spreadValues5(__spreadValues5({}, query), omitPath ? null : { path: [] }));
}

// src/utils/isRuleOrGroupValid.ts
var isValidationResult = (vr) => isPojo(vr) && typeof vr.valid === "boolean";
var isRuleOrGroupValid = (rg, validationResult, validator) => {
  if (typeof validationResult === "boolean") {
    return validationResult;
  }
  if (isValidationResult(validationResult)) {
    return validationResult.valid;
  }
  if (typeof validator === "function" && !isRuleGroup(rg)) {
    const vr = validator(rg);
    if (typeof vr === "boolean") {
      return vr;
    }
    if (isValidationResult(vr)) {
      return vr.valid;
    }
  }
  return true;
};

// src/utils/getParseNumberMethod.ts
var getParseNumberMethod = (_ref) => {
  let { parseNumbers, inputType } = _ref;
  if (typeof parseNumbers === "string") {
    const [method, level] = parseNumbers.split("-");
    if (level === "limited") {
      return inputType === "number" ? method : false;
    }
    return method;
  }
  return parseNumbers ? "strict" : false;
};

// src/utils/formatQuery/utils.ts
var __defProp6 = Object.defineProperty;
var __defProps5 = Object.defineProperties;
var __getOwnPropDescs5 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols6 = Object.getOwnPropertySymbols;
var __hasOwnProp6 = Object.prototype.hasOwnProperty;
var __propIsEnum6 = Object.prototype.propertyIsEnumerable;
var __defNormalProp6 = (obj, key, value) => key in obj ? __defProp6(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues6 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp6.call(b, prop)) __defNormalProp6(a, prop, b[prop]);
  if (__getOwnPropSymbols6) for (var prop of __getOwnPropSymbols6(b)) {
    if (__propIsEnum6.call(b, prop)) __defNormalProp6(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps5 = (a, b) => __defProps5(a, __getOwnPropDescs5(b));
var mapSQLOperator = (rqbOperator) => {
  switch (lc(rqbOperator)) {
    case "null":
      return "is null";
    case "notnull":
      return "is not null";
    case "notin":
      return "not in";
    case "notbetween":
      return "not between";
    case "contains":
    case "beginswith":
    case "endswith":
      return "like";
    case "doesnotcontain":
    case "doesnotbeginwith":
    case "doesnotendwith":
      return "not like";
    default:
      return rqbOperator;
  }
};
var mongoOperators = {
  "=": "$eq",
  "!=": "$ne",
  "<": "$lt",
  "<=": "$lte",
  ">": "$gt",
  ">=": "$gte",
  in: "$in",
  notin: "$nin",
  notIn: "$nin"
  // only here for backwards compatibility
};
var prismaOperators = { "=": "equals", "!=": "not", "<": "lt", "<=": "lte", ">": "gt", ">=": "gte", in: "in", notin: "notIn" };
var celCombinatorMap = { and: "&&", or: "||" };
var jsonLogicAdditionalOperators = { startsWith: (a, b) => typeof a === "string" && a.startsWith(b), endsWith: (a, b) => typeof a === "string" && a.endsWith(b) };
var numerifyValues = (rg, options) => __spreadProps5(__spreadValues6({}, rg), {
  // @ts-expect-error TS doesn't keep track of odd/even indexes here
  rules: rg.rules.map((r) => {
    if (typeof r === "string") {
      return r;
    }
    if (isRuleGroup(r)) {
      return numerifyValues(r, options);
    }
    const fieldData = getOption(options.fields, r.field);
    const parseNumbers = getParseNumberMethod({ parseNumbers: options.parseNumbers, inputType: fieldData == null ? void 0 : fieldData.inputType });
    if (Array.isArray(r.value)) {
      return __spreadProps5(__spreadValues6({}, r), { value: r.value.map((v) => parseNumber(v, { parseNumbers })) });
    }
    const valAsArray = toArray(r.value, { retainEmptyStrings: true }).map((v) => parseNumber(v, { parseNumbers }));
    if (valAsArray.every((v) => typeof v === "number")) {
      if (valAsArray.length > 1) {
        return __spreadProps5(__spreadValues6({}, r), { value: valAsArray });
      } else if (valAsArray.length === 1) {
        return __spreadProps5(__spreadValues6({}, r), { value: valAsArray[0] });
      }
    }
    return r;
  })
});
var isValidValue = (value) => typeof value === "string" && value.length > 0 || typeof value === "number" && !Number.isNaN(value) || typeof value !== "string" && typeof value !== "number";
var shouldRenderAsNumber = (value, parseNumbers) => !!parseNumbers && (typeof value === "number" || typeof value === "bigint" || typeof value === "string" && numericRegex.test(value));
var isValueProcessorLegacy = (valueProcessor) => valueProcessor.length >= 3;
var getQuoteFieldNamesWithArray = function() {
  let quoteFieldNamesWith = arguments.length > 0 && arguments[0] !== void 0 ? arguments[0] : ["", ""];
  return Array.isArray(quoteFieldNamesWith) ? quoteFieldNamesWith : typeof quoteFieldNamesWith === "string" ? [quoteFieldNamesWith, quoteFieldNamesWith] : quoteFieldNamesWith != null ? quoteFieldNamesWith : ["", ""];
};
var getQuotedFieldName = (fieldName, _ref) => {
  let { quoteFieldNamesWith, fieldIdentifierSeparator } = _ref;
  const [qPre, qPost] = getQuoteFieldNamesWithArray(quoteFieldNamesWith);
  return typeof fieldIdentifierSeparator === "string" && fieldIdentifierSeparator.length > 0 ? joinWith(splitBy(fieldName, fieldIdentifierSeparator).map((part) => `${qPre}${part}${qPost}`), fieldIdentifierSeparator) : `${qPre}${fieldName}${qPost}`;
};
var defaultWordOrder = ["S", "V", "O"];
var normalizeConstituentWordOrder = (input) => {
  const result = [];
  const letterSet = new Set(defaultWordOrder);
  for (const char of input.toUpperCase()) {
    if (letterSet.has(char)) {
      result.push(char);
      letterSet.delete(char);
      if (letterSet.size === 0) break;
    }
  }
  for (const letter of defaultWordOrder) {
    if (letterSet.has(letter)) {
      result.push(letter);
    }
  }
  return result;
};
var defaultNLTranslations = {
  // and: 'and',
  // or: 'or',
  // true: 'true',
  // false: 'false',
  groupPrefix: "",
  // groupPrefix_not: '',
  groupPrefix_not_xor: "either zero or more than one of",
  groupPrefix_xor: "exactly one of",
  groupSuffix: "is true",
  groupSuffix_not: "is not true"
  // groupSuffix_not_xor: 'is true',
  // groupSuffix_xor: 'is true',
};
var translationMatchFilter = (key, keyToTest, conditions) => (
  // The translation matches the base key
  keyToTest.startsWith(key) && // The translation specifies all conditions
  conditions.every((c2) => {
    var _a;
    return (
      // This translation specifies _this_ condition
      keyToTest.includes(`_${c2}`) && // This translation specifies the same _total number_ of conditions
      ((_a = keyToTest.match(/_/g)) == null ? void 0 : _a.length) === conditions.length
    );
  })
);
var getNLTranslataion = function(key, translations) {
  let conditions = arguments.length > 2 && arguments[2] !== void 0 ? arguments[2] : [];
  var _a, _b, _c, _d, _e, _f, _g;
  return conditions.length === 0 ? (_b = (_a = translations[key]) != null ? _a : defaultNLTranslations[key]) != null ? _b : (
    /* istanbul ignore next */
    ""
  ) : (_g = (_f = (_e = (_c = Object.entries(translations).find((_ref2) => {
    let [keyToTest] = _ref2;
    return translationMatchFilter(key, keyToTest, conditions);
  })) == null ? void 0 : _c[1]) != null ? _e : (_d = Object.entries(defaultNLTranslations).find((_ref3) => {
    let [keyToTest] = _ref3;
    return translationMatchFilter(key, keyToTest, conditions);
  })) == null ? void 0 : _d[1]) != null ? _f : defaultNLTranslations[key]) != null ? _g : (
    /* istanbul ignore next */
    ""
  );
};
var processMatchMode = (rule) => {
  var _a;
  const { mode, threshold } = (_a = rule.match) != null ? _a : {};
  if (mode) {
    if (!isRuleGroup(rule.value)) return false;
    const matchModeLC = lc(mode);
    const matchModeCoerced = matchModeLC === "atleast" && threshold === 1 ? "some" : matchModeLC === "atmost" && threshold === 0 ? "none" : matchModeLC;
    if ((matchModeCoerced === "atleast" || matchModeCoerced === "atmost" || matchModeCoerced === "exactly") && (typeof threshold !== "number" || threshold < 0)) {
      return false;
    }
    return { mode: matchModeCoerced, threshold };
  }
};
var bigIntJsonStringifyReplacer = (_key, value) => typeof value === "bigint" ? { $bigint: value.toString() } : value;
var bigIntJsonParseReviver = (_key, value) => isPojo(value) && Object.keys(value).length === 1 && typeof value.$bigint === "string" ? BigInt(value.$bigint) : value;

// src/utils/formatQuery/defaultRuleGroupProcessorCEL.ts
var __defProp7 = Object.defineProperty;
var __defProps6 = Object.defineProperties;
var __getOwnPropDescs6 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols7 = Object.getOwnPropertySymbols;
var __hasOwnProp7 = Object.prototype.hasOwnProperty;
var __propIsEnum7 = Object.prototype.propertyIsEnumerable;
var __defNormalProp7 = (obj, key, value) => key in obj ? __defProp7(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues7 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp7.call(b, prop)) __defNormalProp7(a, prop, b[prop]);
  if (__getOwnPropSymbols7) for (var prop of __getOwnPropSymbols7(b)) {
    if (__propIsEnum7.call(b, prop)) __defNormalProp7(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps6 = (a, b) => __defProps6(a, __getOwnPropDescs6(b));
var defaultRuleGroupProcessorCEL = (ruleGroup, options) => {
  const { fields, fallbackExpression, getParseNumberBoolean, placeholderFieldName, placeholderOperatorName, placeholderValueName, ruleProcessor, validateRule, validationMap } = options;
  const processRuleGroup = (rg, outermost) => {
    var _a;
    if (!isRuleOrGroupValid(rg, validationMap[(_a = rg.id) != null ? _a : (
      /* istanbul ignore next */
      ""
    )])) {
      return outermost ? fallbackExpression : "";
    }
    const expression = rg.rules.map((rule) => {
      var _a2;
      if (typeof rule === "string") {
        return celCombinatorMap[rule];
      }
      if (isRuleGroup(rule)) {
        return processRuleGroup(rule);
      }
      const [validationResult, fieldValidator] = validateRule(rule);
      if (!isRuleOrGroupValid(rule, validationResult, fieldValidator) || rule.field === placeholderFieldName || rule.operator === placeholderOperatorName || /* istanbul ignore next */
      placeholderValueName !== void 0 && rule.value === placeholderValueName) {
        return "";
      }
      const fieldData = getOption(fields, rule.field);
      return ruleProcessor(rule, __spreadProps6(__spreadValues7({}, options), { parseNumbers: getParseNumberBoolean(fieldData == null ? void 0 : fieldData.inputType), escapeQuotes: ((_a2 = rule.valueSource) != null ? _a2 : "value") === "value", fieldData }));
    }).filter(Boolean).join(isRuleGroupType(rg) ? ` ${celCombinatorMap[rg.combinator]} ` : " ");
    const [prefix, suffix] = rg.not || !outermost ? [`${rg.not ? "!" : ""}(`, ")"] : ["", ""];
    return expression ? `${prefix}${expression}${suffix}` : fallbackExpression;
  };
  return processRuleGroup(ruleGroup, true);
};

// src/utils/formatQuery/defaultRuleProcessorCEL.ts
var __defProp8 = Object.defineProperty;
var __defProps7 = Object.defineProperties;
var __getOwnPropDescs7 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols8 = Object.getOwnPropertySymbols;
var __hasOwnProp8 = Object.prototype.hasOwnProperty;
var __propIsEnum8 = Object.prototype.propertyIsEnumerable;
var __defNormalProp8 = (obj, key, value) => key in obj ? __defProp8(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues8 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp8.call(b, prop)) __defNormalProp8(a, prop, b[prop]);
  if (__getOwnPropSymbols8) for (var prop of __getOwnPropSymbols8(b)) {
    if (__propIsEnum8.call(b, prop)) __defNormalProp8(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps7 = (a, b) => __defProps7(a, __getOwnPropDescs7(b));
var shouldNegate = (op) => op.startsWith("not") || op.startsWith("doesnot");
var escapeDoubleQuotes = (v, escapeQuotes) => typeof v !== "string" || !escapeQuotes ? v : v.replaceAll(`"`, `\\"`);
var defaultRuleProcessorCEL = function(rule) {
  let opts = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  const { escapeQuotes, parseNumbers, preserveValueOrder } = opts;
  const { field, operator, value, valueSource } = rule;
  const valueIsField = valueSource === "field";
  const operatorTL = lc(operator === "=" ? "==" : operator);
  const useBareValue = typeof value === "number" || typeof value === "boolean" || typeof value === "bigint" || shouldRenderAsNumber(value, parseNumbers);
  const matchEval = processMatchMode(rule);
  if (matchEval === false) {
    return "";
  } else if (matchEval) {
    const { mode, threshold } = matchEval;
    const arrayElementAlias = "elem_alias";
    const celQuery = transformQuery(rule.value, { ruleProcessor: (r) => __spreadProps7(__spreadValues8({}, r), { field: `${arrayElementAlias}${r.field ? `.${r.field}` : ""}` }) });
    const nestedArrayFilter = defaultRuleGroupProcessorCEL(celQuery, opts);
    switch (mode) {
      case "all":
        return `${field}.all(${arrayElementAlias}, ${nestedArrayFilter})`;
      case "none":
      case "some":
        return `${mode === "none" ? "!" : ""}${field}.exists(${arrayElementAlias}, ${nestedArrayFilter})`;
      case "atleast":
      case "atmost":
      case "exactly": {
        const totalCount = `double(${field}.size())`;
        const filteredCount = `${field}.filter(${arrayElementAlias}, ${nestedArrayFilter}).size()`;
        const op = mode === "atleast" ? ">=" : mode === "atmost" ? "<=" : "==";
        if (threshold > 0 && threshold < 1) {
          return `${filteredCount} ${op} (${totalCount} * ${threshold})`;
        }
        return `${filteredCount} ${op} ${threshold}`;
      }
    }
  }
  switch (operatorTL) {
    case "<":
    case "<=":
    case "==":
    case "!=":
    case ">":
    case ">=":
      return `${field} ${operatorTL} ${valueIsField || useBareValue ? trimIfString(value) : `"${escapeDoubleQuotes(value, escapeQuotes)}"`}`;
    case "contains":
    case "doesnotcontain": {
      const negate2 = shouldNegate(operatorTL) ? "!" : "";
      return `${negate2}${field}.contains(${valueIsField ? trimIfString(value) : `"${escapeDoubleQuotes(value, escapeQuotes)}"`})`;
    }
    case "beginswith":
    case "doesnotbeginwith": {
      const negate2 = shouldNegate(operatorTL) ? "!" : "";
      return `${negate2}${field}.startsWith(${valueIsField ? trimIfString(value) : `"${escapeDoubleQuotes(value, escapeQuotes)}"`})`;
    }
    case "endswith":
    case "doesnotendwith": {
      const negate2 = shouldNegate(operatorTL) ? "!" : "";
      return `${negate2}${field}.endsWith(${valueIsField ? trimIfString(value) : `"${escapeDoubleQuotes(value, escapeQuotes)}"`})`;
    }
    case "null":
      return `${field} == null`;
    case "notnull":
      return `${field} != null`;
    case "in":
    case "notin": {
      const [prefix, suffix] = shouldNegate(operatorTL) ? ["!(", ")"] : ["", ""];
      const valueAsArray = toArray(value);
      return `${prefix}${field} in [${valueAsArray.map((val) => valueIsField || shouldRenderAsNumber(val, parseNumbers) ? `${trimIfString(val)}` : `"${escapeDoubleQuotes(val, escapeQuotes)}"`).join(", ")}]${suffix}`;
    }
    case "between":
    case "notbetween": {
      const valueAsArray = toArray(value);
      if (valueAsArray.length >= 2 && !nullOrUndefinedOrEmpty(valueAsArray[0]) && !nullOrUndefinedOrEmpty(valueAsArray[1])) {
        const [first, second] = valueAsArray;
        const shouldParseNumbers = !(parseNumbers === false);
        const firstNum = shouldRenderAsNumber(first, shouldParseNumbers) ? parseNumber(first, { parseNumbers: shouldParseNumbers }) : Number.NaN;
        const secondNum = shouldRenderAsNumber(second, shouldParseNumbers) ? parseNumber(second, { parseNumbers: shouldParseNumbers }) : Number.NaN;
        let firstValue = Number.isNaN(firstNum) ? valueIsField ? `${first}` : `"${escapeDoubleQuotes(first, escapeQuotes)}"` : firstNum;
        let secondValue = Number.isNaN(secondNum) ? valueIsField ? `${second}` : `"${escapeDoubleQuotes(second, escapeQuotes)}"` : secondNum;
        if (!preserveValueOrder && firstValue === firstNum && secondValue === secondNum && secondNum < firstNum) {
          const tempNum = secondNum;
          secondValue = firstNum;
          firstValue = tempNum;
        }
        return operatorTL === "between" ? `(${field} >= ${firstValue} && ${field} <= ${secondValue})` : `(${field} < ${firstValue} || ${field} > ${secondValue})`;
      } else {
        return "";
      }
    }
  }
  return "";
};

// src/utils/formatQuery/defaultRuleGroupProcessorMongoDBQuery.ts
var __defProp9 = Object.defineProperty;
var __defProps8 = Object.defineProperties;
var __getOwnPropDescs8 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols9 = Object.getOwnPropertySymbols;
var __hasOwnProp9 = Object.prototype.hasOwnProperty;
var __propIsEnum9 = Object.prototype.propertyIsEnumerable;
var __defNormalProp9 = (obj, key, value) => key in obj ? __defProp9(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues9 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp9.call(b, prop)) __defNormalProp9(a, prop, b[prop]);
  if (__getOwnPropSymbols9) for (var prop of __getOwnPropSymbols9(b)) {
    if (__propIsEnum9.call(b, prop)) __defNormalProp9(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps8 = (a, b) => __defProps8(a, __getOwnPropDescs8(b));
var mongoDbFallback = { $and: [{ $expr: true }] };
var defaultRuleGroupProcessorMongoDBQuery = (ruleGroup, options, meta) => {
  const { fields, getParseNumberBoolean, placeholderFieldName, placeholderOperatorName, placeholderValueName, ruleProcessor, validateRule, validationMap } = options;
  const processRuleGroup = (rg, outermost) => {
    var _a;
    if (!isRuleOrGroupValid(rg, validationMap[(_a = rg.id) != null ? _a : (
      /* istanbul ignore next */
      ""
    )])) {
      return outermost ? mongoDbFallback : false;
    }
    const combinator = `$${lc(rg.combinator)}`;
    let hasChildRules = false;
    const expressions = rg.rules.map((rule) => {
      if (isRuleGroup(rule)) {
        const processedRuleGroup = processRuleGroup(rule);
        if (processedRuleGroup) {
          hasChildRules = true;
          return processedRuleGroup;
        }
        return false;
      }
      const [validationResult, fieldValidator] = validateRule(rule);
      if (!isRuleOrGroupValid(rule, validationResult, fieldValidator) || rule.field === placeholderFieldName || rule.operator === placeholderOperatorName || /* istanbul ignore next */
      placeholderValueName !== void 0 && rule.value === placeholderValueName) {
        return false;
      }
      const fieldData = getOption(fields, rule.field);
      return ruleProcessor(rule, __spreadProps8(__spreadValues9({}, options), { parseNumbers: getParseNumberBoolean(fieldData == null ? void 0 : fieldData.inputType), fieldData }), meta);
    }).filter(Boolean);
    return expressions.length > 0 ? expressions.length === 1 && !hasChildRules ? expressions[0] : { [combinator]: expressions } : mongoDbFallback;
  };
  return processRuleGroup(convertFromIC(ruleGroup), true);
};

// src/utils/formatQuery/defaultRuleProcessorMongoDBQuery.ts
var __defProp10 = Object.defineProperty;
var __defProps9 = Object.defineProperties;
var __getOwnPropDescs9 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols10 = Object.getOwnPropertySymbols;
var __hasOwnProp10 = Object.prototype.hasOwnProperty;
var __propIsEnum10 = Object.prototype.propertyIsEnumerable;
var __defNormalProp10 = (obj, key, value) => key in obj ? __defProp10(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues10 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp10.call(b, prop)) __defNormalProp10(a, prop, b[prop]);
  if (__getOwnPropSymbols10) for (var prop of __getOwnPropSymbols10(b)) {
    if (__propIsEnum10.call(b, prop)) __defNormalProp10(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps9 = (a, b) => __defProps9(a, __getOwnPropDescs9(b));
var processNumber = function(value, fallback) {
  let parseNumbers = arguments.length > 2 && arguments[2] !== void 0 ? arguments[2] : false;
  return shouldRenderAsNumber(value, parseNumbers || typeof value === "bigint") ? Number(parseNumber(value, { parseNumbers: "strict" })) : fallback;
};
var defaultRuleProcessorMongoDBQuery = function(rule) {
  let options = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  const { field, operator, value, valueSource } = rule;
  const { parseNumbers, preserveValueOrder, context } = options;
  const valueIsField = valueSource === "field";
  const { avoidFieldsAsKeys } = context != null ? context : {};
  const matchEval = processMatchMode(rule);
  if (matchEval === false) {
    return;
  } else if (matchEval) {
    const { mode, threshold } = matchEval;
    const totalCount = { $size: { $ifNull: [`$${field}`, []] } };
    const subQueryNoAggCtx = defaultRuleGroupProcessorMongoDBQuery(transformQuery(value, { ruleProcessor: (r) => __spreadProps9(__spreadValues10({}, r), { field: r.field ? `${field}.${r.field}` : field }) }), __spreadProps9(__spreadValues10({}, options), {
      // We have to override `ruleProcessor` in case original `format` is "mongodb"
      ruleProcessor: defaultRuleProcessorMongoDBQuery,
      context: __spreadProps9(__spreadValues10({}, options.context), { avoidFieldsAsKeys: false })
    }));
    const subQueryWithAggCtx = defaultRuleGroupProcessorMongoDBQuery(transformQuery(value, { ruleProcessor: (r) => __spreadProps9(__spreadValues10({}, r), { field: r.field ? `$item.${r.field}` : "$item" }) }), __spreadProps9(__spreadValues10({}, options), {
      // We have to override `ruleProcessor` in case original `format` is "mongodb"
      ruleProcessor: defaultRuleProcessorMongoDBQuery,
      context: __spreadProps9(__spreadValues10({}, options.context), { avoidFieldsAsKeys: true })
    }));
    const filteredCount = { $size: { $ifNull: [{ $filter: { input: `$${field}`, as: "item", cond: { $and: [subQueryWithAggCtx] } } }, []] } };
    switch (mode) {
      case "all":
        return { $expr: { $eq: [filteredCount, totalCount] } };
      case "none":
        return { $nor: [subQueryNoAggCtx] };
      case "some":
        return subQueryNoAggCtx;
      case "atleast":
      case "atmost":
      case "exactly": {
        const op = mode === "atleast" ? mongoOperators[">="] : mode === "atmost" ? mongoOperators["<="] : mongoOperators["="];
        if (threshold > 0 && threshold < 1) {
          return { $expr: { [op]: [filteredCount, { $multiply: [totalCount, threshold] }] } };
        }
        return { $expr: { [op]: [filteredCount, threshold] } };
      }
    }
  }
  if (operator === "=" && !valueIsField) {
    return avoidFieldsAsKeys ? { $eq: [`$${field}`, processNumber(value, value, parseNumbers)] } : { [field]: processNumber(value, value, parseNumbers) };
  }
  const operatorLC = lc(operator);
  switch (operatorLC) {
    case "<":
    case "<=":
    case "=":
    case "!=":
    case ">":
    case ">=": {
      const mongoOperator = mongoOperators[operatorLC];
      return valueIsField ? { [mongoOperator]: [`$${field}`, `$${value}`] } : avoidFieldsAsKeys ? { $and: [{ $ne: [`$${field}`, null] }, { [mongoOperator]: [`$${field}`, processNumber(value, value, parseNumbers)] }] } : { [field]: { [mongoOperator]: processNumber(value, value, parseNumbers) } };
    }
    case "contains":
      return valueIsField ? { $where: `this.${field}.includes(this.${value})` } : avoidFieldsAsKeys ? { $regexMatch: { input: `$${field}`, regex: value } } : { [field]: { $regex: value } };
    case "beginswith":
      return valueIsField ? { $where: `this.${field}.startsWith(this.${value})` } : avoidFieldsAsKeys ? { $regexMatch: { input: `$${field}`, regex: `^${value}` } } : { [field]: { $regex: `^${value}` } };
    case "endswith":
      return valueIsField ? { $where: `this.${field}.endsWith(this.${value})` } : avoidFieldsAsKeys ? { $regexMatch: { input: `$${field}`, regex: `${value}$` } } : { [field]: { $regex: `${value}$` } };
    case "doesnotcontain":
      return valueIsField ? { $where: `!this.${field}.includes(this.${value})` } : avoidFieldsAsKeys ? { $not: { $regexMatch: { input: `$${field}`, regex: value } } } : { [field]: { $not: { $regex: value } } };
    case "doesnotbeginwith":
      return valueIsField ? { $where: `!this.${field}.startsWith(this.${value})` } : avoidFieldsAsKeys ? { $not: { $regexMatch: { input: `$${field}`, regex: `^${value}` } } } : { [field]: { $not: { $regex: `^${value}` } } };
    case "doesnotendwith":
      return valueIsField ? { $where: `!this.${field}.endsWith(this.${value})` } : avoidFieldsAsKeys ? { $not: { $regexMatch: { input: `$${field}`, regex: `${value}$` } } } : { [field]: { $not: { $regex: `${value}$` } } };
    case "null":
      return avoidFieldsAsKeys ? { $eq: [`$${field}`, null] } : { [field]: null };
    case "notnull":
      return avoidFieldsAsKeys ? { $ne: [`$${field}`, null] } : { [field]: { $ne: null } };
    case "in":
    case "notin": {
      const valueAsArray = toArray(value);
      return valueIsField ? { $where: `${operatorLC === "notin" ? "!" : ""}[${valueAsArray.map((val) => `this.${val}`).join(",")}].includes(this.${field})` } : avoidFieldsAsKeys ? operatorLC === "notin" ? { $not: { [mongoOperators.in]: [`$${field}`, valueAsArray.map((val) => processNumber(val, val, parseNumbers))] } } : { [mongoOperators[operatorLC]]: [`$${field}`, valueAsArray.map((val) => processNumber(val, val, parseNumbers))] } : { [field]: { [mongoOperators[operatorLC]]: valueAsArray.map((val) => processNumber(val, val, parseNumbers)) } };
    }
    case "between":
    case "notbetween": {
      const valueAsArray = toArray(value);
      if (valueAsArray.length >= 2 && isValidValue(valueAsArray[0]) && isValidValue(valueAsArray[1])) {
        const [first, second] = valueAsArray;
        const firstNum = processNumber(first, Number.NaN, true);
        const secondNum = processNumber(second, Number.NaN, true);
        let firstValue = valueIsField ? first : Number.isNaN(firstNum) ? first : firstNum;
        let secondValue = valueIsField ? second : Number.isNaN(secondNum) ? second : secondNum;
        if (!preserveValueOrder && firstValue === firstNum && secondValue === secondNum && secondNum < firstNum) {
          const tempNum = secondNum;
          secondValue = firstNum;
          firstValue = tempNum;
        }
        if (operatorLC === "between") {
          return valueIsField ? { $gte: [`$${field}`, `$${firstValue}`], $lte: [`$${field}`, `$${secondValue}`] } : avoidFieldsAsKeys ? { $and: [{ $gte: [`$${field}`, firstValue] }, { $lte: [`$${field}`, secondValue] }] } : { [field]: { $gte: firstValue, $lte: secondValue } };
        } else {
          return valueIsField ? { $or: [{ $lt: [`$${field}`, `$${firstValue}`] }, { $gt: [`$${field}`, `$${secondValue}`] }] } : avoidFieldsAsKeys ? { $or: [{ $lt: [`$${field}`, firstValue] }, { $gt: [`$${field}`, secondValue] }] } : { $or: [{ [field]: { $lt: firstValue } }, { [field]: { $gt: secondValue } }] };
        }
      } else {
        return "";
      }
    }
  }
  return "";
};

// src/utils/formatQuery/defaultRuleProcessorMongoDB.ts
var defaultRuleProcessorMongoDB = (rule, options) => {
  const queryObj = defaultRuleProcessorMongoDBQuery(rule, options);
  return queryObj ? JSON.stringify(queryObj) : "";
};

// src/utils/formatQuery/defaultRuleGroupProcessorSpEL.ts
var __defProp11 = Object.defineProperty;
var __defProps10 = Object.defineProperties;
var __getOwnPropDescs10 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols11 = Object.getOwnPropertySymbols;
var __hasOwnProp11 = Object.prototype.hasOwnProperty;
var __propIsEnum11 = Object.prototype.propertyIsEnumerable;
var __defNormalProp11 = (obj, key, value) => key in obj ? __defProp11(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues11 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp11.call(b, prop)) __defNormalProp11(a, prop, b[prop]);
  if (__getOwnPropSymbols11) for (var prop of __getOwnPropSymbols11(b)) {
    if (__propIsEnum11.call(b, prop)) __defNormalProp11(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps10 = (a, b) => __defProps10(a, __getOwnPropDescs10(b));
var defaultRuleGroupProcessorSpEL = (ruleGroup, options) => {
  const { fields, fallbackExpression, getParseNumberBoolean, placeholderFieldName, placeholderOperatorName, placeholderValueName, ruleProcessor, validateRule, validationMap } = options;
  const processRuleGroup = (rg, outermost) => {
    var _a;
    if (!isRuleOrGroupValid(rg, validationMap[(_a = rg.id) != null ? _a : (
      /* istanbul ignore next */
      ""
    )])) {
      return outermost ? fallbackExpression : "";
    }
    const expression = rg.rules.map((rule) => {
      var _a2;
      if (typeof rule === "string") {
        return rule;
      }
      if (isRuleGroup(rule)) {
        return processRuleGroup(rule);
      }
      const [validationResult, fieldValidator] = validateRule(rule);
      if (!isRuleOrGroupValid(rule, validationResult, fieldValidator) || rule.field === placeholderFieldName || rule.operator === placeholderOperatorName || /* istanbul ignore next */
      placeholderValueName !== void 0 && rule.value === placeholderValueName) {
        return "";
      }
      const fieldData = getOption(fields, rule.field);
      return ruleProcessor(rule, __spreadProps10(__spreadValues11({}, options), { parseNumbers: getParseNumberBoolean(fieldData == null ? void 0 : fieldData.inputType), escapeQuotes: ((_a2 = rule.valueSource) != null ? _a2 : "value") === "value", fieldData }));
    }).filter(Boolean).join(isRuleGroupType(rg) ? ` ${rg.combinator} ` : " ");
    const [prefix, suffix] = rg.not || !outermost ? [`${rg.not ? "!" : ""}(`, ")"] : ["", ""];
    return expression ? `${prefix}${expression}${suffix}` : fallbackExpression;
  };
  return processRuleGroup(ruleGroup, true);
};

// src/utils/formatQuery/defaultRuleProcessorSpEL.ts
var __defProp12 = Object.defineProperty;
var __defProps11 = Object.defineProperties;
var __getOwnPropDescs11 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols12 = Object.getOwnPropertySymbols;
var __hasOwnProp12 = Object.prototype.hasOwnProperty;
var __propIsEnum12 = Object.prototype.propertyIsEnumerable;
var __defNormalProp12 = (obj, key, value) => key in obj ? __defProp12(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues12 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp12.call(b, prop)) __defNormalProp12(a, prop, b[prop]);
  if (__getOwnPropSymbols12) for (var prop of __getOwnPropSymbols12(b)) {
    if (__propIsEnum12.call(b, prop)) __defNormalProp12(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps11 = (a, b) => __defProps11(a, __getOwnPropDescs11(b));
var shouldNegate2 = (op) => op.startsWith("not") || op.startsWith("doesnot");
var wrapInNegation = (clause, negate2) => negate2 ? `!(${clause})` : `${clause}`;
var escapeSingleQuotes = (v, escapeQuotes) => typeof v !== "string" || !escapeQuotes ? v : v.replaceAll(`'`, `\\'`);
var defaultRuleProcessorSpEL = function(rule) {
  let opts = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  const { field, operator, value, valueSource } = rule;
  const { escapeQuotes, parseNumbers, preserveValueOrder } = opts;
  const valueIsField = valueSource === "field";
  const operatorTL = lc(operator === "=" ? "==" : operator);
  const useBareValue = typeof value === "number" || typeof value === "boolean" || typeof value === "bigint" || shouldRenderAsNumber(value, parseNumbers);
  const matchEval = processMatchMode(rule);
  if (matchEval === false) {
    return "";
  } else if (matchEval) {
    const { mode, threshold } = matchEval;
    const nestedArrayFilter = defaultRuleGroupProcessorSpEL(transformQuery(rule.value, { ruleProcessor: (r) => __spreadProps11(__spreadValues12({}, r), { field: `${r.field || "#this"}` }) }), opts);
    const totalCount = `${field}.size()`;
    const filteredCount = `${field}.?[${nestedArrayFilter}].size()`;
    switch (mode) {
      case "all":
        return `${filteredCount} == ${totalCount}`;
      case "none":
        return `${filteredCount} == 0`;
      case "some":
        return `${filteredCount} >= 1`;
      case "atleast":
      case "atmost":
      case "exactly": {
        const op = mode === "atleast" ? ">=" : mode === "atmost" ? "<=" : "==";
        if (threshold > 0 && threshold < 1) {
          return `${filteredCount} ${op} (${totalCount} * ${threshold})`;
        }
        return `${filteredCount} ${op} ${threshold}`;
      }
    }
  }
  switch (operatorTL) {
    case "<":
    case "<=":
    case "==":
    case "!=":
    case ">":
    case ">=":
      return `${field} ${operatorTL} ${valueIsField || useBareValue ? trimIfString(value) : `'${escapeSingleQuotes(value, escapeQuotes)}'`}`;
    case "contains":
    case "doesnotcontain":
      return wrapInNegation(`${field} matches ${valueIsField || useBareValue ? trimIfString(value) : `'${escapeSingleQuotes(value, escapeQuotes)}'`}`, shouldNegate2(operatorTL));
    case "beginswith":
    case "doesnotbeginwith": {
      const valueTL = valueIsField ? `'^'.concat(${trimIfString(value)})` : `'${typeof value === "string" && !value.startsWith("^") || useBareValue ? "^" : ""}${escapeSingleQuotes(value, escapeQuotes)}'`;
      return wrapInNegation(`${field} matches ${valueTL}`, shouldNegate2(operatorTL));
    }
    case "endswith":
    case "doesnotendwith": {
      const valueTL = valueIsField ? `${trimIfString(value)}.concat('$')` : `'${escapeSingleQuotes(value, escapeQuotes)}${typeof value === "string" && !value.endsWith("$") || useBareValue ? "$" : ""}'`;
      return wrapInNegation(`${field} matches ${valueTL}`, shouldNegate2(operatorTL));
    }
    case "null":
      return `${field} == null`;
    case "notnull":
      return `${field} != null`;
    case "in":
    case "notin": {
      const negate2 = shouldNegate2(operatorTL) ? "!" : "";
      const valueAsArray = toArray(value);
      return valueAsArray.length > 0 ? `${negate2}(${valueAsArray.map((val) => `${field} == ${valueIsField || shouldRenderAsNumber(val, parseNumbers) ? `${trimIfString(val)}` : `'${escapeSingleQuotes(val, escapeQuotes)}'`}`).join(" or ")})` : "";
    }
    case "between":
    case "notbetween": {
      const valueAsArray = toArray(value);
      if (valueAsArray.length >= 2 && !nullOrUndefinedOrEmpty(valueAsArray[0]) && !nullOrUndefinedOrEmpty(valueAsArray[1])) {
        const [first, second] = valueAsArray;
        const shouldParseNumbers = !(parseNumbers === false);
        const firstNum = shouldRenderAsNumber(first, shouldParseNumbers) ? parseNumber(first, { parseNumbers: shouldParseNumbers }) : Number.NaN;
        const secondNum = shouldRenderAsNumber(second, shouldParseNumbers) ? parseNumber(second, { parseNumbers: shouldParseNumbers }) : Number.NaN;
        let firstValue = Number.isNaN(firstNum) ? valueIsField ? `${first}` : `'${escapeSingleQuotes(first, escapeQuotes)}'` : firstNum;
        let secondValue = Number.isNaN(secondNum) ? valueIsField ? `${second}` : `'${escapeSingleQuotes(second, escapeQuotes)}'` : secondNum;
        if (!preserveValueOrder && firstValue === firstNum && secondValue === secondNum && secondNum < firstNum) {
          const tempNum = secondNum;
          secondValue = firstNum;
          firstValue = tempNum;
        }
        return operatorTL === "between" ? `(${field} >= ${firstValue} and ${field} <= ${secondValue})` : `(${field} < ${firstValue} or ${field} > ${secondValue})`;
      } else {
        return "";
      }
    }
  }
  return "";
};

// src/utils/formatQuery/defaultValueProcessorByRule.ts
var escapeStringValueQuotes = (v, quoteChar, escapeQuotes) => escapeQuotes && typeof v === "string" ? v.replaceAll(`${quoteChar}`, `${quoteChar}${quoteChar}`) : v;
var defaultValueProcessorByRule = function(_ref) {
  let { operator, value, valueSource } = _ref;
  let { escapeQuotes, parseNumbers, preserveValueOrder, quoteFieldNamesWith, quoteValuesWith, concatOperator = "||", fieldIdentifierSeparator, wrapValueWith = ["", ""], translations } = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  var _a;
  const valueIsField = valueSource === "field";
  const operatorLowerCase = lc(operator);
  const quoteChar = quoteValuesWith || "'";
  const quoteValue = (v) => `${wrapValueWith[0]}${quoteChar}${v}${quoteChar}${wrapValueWith[1]}`;
  const escapeValue = (v) => escapeStringValueQuotes(v, quoteChar, escapeQuotes);
  const wrapAndEscape = (v) => quoteValue(escapeValue(v));
  const wrapFieldName = (v) => getQuotedFieldName(v, { quoteFieldNamesWith, fieldIdentifierSeparator });
  const concat = function() {
    for (var _len = arguments.length, values = new Array(_len), _key = 0; _key < _len; _key++) {
      values[_key] = arguments[_key];
    }
    return concatOperator.toUpperCase() === "CONCAT" ? `CONCAT(${values.join(", ")})` : values.join(` ${concatOperator} `);
  };
  switch (operatorLowerCase) {
    case "null":
    case "notnull": {
      return "";
    }
    case "in":
    case "notin": {
      const valueAsArray = toArray(value);
      if (valueAsArray.length > 0) {
        return `(${valueAsArray.map((v) => valueIsField ? wrapFieldName(v) : shouldRenderAsNumber(v, parseNumbers) ? `${trimIfString(v)}` : `${wrapAndEscape(v)}`).join(", ")})`;
      }
      return "";
    }
    case "between":
    case "notbetween": {
      const valueAsArray = toArray(value, { retainEmptyStrings: true });
      if (valueAsArray.length < 2 || !isValidValue(valueAsArray[0]) || !isValidValue(valueAsArray[1])) {
        return "";
      }
      const [first, second] = valueAsArray;
      const firstNum = shouldRenderAsNumber(first, parseNumbers) ? parseNumber(first, { parseNumbers: "strict" }) : Number.NaN;
      const secondNum = shouldRenderAsNumber(second, parseNumbers) ? parseNumber(second, { parseNumbers: "strict" }) : Number.NaN;
      const firstValue = Number.isNaN(firstNum) ? valueIsField ? `${first}` : first : firstNum;
      const secondValue = Number.isNaN(secondNum) ? valueIsField ? `${second}` : second : secondNum;
      const valsOneAndTwoOnly = [firstValue, secondValue];
      if (!preserveValueOrder && firstValue === firstNum && secondValue === secondNum && secondNum < firstNum) {
        valsOneAndTwoOnly[0] = secondNum;
        valsOneAndTwoOnly[1] = firstNum;
      }
      return (valueIsField ? valsOneAndTwoOnly.map((v) => wrapFieldName(v)) : valsOneAndTwoOnly.every((v) => shouldRenderAsNumber(v, parseNumbers)) ? valsOneAndTwoOnly.map((v) => parseNumber(v, { parseNumbers: "strict" })) : valsOneAndTwoOnly.map((v) => wrapAndEscape(v))).join(` ${(_a = translations == null ? void 0 : translations.and) != null ? _a : "and"} `);
    }
    case "contains":
    case "doesnotcontain":
      return valueIsField ? concat(quoteValue("%"), wrapFieldName(value), quoteValue("%")) : quoteValue(`%${escapeValue(value)}%`);
    case "beginswith":
    case "doesnotbeginwith":
      return valueIsField ? concat(wrapFieldName(value), quoteValue("%")) : quoteValue(`${escapeValue(value)}%`);
    case "endswith":
    case "doesnotendwith":
      return valueIsField ? concat(quoteValue("%"), wrapFieldName(value)) : quoteValue(`%${escapeValue(value)}`);
  }
  if (typeof value === "boolean") {
    return value ? "TRUE" : "FALSE";
  }
  return valueIsField ? wrapFieldName(value) : shouldRenderAsNumber(value, parseNumbers) ? `${trimIfString(value)}` : `${wrapAndEscape(value)}`;
};

// src/utils/formatQuery/defaultRuleGroupProcessorDrizzle.ts
var __defProp13 = Object.defineProperty;
var __defProps12 = Object.defineProperties;
var __getOwnPropDescs12 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols13 = Object.getOwnPropertySymbols;
var __hasOwnProp13 = Object.prototype.hasOwnProperty;
var __propIsEnum13 = Object.prototype.propertyIsEnumerable;
var __defNormalProp13 = (obj, key, value) => key in obj ? __defProp13(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues13 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp13.call(b, prop)) __defNormalProp13(a, prop, b[prop]);
  if (__getOwnPropSymbols13) for (var prop of __getOwnPropSymbols13(b)) {
    if (__propIsEnum13.call(b, prop)) __defNormalProp13(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps12 = (a, b) => __defProps12(a, __getOwnPropDescs12(b));
var defaultRuleGroupProcessorDrizzle = (ruleGroup, options, _meta) => (columns, drizzleOperators) => {
  const { fields, getParseNumberBoolean, placeholderFieldName, placeholderOperatorName, placeholderValueName, validateRule, validationMap } = options;
  if (!columns || !drizzleOperators) return;
  const { and, not, or } = drizzleOperators;
  const query = isRuleGroupType(ruleGroup) ? ruleGroup : convertFromIC(ruleGroup);
  const ruleProcessor = defaultRuleProcessorDrizzle;
  const processRuleGroup = (rg, _outermost) => {
    var _a;
    if (!isRuleOrGroupValid(rg, validationMap[(_a = rg.id) != null ? _a : (
      /* istanbul ignore next */
      ""
    )])) {
      return;
    }
    const processedRules = rg.rules.map((rule) => {
      if (isRuleGroup(rule)) {
        return processRuleGroup(rule);
      }
      const [validationResult, fieldValidator] = validateRule(rule);
      if (!isRuleOrGroupValid(rule, validationResult, fieldValidator) || rule.field === placeholderFieldName || rule.operator === placeholderOperatorName || /* istanbul ignore next */
      placeholderValueName !== void 0 && rule.value === placeholderValueName) {
        return;
      }
      const fieldData = getOption(fields, rule.field);
      return ruleProcessor(rule, __spreadProps12(__spreadValues13({}, options), { parseNumbers: getParseNumberBoolean(fieldData == null ? void 0 : fieldData.inputType), fieldData, context: __spreadProps12(__spreadValues13({}, options.context), { columns, drizzleOperators }) }));
    }).filter(Boolean);
    if (processedRules.length === 0) {
      return;
    }
    const ruleGroupSQL = rg.combinator === "or" ? or(...processedRules) : and(...processedRules);
    return rg.not ? not(ruleGroupSQL) : ruleGroupSQL;
  };
  return processRuleGroup(query, true);
};

// src/utils/formatQuery/defaultRuleProcessorDrizzle.ts
var __defProp14 = Object.defineProperty;
var __defProps13 = Object.defineProperties;
var __getOwnPropDescs13 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols14 = Object.getOwnPropertySymbols;
var __hasOwnProp14 = Object.prototype.hasOwnProperty;
var __propIsEnum14 = Object.prototype.propertyIsEnumerable;
var __defNormalProp14 = (obj, key, value) => key in obj ? __defProp14(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues14 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp14.call(b, prop)) __defNormalProp14(a, prop, b[prop]);
  if (__getOwnPropSymbols14) for (var prop of __getOwnPropSymbols14(b)) {
    if (__propIsEnum14.call(b, prop)) __defNormalProp14(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps13 = (a, b) => __defProps13(a, __getOwnPropDescs13(b));
var defaultRuleProcessorDrizzle = (rule, _options) => {
  const opts = _options != null ? _options : (
    /* istanbul ignore next */
    {}
  );
  const { parseNumbers, preserveValueOrder, context = {} } = opts;
  const { columns, drizzleOperators, useRawFields } = context;
  if (!columns || !drizzleOperators) return;
  const { between, eq, gt, gte, inArray, isNotNull, isNull, like, lt, lte, ne, notBetween, notInArray, notLike, sql } = drizzleOperators;
  const { field, operator, value, valueSource } = rule;
  const column = useRawFields && /[a-z][a-z0-9]*/i.test(field) ? sql.raw(field) : columns[field];
  const operatorLC = lc(operator);
  const valueIsField = valueSource === "field";
  const asFieldOrValue = (v) => valueIsField ? columns[v] : v;
  if (!column) return;
  const matchEval = processMatchMode(rule);
  if (matchEval === false) {
    return;
  } else if (matchEval) {
    if (opts.preset !== "postgresql") return;
    const { mode, threshold } = matchEval;
    const arrayElementAlias = "elem_alias";
    const sqlQuery = transformQuery(rule.value, { ruleProcessor: (r) => __spreadProps13(__spreadValues14({}, r), { field: arrayElementAlias }) });
    const nestedArrayFilter = defaultRuleGroupProcessorDrizzle(sqlQuery, __spreadProps13(__spreadValues14({}, opts), { context: __spreadProps13(__spreadValues14({}, opts.context), { useRawFields: true }) }));
    switch (mode) {
      case "all":
        return sql`(select count(*) from unnest(${column}) as ${sql.raw(arrayElementAlias)} where ${nestedArrayFilter({}, drizzleOperators)}) = array_length(${column}, 1)`;
      case "none":
        return sql`not exists (select 1 from unnest(${column}) as ${sql.raw(arrayElementAlias)} where ${nestedArrayFilter({}, drizzleOperators)})`;
      case "some":
        return sql`exists (select 1 from unnest(${column}) as ${sql.raw(arrayElementAlias)} where ${nestedArrayFilter({}, drizzleOperators)})`;
      case "atleast":
      case "atmost":
      case "exactly": {
        const op = mode === "atleast" ? ">=" : mode === "atmost" ? "<=" : "=";
        return threshold > 0 && threshold < 1 ? sql`(select count(*) / array_length(${column}, 1) from unnest(${column}) as ${sql.raw(arrayElementAlias)} where ${nestedArrayFilter({}, drizzleOperators)}) ${sql.raw(`${op} ${threshold}`)}` : sql`(select count(*) from unnest(${column}) as ${sql.raw(arrayElementAlias)} where ${nestedArrayFilter({}, drizzleOperators)}) ${sql.raw(`${op} ${threshold}`)}`;
      }
    }
  }
  switch (operatorLC) {
    case "=":
      return eq(column, asFieldOrValue(value));
    case "!=":
      return ne(column, asFieldOrValue(value));
    case ">":
      return gt(column, asFieldOrValue(value));
    case "<":
      return lt(column, asFieldOrValue(value));
    case ">=":
      return gte(column, asFieldOrValue(value));
    case "<=":
      return lte(column, asFieldOrValue(value));
    case "beginswith":
    case "doesnotbeginwith":
      return (operatorLC === "doesnotbeginwith" ? notLike : like)(column, valueIsField ? sql`${asFieldOrValue(value)} || '%'` : `${value}%`);
    case "contains":
    case "doesnotcontain":
      return (operatorLC === "doesnotcontain" ? notLike : like)(column, valueIsField ? sql`'%' || ${asFieldOrValue(value)} || '%'` : `%${value}%`);
    case "endswith":
    case "doesnotendwith":
      return (operatorLC === "doesnotendwith" ? notLike : like)(column, valueIsField ? sql`'%' || ${asFieldOrValue(value)}` : `%${value}`);
    case "null":
      return isNull(column);
    case "notnull":
      return isNotNull(column);
    case "in":
    case "notin": {
      const valueAsArray = toArray(value).map((v) => asFieldOrValue(v));
      return operatorLC === "notin" ? notInArray(column, valueAsArray) : inArray(column, valueAsArray);
    }
    case "between":
    case "notbetween": {
      const valueAsArray = toArray(value);
      if (valueAsArray.length >= 2 && isValidValue(valueAsArray[0]) && isValidValue(valueAsArray[1])) {
        let [first, second] = valueAsArray;
        const shouldParseNumbers = !(parseNumbers === false);
        if (!valueIsField && shouldRenderAsNumber(first, shouldParseNumbers) && shouldRenderAsNumber(second, shouldParseNumbers)) {
          const firstNum = parseNumber(first, { parseNumbers: shouldParseNumbers });
          const secondNum = parseNumber(second, { parseNumbers: shouldParseNumbers });
          if (!preserveValueOrder && secondNum < firstNum) {
            const tempNum = secondNum;
            second = firstNum;
            first = tempNum;
          } else {
            first = firstNum;
            second = secondNum;
          }
        } else {
          if (valueIsField) {
            first = asFieldOrValue(first);
            second = asFieldOrValue(second);
          }
        }
        return operatorLC === "notbetween" ? notBetween(column, first, second) : between(column, first, second);
      }
      return;
    }
    default:
      return;
  }
};

// src/utils/formatQuery/defaultRuleGroupProcessorElasticSearch.ts
var __defProp15 = Object.defineProperty;
var __defProps14 = Object.defineProperties;
var __getOwnPropDescs14 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols15 = Object.getOwnPropertySymbols;
var __hasOwnProp15 = Object.prototype.hasOwnProperty;
var __propIsEnum15 = Object.prototype.propertyIsEnumerable;
var __defNormalProp15 = (obj, key, value) => key in obj ? __defProp15(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues15 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp15.call(b, prop)) __defNormalProp15(a, prop, b[prop]);
  if (__getOwnPropSymbols15) for (var prop of __getOwnPropSymbols15(b)) {
    if (__propIsEnum15.call(b, prop)) __defNormalProp15(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps14 = (a, b) => __defProps14(a, __getOwnPropDescs14(b));
var defaultRuleGroupProcessorElasticSearch = (ruleGroup, options) => {
  const { fields, getParseNumberBoolean, placeholderFieldName, placeholderOperatorName, placeholderValueName, ruleProcessor, validateRule, validationMap } = options;
  const query = convertFromIC(ruleGroup);
  const processRuleGroup = (rg) => {
    var _a;
    if (!isRuleOrGroupValid(rg, validationMap[(_a = rg.id) != null ? _a : (
      /* istanbul ignore next */
      ""
    )])) {
      return false;
    }
    const processedRules = rg.rules.map((rule) => {
      if (isRuleGroup(rule)) {
        return processRuleGroup(rule);
      }
      const [validationResult, fieldValidator] = validateRule(rule);
      if (!isRuleOrGroupValid(rule, validationResult, fieldValidator) || rule.field === placeholderFieldName || rule.operator === placeholderOperatorName || /* istanbul ignore next */
      placeholderValueName !== void 0 && rule.value === placeholderValueName) {
        return false;
      }
      const fieldData = getOption(fields, rule.field);
      return ruleProcessor(rule, __spreadProps14(__spreadValues15({}, options), { parseNumbers: getParseNumberBoolean(fieldData == null ? void 0 : fieldData.inputType), fieldData }));
    }).filter(Boolean);
    if (processedRules.length === 0) {
      return false;
    }
    return { bool: rg.not ? { must_not: /^or$/i.test(rg.combinator) ? { bool: { should: processedRules } } : processedRules } : { [/^or$/i.test(rg.combinator) ? "should" : "must"]: processedRules } };
  };
  const processedRuleGroup = processRuleGroup(query);
  return processedRuleGroup === false ? {} : processedRuleGroup;
};

// src/utils/formatQuery/defaultRuleGroupProcessorJSONata.ts
var __defProp16 = Object.defineProperty;
var __defProps15 = Object.defineProperties;
var __getOwnPropDescs15 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols16 = Object.getOwnPropertySymbols;
var __hasOwnProp16 = Object.prototype.hasOwnProperty;
var __propIsEnum16 = Object.prototype.propertyIsEnumerable;
var __defNormalProp16 = (obj, key, value) => key in obj ? __defProp16(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues16 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp16.call(b, prop)) __defNormalProp16(a, prop, b[prop]);
  if (__getOwnPropSymbols16) for (var prop of __getOwnPropSymbols16(b)) {
    if (__propIsEnum16.call(b, prop)) __defNormalProp16(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps15 = (a, b) => __defProps15(a, __getOwnPropDescs15(b));
var defaultRuleGroupProcessorJSONata = (ruleGroup, options) => {
  const { fields, fallbackExpression, getParseNumberBoolean, placeholderFieldName, placeholderOperatorName, placeholderValueName, ruleProcessor, validateRule, validationMap } = options;
  const processRuleGroup = (rg, outermost) => {
    var _a;
    if (!isRuleOrGroupValid(rg, validationMap[(_a = rg.id) != null ? _a : (
      /* istanbul ignore next */
      ""
    )])) {
      return outermost ? fallbackExpression : "";
    }
    const expression = rg.rules.map((rule) => {
      var _a2;
      if (typeof rule === "string") {
        return rule;
      }
      if (isRuleGroup(rule)) {
        return processRuleGroup(rule);
      }
      const [validationResult, fieldValidator] = validateRule(rule);
      if (!isRuleOrGroupValid(rule, validationResult, fieldValidator) || rule.field === placeholderFieldName || rule.operator === placeholderOperatorName || /* istanbul ignore next */
      placeholderValueName !== void 0 && rule.value === placeholderValueName) {
        return "";
      }
      const fieldData = getOption(fields, rule.field);
      return ruleProcessor(rule, __spreadProps15(__spreadValues16({}, options), { parseNumbers: getParseNumberBoolean(fieldData == null ? void 0 : fieldData.inputType), escapeQuotes: ((_a2 = rule.valueSource) != null ? _a2 : "value") === "value", fieldData }));
    }).filter(Boolean).join(isRuleGroupType(rg) ? ` ${rg.combinator} ` : " ");
    const [prefix, suffix] = rg.not || !outermost ? [`${rg.not ? "$not" : ""}(`, ")"] : ["", ""];
    return expression ? `${prefix}${expression}${suffix}` : fallbackExpression;
  };
  return processRuleGroup(ruleGroup, true);
};

// src/utils/formatQuery/defaultRuleGroupProcessorJsonLogic.ts
var __defProp17 = Object.defineProperty;
var __defProps16 = Object.defineProperties;
var __getOwnPropDescs16 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols17 = Object.getOwnPropertySymbols;
var __hasOwnProp17 = Object.prototype.hasOwnProperty;
var __propIsEnum17 = Object.prototype.propertyIsEnumerable;
var __defNormalProp17 = (obj, key, value) => key in obj ? __defProp17(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues17 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp17.call(b, prop)) __defNormalProp17(a, prop, b[prop]);
  if (__getOwnPropSymbols17) for (var prop of __getOwnPropSymbols17(b)) {
    if (__propIsEnum17.call(b, prop)) __defNormalProp17(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps16 = (a, b) => __defProps16(a, __getOwnPropDescs16(b));
var defaultRuleGroupProcessorJsonLogic = (ruleGroup, options) => {
  const { fields, getParseNumberBoolean, placeholderFieldName, placeholderOperatorName, placeholderValueName, ruleProcessor, validateRule, validationMap } = options;
  const query = convertFromIC(ruleGroup);
  const processRuleGroup = (rg, _outermost) => {
    var _a;
    if (!isRuleOrGroupValid(rg, validationMap[(_a = rg.id) != null ? _a : (
      /* istanbul ignore next */
      ""
    )])) {
      return false;
    }
    const processedRules = rg.rules.map((rule) => {
      if (isRuleGroup(rule)) {
        return processRuleGroup(rule);
      }
      const [validationResult, fieldValidator] = validateRule(rule);
      if (!isRuleOrGroupValid(rule, validationResult, fieldValidator) || rule.field === placeholderFieldName || rule.operator === placeholderOperatorName || /* istanbul ignore next */
      placeholderValueName !== void 0 && rule.value === placeholderValueName) {
        return false;
      }
      const fieldData = getOption(fields, rule.field);
      return ruleProcessor(rule, __spreadProps16(__spreadValues17({}, options), { parseNumbers: getParseNumberBoolean(fieldData == null ? void 0 : fieldData.inputType), fieldData }));
    }).filter(Boolean);
    if (processedRules.length === 0) {
      return false;
    }
    const jsonRuleGroup = { [rg.combinator]: processedRules };
    return rg.not ? { "!": jsonRuleGroup } : jsonRuleGroup;
  };
  return processRuleGroup(query, true);
};

// src/utils/formatQuery/defaultRuleGroupProcessorLDAP.ts
var __defProp18 = Object.defineProperty;
var __defProps17 = Object.defineProperties;
var __getOwnPropDescs17 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols18 = Object.getOwnPropertySymbols;
var __hasOwnProp18 = Object.prototype.hasOwnProperty;
var __propIsEnum18 = Object.prototype.propertyIsEnumerable;
var __defNormalProp18 = (obj, key, value) => key in obj ? __defProp18(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues18 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp18.call(b, prop)) __defNormalProp18(a, prop, b[prop]);
  if (__getOwnPropSymbols18) for (var prop of __getOwnPropSymbols18(b)) {
    if (__propIsEnum18.call(b, prop)) __defNormalProp18(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps17 = (a, b) => __defProps17(a, __getOwnPropDescs17(b));
var defaultRuleGroupProcessorLDAP = (ruleGroup, options) => {
  const { fields, fallbackExpression, getParseNumberBoolean, placeholderFieldName, placeholderOperatorName, placeholderValueName, ruleProcessor, validateRule, validationMap } = options;
  const query = convertFromIC(ruleGroup);
  const processRuleGroup = (rg, outermost) => {
    var _a;
    if (!isRuleOrGroupValid(rg, validationMap[(_a = rg.id) != null ? _a : (
      /* istanbul ignore next */
      ""
    )])) {
      return outermost ? fallbackExpression : "";
    }
    const rules = rg.rules.map((rule) => {
      var _a2;
      if (isRuleGroup(rule)) {
        return processRuleGroup(rule);
      }
      const [validationResult, fieldValidator] = validateRule(rule);
      if (!isRuleOrGroupValid(rule, validationResult, fieldValidator) || rule.field === placeholderFieldName || rule.operator === placeholderOperatorName || /* istanbul ignore next */
      placeholderValueName !== void 0 && rule.value === placeholderValueName) {
        return "";
      }
      const fieldData = getOption(fields, rule.field);
      return ruleProcessor(rule, __spreadProps17(__spreadValues18({}, options), { parseNumbers: getParseNumberBoolean(fieldData == null ? void 0 : fieldData.inputType), escapeQuotes: ((_a2 = rule.valueSource) != null ? _a2 : "value") === "value", fieldData }));
    }).filter(Boolean);
    const expression = rules.join("");
    const [notPrefix, notSuffix] = rg.not ? ["(!", ")"] : ["", ""];
    const [prefix, suffix] = rules.length > 1 ? [`${notPrefix}(${rg.combinator === "or" ? "|" : "&"}`, `)${notSuffix}`] : [notPrefix, notSuffix];
    return expression ? `${prefix}${expression}${suffix}` : fallbackExpression;
  };
  return processRuleGroup(query, true);
};

// src/utils/formatQuery/defaultRuleGroupProcessorMongoDB.ts
var __defProp19 = Object.defineProperty;
var __defProps18 = Object.defineProperties;
var __getOwnPropDescs18 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols19 = Object.getOwnPropertySymbols;
var __hasOwnProp19 = Object.prototype.hasOwnProperty;
var __propIsEnum19 = Object.prototype.propertyIsEnumerable;
var __defNormalProp19 = (obj, key, value) => key in obj ? __defProp19(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues19 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp19.call(b, prop)) __defNormalProp19(a, prop, b[prop]);
  if (__getOwnPropSymbols19) for (var prop of __getOwnPropSymbols19(b)) {
    if (__propIsEnum19.call(b, prop)) __defNormalProp19(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps18 = (a, b) => __defProps18(a, __getOwnPropDescs18(b));
var isBracketed = (str) => str.startsWith("{") && str.endsWith("}");
var defaultRuleGroupProcessorMongoDB = (ruleGroup, options, meta) => {
  const { fields, fallbackExpression, getParseNumberBoolean, placeholderFieldName, placeholderOperatorName, placeholderValueName, ruleProcessor, validateRule, validationMap } = options;
  const processRuleGroup = (rg, outermost) => {
    var _a;
    if (!isRuleOrGroupValid(rg, validationMap[(_a = rg.id) != null ? _a : (
      /* istanbul ignore next */
      ""
    )])) {
      return outermost ? fallbackExpression : "";
    }
    const combinator = `"$${lc(rg.combinator)}"`;
    let hasChildRules = false;
    const expressions = rg.rules.map((rule) => {
      if (isRuleGroup(rule)) {
        const processedRuleGroup = processRuleGroup(rule);
        if (processedRuleGroup) {
          hasChildRules = true;
          return isBracketed(processedRuleGroup) ? processedRuleGroup : `{${processedRuleGroup}}`;
        }
        return "";
      }
      const [validationResult, fieldValidator] = validateRule(rule);
      if (!isRuleOrGroupValid(rule, validationResult, fieldValidator) || rule.field === placeholderFieldName || rule.operator === placeholderOperatorName || /* istanbul ignore next */
      placeholderValueName !== void 0 && rule.value === placeholderValueName) {
        return "";
      }
      const fieldData = getOption(fields, rule.field);
      return ruleProcessor(rule, __spreadProps18(__spreadValues19({}, options), { parseNumbers: getParseNumberBoolean(fieldData == null ? void 0 : fieldData.inputType), fieldData }), meta);
    }).filter(Boolean);
    return expressions.length > 0 ? expressions.length === 1 && !hasChildRules ? expressions[0] : `${combinator}:[${expressions.join(",")}]` : fallbackExpression;
  };
  const processedQuery = processRuleGroup(convertFromIC(ruleGroup), true);
  return isBracketed(processedQuery) ? processedQuery : `{${processedQuery}}`;
};

// src/utils/formatQuery/defaultRuleGroupProcessorNL.ts
var __defProp20 = Object.defineProperty;
var __defProps19 = Object.defineProperties;
var __getOwnPropDescs19 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols20 = Object.getOwnPropertySymbols;
var __hasOwnProp20 = Object.prototype.hasOwnProperty;
var __propIsEnum20 = Object.prototype.propertyIsEnumerable;
var __defNormalProp20 = (obj, key, value) => key in obj ? __defProp20(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues20 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp20.call(b, prop)) __defNormalProp20(a, prop, b[prop]);
  if (__getOwnPropSymbols20) for (var prop of __getOwnPropSymbols20(b)) {
    if (__propIsEnum20.call(b, prop)) __defNormalProp20(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps19 = (a, b) => __defProps19(a, __getOwnPropDescs19(b));
var defaultRuleGroupProcessorNL = (ruleGroup, options) => {
  const { fields, fallbackExpression, getParseNumberBoolean, placeholderFieldName, placeholderOperatorName, placeholderValueName, ruleProcessor, translations, validateRule, validationMap } = options;
  const processRuleGroup = (rg, outermostOrLonelyInGroup) => {
    var _a, _b, _c;
    if (!isRuleOrGroupValid(rg, validationMap[(_a = rg.id) != null ? _a : (
      /* istanbul ignore next */
      ""
    )])) {
      return outermostOrLonelyInGroup ? fallbackExpression : (
        /* istanbul ignore next */
        ""
      );
    }
    const rg2 = isRuleGroupTypeIC(rg) && rg.rules.some((r) => typeof r === "string" && lc(r) === "xor") ? convertFromIC(rg) : rg;
    const processedRules = rg2.rules.map((rule) => {
      var _a2, _b2, _c2;
      if (typeof rule === "string") {
        return `, ${(_a2 = translations[rule]) != null ? _a2 : rule} `;
      }
      if (isRuleGroup(rule)) {
        return processRuleGroup(rule, rg2.rules.length === 1 && !(rg2.not || /^xor$/i.test((_b2 = rg2.combinator) != null ? _b2 : (
          /* istanbul ignore next */
          ""
        ))));
      }
      const [validationResult, fieldValidator] = validateRule(rule);
      if (!isRuleOrGroupValid(rule, validationResult, fieldValidator) || rule.field === placeholderFieldName || rule.operator === placeholderOperatorName || /* istanbul ignore next */
      placeholderValueName !== void 0 && rule.value === placeholderValueName) {
        return "";
      }
      const escapeQuotes = ((_c2 = rule.valueSource) != null ? _c2 : "value") === "value";
      const fieldData = getOption(fields, rule.field);
      return ruleProcessor(rule, __spreadProps19(__spreadValues20({}, options), { parseNumbers: getParseNumberBoolean(fieldData == null ? void 0 : fieldData.inputType), escapeQuotes, fieldData }));
    });
    if (processedRules.length === 0) {
      return fallbackExpression;
    }
    const isXOR = lc((_b = rg2.combinator) != null ? _b : "") === "xor";
    const combinator = isXOR ? rg2.combinator.slice(1) : rg2.combinator;
    const mustWrap = rg2.not || !outermostOrLonelyInGroup || isXOR && processedRules.length > 1;
    const [prefixTL, suffixTL] = ["groupPrefix", "groupSuffix"].map((key) => rg2.not ? isXOR ? getNLTranslataion(key, translations, ["not", "xor"]) : getNLTranslataion(key, translations, ["not"]) : isXOR ? getNLTranslataion(key, translations, ["xor"]) : getNLTranslataion(key, translations));
    const prefix = mustWrap ? `${prefixTL} (`.trim() : "";
    const suffix = mustWrap ? `) ${suffixTL}`.trim() : "";
    return `${prefix}${processedRules.filter(Boolean).join(isRuleGroupType(rg2) ? `, ${(_c = translations[combinator]) != null ? _c : combinator} ` : "")}${suffix}`;
  };
  return processRuleGroup(ruleGroup, true);
};

// src/utils/formatQuery/defaultRuleGroupProcessorParameterized.ts
var __defProp21 = Object.defineProperty;
var __defProps20 = Object.defineProperties;
var __getOwnPropDescs20 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols21 = Object.getOwnPropertySymbols;
var __hasOwnProp21 = Object.prototype.hasOwnProperty;
var __propIsEnum21 = Object.prototype.propertyIsEnumerable;
var __defNormalProp21 = (obj, key, value) => key in obj ? __defProp21(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues21 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp21.call(b, prop)) __defNormalProp21(a, prop, b[prop]);
  if (__getOwnPropSymbols21) for (var prop of __getOwnPropSymbols21(b)) {
    if (__propIsEnum21.call(b, prop)) __defNormalProp21(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps20 = (a, b) => __defProps20(a, __getOwnPropDescs20(b));
var defaultRuleGroupProcessorParameterized = (ruleGroup, options) => {
  const { format, fields, fallbackExpression, getParseNumberBoolean, placeholderFieldName, placeholderOperatorName, placeholderValueName, ruleProcessor, validateRule, validationMap } = options;
  const parameterized = format === "parameterized";
  const params = [];
  const paramsNamed = {};
  const fieldParams = /* @__PURE__ */ new Map();
  const getNextNamedParam = (field) => {
    if (!fieldParams.has(field)) {
      fieldParams.set(field, /* @__PURE__ */ new Set());
    }
    const nextNamedParam = `${field}_${fieldParams.get(field).size + 1}`;
    fieldParams.get(field).add(nextNamedParam);
    return nextNamedParam;
  };
  const processRule = (rule) => {
    var _a;
    const [validationResult, fieldValidator] = validateRule(rule);
    if (!isRuleOrGroupValid(rule, validationResult, fieldValidator) || rule.field === placeholderFieldName || rule.operator === placeholderOperatorName || /* istanbul ignore next */
    placeholderValueName !== void 0 && rule.value === placeholderValueName) {
      return "";
    }
    const fieldData = getOption(fields, rule.field);
    const fieldParamNames = Object.fromEntries([...fieldParams.entries()].map((_ref) => {
      let [f, s] = _ref;
      return [f, [...s]];
    }));
    const processedRule = ruleProcessor(rule, __spreadProps20(__spreadValues21({}, options), { parseNumbers: getParseNumberBoolean(fieldData == null ? void 0 : fieldData.inputType), getNextNamedParam, fieldParamNames, fieldData }), { processedParams: params });
    if (!isPojo(processedRule)) {
      return "";
    }
    const { sql, params: customParams } = processedRule;
    if (typeof sql !== "string" || !sql) {
      return "";
    }
    if (format === "parameterized" && Array.isArray(customParams)) {
      params.push(...customParams);
    } else if (format === "parameterized_named" && isPojo(customParams)) {
      Object.assign(paramsNamed, customParams);
      for (const p of Object.keys(customParams)) (_a = fieldParams.get(rule.field)) == null ? void 0 : _a.add(p);
    }
    return sql;
  };
  const processRuleGroup = (rg, outermostOrLonelyInGroup) => {
    var _a;
    if (!isRuleOrGroupValid(rg, validationMap[(_a = rg.id) != null ? _a : (
      /* istanbul ignore next */
      ""
    )])) {
      return outermostOrLonelyInGroup ? fallbackExpression : (
        /* istanbul ignore next */
        ""
      );
    }
    const processedRules = rg.rules.map((rule) => {
      if (typeof rule === "string") {
        return rule;
      }
      if (isRuleGroup(rule)) {
        return processRuleGroup(rule, rg.rules.length === 1);
      }
      return processRule(rule);
    }).filter(Boolean);
    if (processedRules.length === 0) {
      return fallbackExpression;
    }
    return `${rg.not ? "NOT " : ""}(${processedRules.join(isRuleGroupType(rg) ? ` ${rg.combinator} ` : " ")})`;
  };
  if (parameterized) {
    return { sql: processRuleGroup(ruleGroup, true), params };
  }
  return { sql: processRuleGroup(ruleGroup, true), params: paramsNamed };
};

// src/utils/formatQuery/defaultRuleGroupProcessorPrisma.ts
var __defProp22 = Object.defineProperty;
var __defProps21 = Object.defineProperties;
var __getOwnPropDescs21 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols22 = Object.getOwnPropertySymbols;
var __hasOwnProp22 = Object.prototype.hasOwnProperty;
var __propIsEnum22 = Object.prototype.propertyIsEnumerable;
var __defNormalProp22 = (obj, key, value) => key in obj ? __defProp22(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues22 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp22.call(b, prop)) __defNormalProp22(a, prop, b[prop]);
  if (__getOwnPropSymbols22) for (var prop of __getOwnPropSymbols22(b)) {
    if (__propIsEnum22.call(b, prop)) __defNormalProp22(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps21 = (a, b) => __defProps21(a, __getOwnPropDescs21(b));
var prismaFallback = {};
var defaultRuleGroupProcessorPrisma = (ruleGroup, options) => {
  const { fields, getParseNumberBoolean, placeholderFieldName, placeholderOperatorName, placeholderValueName, ruleProcessor, validateRule, validationMap } = options;
  const processRuleGroup = (rg, outermost) => {
    var _a;
    if (!isRuleOrGroupValid(rg, validationMap[(_a = rg.id) != null ? _a : (
      /* istanbul ignore next */
      ""
    )])) {
      return outermost ? prismaFallback : void 0;
    }
    const combinator = rg.combinator.toUpperCase();
    let hasChildRules = false;
    const expressions = rg.rules.map((rule) => {
      if (isRuleGroup(rule)) {
        const processedRuleGroup = processRuleGroup(rule);
        if (processedRuleGroup) {
          hasChildRules = true;
          return processedRuleGroup;
        }
        return;
      }
      const [validationResult, fieldValidator] = validateRule(rule);
      if (!isRuleOrGroupValid(rule, validationResult, fieldValidator) || rule.field === placeholderFieldName || rule.operator === placeholderOperatorName || /* istanbul ignore next */
      placeholderValueName !== void 0 && rule.value === placeholderValueName) {
        return;
      }
      const fieldData = getOption(fields, rule.field);
      return ruleProcessor(rule, __spreadProps21(__spreadValues22({}, options), { parseNumbers: getParseNumberBoolean(fieldData == null ? void 0 : fieldData.inputType), fieldData }));
    }).filter(Boolean);
    return expressions.length > 0 ? expressions.length === 1 && !hasChildRules ? expressions[0] : { [combinator]: expressions } : prismaFallback;
  };
  const result = processRuleGroup(convertFromIC(ruleGroup), true);
  return ruleGroup.not ? { NOT: result } : result;
};

// src/utils/formatQuery/defaultRuleGroupProcessorSequelize.ts
var __defProp23 = Object.defineProperty;
var __defProps22 = Object.defineProperties;
var __getOwnPropDescs22 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols23 = Object.getOwnPropertySymbols;
var __hasOwnProp23 = Object.prototype.hasOwnProperty;
var __propIsEnum23 = Object.prototype.propertyIsEnumerable;
var __defNormalProp23 = (obj, key, value) => key in obj ? __defProp23(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues23 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp23.call(b, prop)) __defNormalProp23(a, prop, b[prop]);
  if (__getOwnPropSymbols23) for (var prop of __getOwnPropSymbols23(b)) {
    if (__propIsEnum23.call(b, prop)) __defNormalProp23(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps22 = (a, b) => __defProps22(a, __getOwnPropDescs22(b));
var defaultRuleGroupProcessorSequelize = (ruleGroup, options) => {
  const { fields, getParseNumberBoolean, placeholderFieldName, placeholderOperatorName, placeholderValueName, ruleProcessor, validateRule, validationMap, context = {} } = options;
  const { sequelizeOperators: Op } = context;
  if (!Op) return;
  const processRuleGroup = (rg, _outermost) => {
    var _a;
    if (!isRuleOrGroupValid(rg, validationMap[(_a = rg.id) != null ? _a : (
      /* istanbul ignore next */
      ""
    )])) {
      return;
    }
    const combinator = rg.combinator.toUpperCase();
    let hasChildRules = false;
    const expressions = rg.rules.map((rule) => {
      if (isRuleGroup(rule)) {
        const processedRuleGroup = processRuleGroup(rule);
        if (processedRuleGroup) {
          hasChildRules = true;
          return processedRuleGroup;
        }
        return;
      }
      const [validationResult, fieldValidator] = validateRule(rule);
      if (!isRuleOrGroupValid(rule, validationResult, fieldValidator) || rule.field === placeholderFieldName || rule.operator === placeholderOperatorName || /* istanbul ignore next */
      placeholderValueName !== void 0 && rule.value === placeholderValueName) {
        return;
      }
      const fieldData = getOption(fields, rule.field);
      return ruleProcessor(rule, __spreadProps22(__spreadValues23({}, options), { parseNumbers: getParseNumberBoolean(fieldData == null ? void 0 : fieldData.inputType), fieldData }));
    }).filter(Boolean);
    if (expressions.length === 0) return;
    const result = expressions.length === 1 && !hasChildRules ? expressions[0] : { [lc(combinator) === "or" ? Op.or : Op.and]: expressions };
    return rg.not ? { [Op.not]: result } : result;
  };
  return processRuleGroup(convertFromIC(ruleGroup), true);
};

// src/utils/formatQuery/defaultRuleGroupProcessorSQL.ts
var __defProp24 = Object.defineProperty;
var __defProps23 = Object.defineProperties;
var __getOwnPropDescs23 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols24 = Object.getOwnPropertySymbols;
var __hasOwnProp24 = Object.prototype.hasOwnProperty;
var __propIsEnum24 = Object.prototype.propertyIsEnumerable;
var __defNormalProp24 = (obj, key, value) => key in obj ? __defProp24(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues24 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp24.call(b, prop)) __defNormalProp24(a, prop, b[prop]);
  if (__getOwnPropSymbols24) for (var prop of __getOwnPropSymbols24(b)) {
    if (__propIsEnum24.call(b, prop)) __defNormalProp24(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps23 = (a, b) => __defProps23(a, __getOwnPropDescs23(b));
var defaultRuleGroupProcessorSQL = (ruleGroup, options) => {
  const { fields, fallbackExpression, getParseNumberBoolean, placeholderFieldName, placeholderOperatorName, placeholderValueName, ruleProcessor, validateRule, validationMap } = options;
  const processRuleGroup = (rg, outermostOrLonelyInGroup) => {
    var _a;
    if (!isRuleOrGroupValid(rg, validationMap[(_a = rg.id) != null ? _a : (
      /* istanbul ignore next */
      ""
    )])) {
      return outermostOrLonelyInGroup ? fallbackExpression : (
        /* istanbul ignore next */
        ""
      );
    }
    const processedRules = rg.rules.map((rule) => {
      var _a2;
      if (typeof rule === "string") {
        return rule;
      }
      if (isRuleGroup(rule)) {
        return processRuleGroup(rule, rg.rules.length === 1);
      }
      const [validationResult, fieldValidator] = validateRule(rule);
      if (!isRuleOrGroupValid(rule, validationResult, fieldValidator) || rule.field === placeholderFieldName || rule.operator === placeholderOperatorName || placeholderValueName !== void 0 && rule.value === placeholderValueName) {
        return "";
      }
      const escapeQuotes = ((_a2 = rule.valueSource) != null ? _a2 : "value") === "value";
      const fieldData = getOption(fields, rule.field);
      return ruleProcessor(rule, __spreadProps23(__spreadValues24({}, options), { parseNumbers: getParseNumberBoolean(fieldData == null ? void 0 : fieldData.inputType), escapeQuotes, fieldData }));
    }).filter(Boolean);
    if (processedRules.length === 0) {
      return fallbackExpression;
    }
    return `${rg.not ? "NOT " : ""}(${processedRules.join(isRuleGroupType(rg) ? ` ${rg.combinator} ` : " ")})`;
  };
  return processRuleGroup(ruleGroup, true);
};

// src/utils/formatQuery/defaultRuleProcessorElasticSearch.ts
var rangeOperatorMap = { "<": "lt", "<=": "lte", ">": "gt", ">=": "gte" };
var negateIfNotOp = (op, elasticSearchRule) => op.startsWith("not") || op.startsWith("doesnot") ? { bool: { must_not: elasticSearchRule } } : elasticSearchRule;
var escapeSQ = (s) => s == null ? void 0 : s.replace(/('|\\)/g, `\\$1`);
var textFunctionMap = { beginswith: "startsWith", doesnotbeginwith: "startsWith", doesnotcontain: "contains", doesnotendwith: "endsWith", endswith: "endsWith" };
var getTextScript = (f, o, v) => {
  var _a;
  const script = `doc['${f}'].value.${(_a = textFunctionMap[o]) != null ? _a : o}(doc['${v}'].value)`;
  return o.startsWith("d") ? `!${script}` : script;
};
var valueRenderer = (v, parseNumbers) => typeof v === "boolean" ? v : shouldRenderAsNumber(v, parseNumbers) ? parseNumber(v, { parseNumbers }) : v;
var defaultRuleProcessorElasticSearch = function(rule) {
  let options = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  const { field, operator, value, valueSource } = rule;
  const { parseNumbers, preserveValueOrder } = options;
  const operatorLC = lc(operator);
  if (processMatchMode(rule)) return false;
  if (valueSource === "field") {
    if (toArray(value).some((v) => typeof v !== "string")) return false;
    const fieldForScript = escapeSQ(field);
    switch (operatorLC) {
      case "=":
      case "!=":
      case ">":
      case ">=":
      case "<":
      case "<=": {
        const operatorForScript = operatorLC === "=" ? "==" : operatorLC;
        const valueForScript = escapeSQ(value);
        return valueForScript ? { bool: { filter: { script: { script: `doc['${fieldForScript}'].value ${operatorForScript} doc['${valueForScript}'].value` } } } } : false;
      }
      case "in":
      case "notin": {
        const valueAsArray = toArray(value);
        if (valueAsArray.length > 0) {
          const arr = valueAsArray.map((v) => ({ bool: { filter: { script: { script: `doc['${fieldForScript}'].value == doc['${v}'].value` } } } }));
          return { bool: operatorLC === "in" ? { should: arr } : { must_not: arr } };
        }
        return false;
      }
      case "between":
      case "notbetween": {
        const valueAsArray = toArray(value);
        if (valueAsArray.length >= 2 && valueAsArray[0] && valueAsArray[1]) {
          const script = `doc['${fieldForScript}'].value >= doc['${valueAsArray[0]}'].value && doc['${fieldForScript}'].value <= doc['${valueAsArray[1]}'].value`;
          return { bool: { filter: { script: { script: operatorLC === "notbetween" ? `!(${script})` : script } } } };
        }
        return false;
      }
      case "contains":
      case "doesnotcontain":
      case "beginswith":
      case "doesnotbeginwith":
      case "endswith":
      case "doesnotendwith": {
        const valueForScript = escapeSQ(value);
        if (!valueForScript) return false;
        const script = getTextScript(fieldForScript, operatorLC, valueForScript);
        return { bool: { filter: { script: { script } } } };
      }
    }
  }
  switch (operatorLC) {
    case "<":
    case "<=":
    case ">":
    case ">=":
      return { range: { [field]: { [rangeOperatorMap[operatorLC]]: valueRenderer(value, parseNumbers) } } };
    case "=":
      return { term: { [field]: valueRenderer(value, parseNumbers) } };
    case "!=":
      return { bool: { must_not: { term: { [field]: valueRenderer(value, parseNumbers) } } } };
    case "null":
      return { bool: { must_not: { exists: { field } } } };
    case "notnull":
      return { exists: { field } };
    case "in":
    case "notin": {
      const valueAsArray = toArray(value).map((v) => valueRenderer(v, parseNumbers));
      if (valueAsArray.length > 0) {
        const arr = valueAsArray.map((v) => ({ term: { [field]: valueRenderer(v, parseNumbers) } }));
        return { bool: operatorLC === "in" ? { should: arr } : { must_not: arr } };
      }
      return false;
    }
    case "between":
    case "notbetween": {
      const valueAsArray = toArray(value);
      if (valueAsArray.length >= 2 && isValidValue(valueAsArray[0]) && isValidValue(valueAsArray[1])) {
        let [first, second] = valueAsArray;
        const shouldParseNumbers = !(parseNumbers === false);
        if (shouldRenderAsNumber(first, shouldParseNumbers) && shouldRenderAsNumber(second, shouldParseNumbers)) {
          const firstNum = parseNumber(first, { parseNumbers: shouldParseNumbers });
          const secondNum = parseNumber(second, { parseNumbers: shouldParseNumbers });
          if (!preserveValueOrder && secondNum < firstNum) {
            const tempNum = secondNum;
            second = firstNum;
            first = tempNum;
          } else {
            first = firstNum;
            second = secondNum;
          }
        }
        return negateIfNotOp(operatorLC, { range: { [field]: { gte: first, lte: second } } });
      }
      return false;
    }
    case "contains":
    case "doesnotcontain":
      return negateIfNotOp(operatorLC, { regexp: { [field]: { value: `.*${value}.*` } } });
    case "beginswith":
    case "doesnotbeginwith":
      return negateIfNotOp(operatorLC, { regexp: { [field]: { value: `${value}.*` } } });
    case "endswith":
    case "doesnotendwith":
      return negateIfNotOp(operatorLC, { regexp: { [field]: { value: `.*${value}` } } });
  }
  return false;
};

// src/utils/formatQuery/defaultRuleProcessorJSONata.ts
var __defProp25 = Object.defineProperty;
var __defProps24 = Object.defineProperties;
var __getOwnPropDescs24 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols25 = Object.getOwnPropertySymbols;
var __hasOwnProp25 = Object.prototype.hasOwnProperty;
var __propIsEnum25 = Object.prototype.propertyIsEnumerable;
var __defNormalProp25 = (obj, key, value) => key in obj ? __defProp25(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues25 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp25.call(b, prop)) __defNormalProp25(a, prop, b[prop]);
  if (__getOwnPropSymbols25) for (var prop of __getOwnPropSymbols25(b)) {
    if (__propIsEnum25.call(b, prop)) __defNormalProp25(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps24 = (a, b) => __defProps24(a, __getOwnPropDescs24(b));
var shouldNegate3 = (op) => op.startsWith("not") || op.startsWith("doesnot");
var quote = (v, escapeQuotes) => `"${typeof v !== "string" || !escapeQuotes ? v : v.replaceAll(`"`, `\\"`)}"`;
var negate = (clause, negate2) => negate2 ? `$not(${clause})` : `${clause}`;
var escapeStringRegex = (s) => `${s}`.replaceAll(/[/$()*+.?[\\\]^{|}]/g, String.raw`\$&`).replaceAll("-", String.raw`\x2d`);
var defaultRuleProcessorJSONata = function(rule) {
  let options = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  const { field, operator, value, valueSource } = rule;
  const { escapeQuotes, parseNumbers, preserveValueOrder, quoteFieldNamesWith = ["", ""], fieldIdentifierSeparator = "" } = options;
  const valueIsField = valueSource === "field";
  const useBareValue = typeof value === "number" || typeof value === "boolean" || typeof value === "bigint" || shouldRenderAsNumber(value, parseNumbers);
  const qfn = (f) => getQuotedFieldName(f, { quoteFieldNamesWith, fieldIdentifierSeparator });
  const matchEval = processMatchMode(rule);
  if (matchEval === false) {
    return;
  } else if (matchEval) {
    const { mode, threshold } = matchEval;
    const totalCount = `$count(${qfn(field)})`;
    const filteredCount = `$count($filter(${qfn(field)}, function($v) {${defaultRuleGroupProcessorJSONata(transformQuery(value, { ruleProcessor: (r) => __spreadProps24(__spreadValues25({}, r), { field: r.field ? `$v.${r.field}` : "$v" }) }), options)}}))`;
    switch (mode) {
      case "all":
        return `${filteredCount} = ${totalCount}`;
      case "none":
        return `${filteredCount} = 0`;
      case "some":
        return `${filteredCount} > 0`;
      case "atleast":
      case "atmost":
      case "exactly": {
        const op = mode === "atleast" ? ">=" : mode === "atmost" ? "<=" : "=";
        if (threshold > 0 && threshold < 1) {
          return `${filteredCount} ${op} (${totalCount} * ${threshold})`;
        }
        return `${filteredCount} ${op} ${threshold}`;
      }
    }
  }
  const operatorLC = lc(operator);
  switch (operatorLC) {
    case "<":
    case "<=":
    case "=":
    case "!=":
    case ">":
    case ">=":
      return `${qfn(field)} ${operatorLC} ${valueIsField ? qfn(trimIfString(value)) : useBareValue ? trimIfString(value) : quote(value, escapeQuotes)}`;
    case "contains":
    case "doesnotcontain":
      return negate(`$contains(${qfn(field)}, ${valueIsField ? qfn(trimIfString(value)) : quote(value, escapeQuotes)})`, shouldNegate3(operatorLC));
    case "beginswith":
    case "doesnotbeginwith":
      return negate(valueIsField ? `$substring(${qfn(field)}, 0, $length(${qfn(trimIfString(value))})) = ${qfn(trimIfString(value))}` : `$contains(${qfn(field)}, /^${escapeStringRegex(value)}/)`, shouldNegate3(operatorLC));
    case "endswith":
    case "doesnotendwith":
      return negate(valueIsField ? `$substring(${qfn(field)}, $length(${qfn(field)}) - $length(${qfn(trimIfString(value))})) = ${qfn(trimIfString(value))}` : `$contains(${qfn(field)}, /${escapeStringRegex(value)}$/)`, shouldNegate3(operatorLC));
    case "null":
      return `${qfn(field)} = null`;
    case "notnull":
      return `${qfn(field)} != null`;
    case "in":
    case "notin": {
      const valueAsArray = toArray(value);
      return negate(`${qfn(field)} in [${valueAsArray.map((val) => valueIsField ? `${qfn(trimIfString(val))}` : shouldRenderAsNumber(val, parseNumbers) ? `${trimIfString(val)}` : quote(val, escapeQuotes)).join(", ")}]`, shouldNegate3(operatorLC));
    }
    case "between":
    case "notbetween": {
      const valueAsArray = toArray(value);
      if (valueAsArray.length < 2 || nullOrUndefinedOrEmpty(valueAsArray[0]) || nullOrUndefinedOrEmpty(valueAsArray[1])) {
        return "";
      }
      const [first, second] = valueAsArray;
      const shouldParseNumbers = !(parseNumbers === false);
      const firstNum = shouldRenderAsNumber(first, shouldParseNumbers) ? parseNumber(first, { parseNumbers: shouldParseNumbers }) : Number.NaN;
      const secondNum = shouldRenderAsNumber(second, shouldParseNumbers) ? parseNumber(second, { parseNumbers: shouldParseNumbers }) : Number.NaN;
      let firstValue = Number.isNaN(firstNum) ? valueIsField ? `${first}` : first : firstNum;
      let secondValue = Number.isNaN(secondNum) ? valueIsField ? `${second}` : second : secondNum;
      if (!preserveValueOrder && firstValue === firstNum && secondValue === secondNum && secondNum < firstNum) {
        const tempNum = secondNum;
        secondValue = firstNum;
        firstValue = tempNum;
      }
      const renderAsNumbers = shouldRenderAsNumber(first, parseNumbers) && shouldRenderAsNumber(second, parseNumbers);
      const getValueString = (raw, val) => valueIsField ? qfn(raw) : renderAsNumbers ? val : quote(val, escapeQuotes);
      const expression = `${qfn(field)} >= ${getValueString(first, firstValue)} and ${qfn(field)} <= ${getValueString(second, secondValue)}`;
      return operatorLC === "between" ? `(${expression})` : negate(expression, true);
    }
  }
  return "";
};

// src/utils/formatQuery/defaultRuleProcessorJsonLogic.ts
var convertOperator = (op) => op.replace(/^(=)$/, "$1=").replace(/^notnull$/i, "!=").replace(/^null$/i, "==");
var negateIfNotOp2 = (op, jsonRule) => op.startsWith("not") || op.startsWith("doesnot") ? { "!": jsonRule } : jsonRule;
var defaultRuleProcessorJsonLogic = function(rule) {
  let options = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  const { field, operator, value, valueSource } = rule;
  const { parseNumbers, preserveValueOrder } = options;
  const valueIsField = valueSource === "field";
  const fieldObject = { var: field };
  const fieldOrNumberRenderer = (v) => valueIsField ? { var: `${v}` } : shouldRenderAsNumber(v, parseNumbers) ? parseNumber(v, { parseNumbers }) : v;
  const matchEval = processMatchMode(rule);
  if (matchEval === false) {
    return false;
  } else if (matchEval) {
    const { mode, threshold } = matchEval;
    switch (mode) {
      case "all":
      case "none":
      case "some":
        return { [mode]: [{ var: field }, value.rules.length === 1 && !isRuleGroup(value.rules[0]) ? defaultRuleProcessorJsonLogic(value.rules[0], options) : defaultRuleGroupProcessorJsonLogic(value, options)] };
      case "atleast":
      case "atmost":
      case "exactly": {
        const op = mode === "atleast" ? ">=" : mode === "atmost" ? "<=" : "==";
        const filteredCount = { reduce: [{ filter: [{ var: field }, value.rules.length === 1 && !isRuleGroup(value.rules[0]) ? defaultRuleProcessorJsonLogic(value.rules[0], options) : defaultRuleGroupProcessorJsonLogic(value, options)] }, { "+": [1, { var: "accumulator" }] }, 0] };
        if (threshold > 0 && threshold < 1) {
          const totalCount = { reduce: [{ var: field }, { "+": [1, { var: "accumulator" }] }, 0] };
          return { [op]: [filteredCount, { "*": [totalCount, threshold] }] };
        }
        return { [op]: [filteredCount, threshold] };
      }
    }
  }
  const operatorLC = lc(operator);
  switch (operatorLC) {
    case "<":
    case "<=":
    case "=":
    case "!=":
    case ">":
    case ">=":
      return { [convertOperator(operatorLC)]: [fieldObject, fieldOrNumberRenderer(value)] };
    case "null":
    case "notnull": {
      return { [`${operatorLC === "notnull" ? "!" : "="}=`]: [fieldObject, null] };
    }
    case "in":
    case "notin": {
      const valueAsArray = toArray(value).map((v) => fieldOrNumberRenderer(v));
      return negateIfNotOp2(operatorLC, { in: [fieldObject, valueAsArray] });
    }
    case "between":
    case "notbetween": {
      const valueAsArray = toArray(value);
      if (valueAsArray.length >= 2 && isValidValue(valueAsArray[0]) && isValidValue(valueAsArray[1])) {
        let [first, second] = valueAsArray;
        const shouldParseNumbers = !(parseNumbers === false);
        if (!valueIsField && shouldRenderAsNumber(first, shouldParseNumbers) && shouldRenderAsNumber(second, shouldParseNumbers)) {
          const firstNum = parseNumber(first, { parseNumbers: shouldParseNumbers });
          const secondNum = parseNumber(second, { parseNumbers: shouldParseNumbers });
          if (!preserveValueOrder && secondNum < firstNum) {
            const tempNum = secondNum;
            second = firstNum;
            first = tempNum;
          } else {
            first = firstNum;
            second = secondNum;
          }
        } else if (valueIsField) {
          first = { var: first };
          second = { var: second };
        }
        const jsonRule = { "<=": [first, fieldObject, second] };
        return negateIfNotOp2(operatorLC, jsonRule);
      }
      return false;
    }
    case "contains":
    case "doesnotcontain": {
      const jsonRule = { in: [fieldOrNumberRenderer(value), fieldObject] };
      return negateIfNotOp2(operatorLC, jsonRule);
    }
    case "beginswith":
    case "doesnotbeginwith": {
      const jsonRule = { startsWith: [fieldObject, fieldOrNumberRenderer(value)] };
      return negateIfNotOp2(operatorLC, jsonRule);
    }
    case "endswith":
    case "doesnotendwith": {
      const jsonRule = { endsWith: [fieldObject, fieldOrNumberRenderer(value)] };
      return negateIfNotOp2(operatorLC, jsonRule);
    }
  }
  return false;
};

// src/utils/formatQuery/defaultRuleProcessorLDAP.ts
var negateIf = (clause, negate2) => negate2 ? `(!${clause})` : `${clause}`;
var ldapEscape = (s) => `${trimIfString(s)}`.replaceAll(/[()&|=<>~*\\/]/g, (m) => `\\${m.codePointAt(0).toString(16)}`);
var defaultRuleProcessorLDAP = function(rule) {
  let options = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  const { field, operator, value, valueSource } = rule;
  const { preserveValueOrder } = options;
  const operatorLC = lc(operator);
  if (
    // This is a field comparison (which LDAP does not support), or
    valueSource === "field" || // `value` is null/undefined/empty and the operator is not unary, or
    nullOrUndefinedOrEmpty(value) && operatorLC !== "null" && operatorLC !== "notnull" || // a valid `match` property is found (match modes are not supported in this format)
    processMatchMode(rule)
  ) {
    return "";
  }
  switch (operatorLC) {
    case "=":
    case "!=":
      return negateIf(`(${field}=${ldapEscape(value)})`, operatorLC === "!=");
    case ">":
    case ">=":
      return `(${field}>=${ldapEscape(value)})`;
    case "<":
    case "<=":
      return `(${field}<=${ldapEscape(value)})`;
    case "contains":
    case "doesnotcontain":
      return negateIf(`(${field}=*${ldapEscape(value)}*)`, operatorLC === "doesnotcontain");
    case "beginswith":
    case "doesnotbeginwith":
      return negateIf(`(${field}=${ldapEscape(value)}*)`, operatorLC === "doesnotbeginwith");
    case "endswith":
    case "doesnotendwith":
      return negateIf(`(${field}=*${ldapEscape(value)})`, operatorLC === "doesnotendwith");
    case "null":
    case "notnull":
      return negateIf(`(${field}=*)`, operatorLC === "notnull");
    case "in":
    case "notin": {
      const valueAsArray = toArray(value);
      return negateIf(`(|${valueAsArray.map((val) => `(${field}=${ldapEscape(val)})`).join("")})`, operatorLC === "notin");
    }
    case "between":
    case "notbetween": {
      const valueAsArray = toArray(value);
      if (valueAsArray.length < 2 || nullOrUndefinedOrEmpty(valueAsArray[0]) || nullOrUndefinedOrEmpty(valueAsArray[1])) {
        return "";
      }
      const [first, second] = valueAsArray;
      const firstNum = shouldRenderAsNumber(first, true) ? parseNumber(first, { parseNumbers: true }) : Number.NaN;
      const secondNum = shouldRenderAsNumber(second, true) ? parseNumber(second, { parseNumbers: true }) : Number.NaN;
      let firstValue = Number.isNaN(firstNum) ? first : firstNum;
      let secondValue = Number.isNaN(secondNum) ? second : secondNum;
      if (!preserveValueOrder && firstValue === firstNum && secondValue === secondNum && secondNum < firstNum) {
        const tempNum = secondNum;
        secondValue = firstNum;
        firstValue = tempNum;
      }
      return negateIf(`(&(${field}>=${ldapEscape(firstValue)})(${field}<=${ldapEscape(secondValue)}))`, operatorLC === "notbetween");
    }
  }
  return "";
};

// src/utils/formatQuery/defaultValueProcessorNL.ts
var __defProp26 = Object.defineProperty;
var __defProps25 = Object.defineProperties;
var __getOwnPropDescs25 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols26 = Object.getOwnPropertySymbols;
var __hasOwnProp26 = Object.prototype.hasOwnProperty;
var __propIsEnum26 = Object.prototype.propertyIsEnumerable;
var __defNormalProp26 = (obj, key, value) => key in obj ? __defProp26(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues26 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp26.call(b, prop)) __defNormalProp26(a, prop, b[prop]);
  if (__getOwnPropSymbols26) for (var prop of __getOwnPropSymbols26(b)) {
    if (__propIsEnum26.call(b, prop)) __defNormalProp26(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps25 = (a, b) => __defProps25(a, __getOwnPropDescs25(b));
var escapeStringValueQuotes2 = (v, quoteChar, escapeQuotes) => escapeQuotes && typeof v === "string" ? v.replaceAll(`${quoteChar}`, `${quoteChar}${quoteChar}`) : (
  /* istanbul ignore next */
  v
);
var defaultValueProcessorNL = function(rule) {
  let opts = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  var _a, _b, _c, _d, _e;
  const { escapeQuotes, fields, parseNumbers, quoteFieldNamesWith, quoteValuesWith, fieldIdentifierSeparator, translations } = opts;
  const valueIsField = rule.valueSource === "field";
  const operatorLowerCase = lc(rule.operator);
  const quoteChar = quoteValuesWith || /* istanbul ignore next */
  "'";
  const quoteValue = (v) => `${quoteChar}${v}${quoteChar}`;
  const escapeValue = (v) => escapeStringValueQuotes2(v, quoteChar, escapeQuotes);
  const wrapAndEscape = (v) => quoteValue(escapeValue(v));
  const wrapFieldName = (v) => getQuotedFieldName(v, { quoteFieldNamesWith, fieldIdentifierSeparator });
  const t = translations != null ? translations : (
    /* istanbul ignore next */
    {}
  );
  const orTL = (_a = t.or) != null ? _a : "or";
  const trueTL = (_b = t.true) != null ? _b : "true";
  const falseTL = (_c = t.false) != null ? _c : "false";
  switch (operatorLowerCase) {
    case "null":
    case "notnull": {
      return "";
    }
    case "between":
    case "notbetween": {
      if (!valueIsField) {
        return defaultValueProcessorByRule(rule, opts);
      }
      const valueAsArray = toArray(rule.value, { retainEmptyStrings: true }).slice(0, 2).map((v) => {
        var _a2, _b2;
        return wrapFieldName((_b2 = (_a2 = getOption(fields != null ? fields : (
          /* istanbul ignore next */
          []
        ), v)) == null ? void 0 : _a2.label) != null ? _b2 : v);
      });
      if (valueAsArray.length < 2 || !isValidValue(valueAsArray[0]) || !isValidValue(valueAsArray[1])) {
        return "";
      }
      return defaultValueProcessorByRule(__spreadProps25(__spreadValues26({}, rule), { value: valueAsArray }), opts);
    }
    case "in":
    case "notin": {
      const valueAsArray = toArray(rule.value);
      if (valueAsArray.length === 0) return "";
      const valStringArray = valueAsArray.map((v) => {
        var _a2, _b2;
        return valueIsField ? wrapFieldName((_b2 = (_a2 = getOption(fields != null ? fields : (
          /* istanbul ignore next */
          []
        ), v)) == null ? void 0 : _a2.label) != null ? _b2 : v) : shouldRenderAsNumber(v, parseNumbers) ? `${trimIfString(v)}` : `${wrapAndEscape(v)}`;
      });
      return `${valStringArray.slice(0, -1).join(", ")}${valStringArray.length > 2 ? "," : ""} ${orTL} ${valStringArray.at(-1)}`;
    }
  }
  if (typeof rule.value === "boolean") {
    return rule.value ? trueTL : falseTL;
  }
  return valueIsField ? wrapFieldName((_e = (_d = getOption(fields != null ? fields : (
    /* istanbul ignore next */
    []
  ), rule.value)) == null ? void 0 : _d.label) != null ? _e : rule.value) : shouldRenderAsNumber(rule.value, parseNumbers) ? `${trimIfString(rule.value)}` : `${wrapAndEscape(rule.value)}`;
};

// src/utils/formatQuery/defaultRuleProcessorNL.ts
var __defProp27 = Object.defineProperty;
var __defProps26 = Object.defineProperties;
var __getOwnPropDescs26 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols27 = Object.getOwnPropertySymbols;
var __hasOwnProp27 = Object.prototype.hasOwnProperty;
var __propIsEnum27 = Object.prototype.propertyIsEnumerable;
var __defNormalProp27 = (obj, key, value) => key in obj ? __defProp27(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues27 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp27.call(b, prop)) __defNormalProp27(a, prop, b[prop]);
  if (__getOwnPropSymbols27) for (var prop of __getOwnPropSymbols27(b)) {
    if (__propIsEnum27.call(b, prop)) __defNormalProp27(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps26 = (a, b) => __defProps26(a, __getOwnPropDescs26(b));
var defaultExportOperatorMap = { "=": ["is", "is the same as the value in"], "!=": ["is not", "is not the same as the value in"], "<": ["is less than", "is less than the value in"], ">": ["is greater than", "is greater than the value in"], "<=": ["is less than or equal to", "is less than or equal to the value in"], ">=": ["is greater than or equal to", "is greater than or equal to the value in"], contains: ["contains", "contains the value in"], beginswith: ["starts with", "starts with the value in"], endswith: ["ends with", "ends with the value in"], doesnotcontain: ["does not contain", "does not contain the value in"], doesnotbeginwith: ["does not start with", "does not start with the value in"], doesnotendwith: ["does not end with", "does not end with the value in"], null: "is null", notnull: "is not null", in: ["is one of the values", "is the same as a value in"], notin: ["is not one of the values", "is not the same as any value in"], between: ["is between", "is between the values in"], notbetween: ["is not between", "is not between the values in"] };
var defaultGetOperators = () => [];
var defaultOperatorProcessorNL = function(rule) {
  let opts = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  var _a, _b, _c, _d, _e;
  const { field, operator, valueSource = "value" } = rule;
  const { getOperators = defaultGetOperators, operatorMap: operatorMapParam = defaultExportOperatorMap } = opts;
  const mapOperatorMap = new Map(Object.entries(defaultExportOperatorMap));
  for (const [key, value] of Object.entries(operatorMapParam)) {
    mapOperatorMap.set(lc(key), value);
  }
  const operatorMap = Object.fromEntries(mapOperatorMap);
  const { value: operatorNL, label } = (_c = getOption(toFullOptionList((_b = getOperators(field, { fieldData: (_a = opts.fieldData) != null ? _a : { name: field, value: field, label: field } })) != null ? _b : (
    /* istanbul ignore next */
    []
  )), operator)) != null ? _c : { name: operator, value: operator, label: operator };
  const operatorTL = (_e = (_d = operatorMap[operatorNL]) != null ? _d : operatorMap[lc(operatorNL)]) != null ? _e : [label, label];
  return typeof operatorTL === "string" ? operatorTL : operatorTL[valueSource === "field" ? 1 : 0];
};
var defaultRuleProcessorNL = (rule, opts) => {
  var _a, _b, _c, _d;
  const { field, operator } = rule;
  const { fieldData, quoteFieldNamesWith = ["", ""], fieldIdentifierSeparator = "", quoteValuesWith = `'`, operatorProcessor = defaultOperatorProcessorNL, valueProcessor = defaultValueProcessorNL, concatOperator = "||", wordOrder = "SVO" } = opts != null ? opts : (
    /* istanbul ignore next */
    {}
  );
  const processedField = getQuotedFieldName((_a = fieldData == null ? void 0 : fieldData.label) != null ? _a : field, { quoteFieldNamesWith, fieldIdentifierSeparator });
  const matchEval = processMatchMode(rule);
  if (matchEval === false) {
    return "";
  } else if (matchEval) {
    const { mode, threshold } = matchEval;
    const nestedArrayFilter = defaultRuleGroupProcessorNL(rule.value, __spreadProps26(__spreadValues27({}, opts), { fields: toFullOptionList((_b = fieldData == null ? void 0 : fieldData.subproperties) != null ? _b : []) }));
    const hsp = ((_d = (_c = fieldData == null ? void 0 : fieldData.subproperties) == null ? void 0 : _c.length) != null ? _d : 0) > 0;
    switch (mode) {
      case "all":
        return `(${hsp ? "for " : ""}every item in ${processedField}${hsp ? "," : ""} ${nestedArrayFilter})`;
      case "none":
        return `(${hsp ? "for " : ""}no item in ${processedField}${hsp ? "," : ""} ${nestedArrayFilter})`;
      case "some":
        return `(${hsp ? "for " : ""}at least one item in ${processedField}${hsp ? "," : ""} ${nestedArrayFilter})`;
      case "atleast":
      case "atmost":
      case "exactly": {
        const mm = mode.replace("at", "at ");
        if (threshold > 0 && threshold < 1) {
          return `(${hsp ? "for " : ""}${mm} ${threshold * 100}% of the items in ${processedField}${hsp ? "," : ""} ${nestedArrayFilter})`;
        }
        return `(${hsp ? "for " : ""}${mm} ${threshold} of the items in ${processedField}${hsp ? "," : ""} ${nestedArrayFilter})`;
      }
    }
  }
  const value = valueProcessor(rule, __spreadProps26(__spreadValues27({}, opts), { quoteFieldNamesWith, fieldIdentifierSeparator, quoteValuesWith, concatOperator }));
  const operatorLC = lc(operator);
  if ((operatorLC === "in" || operatorLC === "notin" || operatorLC === "between" || operatorLC === "notbetween") && !value) {
    return "";
  }
  const processedOperator = operatorProcessor(rule, opts);
  const wordOrderMap = { S: processedField, V: processedOperator, O: value };
  return normalizeConstituentWordOrder(wordOrder).map((term) => `${wordOrderMap[term]}`).join(" ").trim();
};

// src/utils/formatQuery/defaultRuleProcessorSQL.ts
var __defProp28 = Object.defineProperty;
var __defProps27 = Object.defineProperties;
var __getOwnPropDescs27 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols28 = Object.getOwnPropertySymbols;
var __hasOwnProp28 = Object.prototype.hasOwnProperty;
var __propIsEnum28 = Object.prototype.propertyIsEnumerable;
var __defNormalProp28 = (obj, key, value) => key in obj ? __defProp28(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues28 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp28.call(b, prop)) __defNormalProp28(a, prop, b[prop]);
  if (__getOwnPropSymbols28) for (var prop of __getOwnPropSymbols28(b)) {
    if (__propIsEnum28.call(b, prop)) __defNormalProp28(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps27 = (a, b) => __defProps27(a, __getOwnPropDescs27(b));
var defaultOperatorProcessorSQL = (rule) => lc(mapSQLOperator(rule.operator));
var defaultRuleProcessorSQL = function(rule) {
  let opts = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  const { quoteFieldNamesWith = ["", ""], fieldIdentifierSeparator = "", quoteValuesWith = `'`, operatorProcessor = defaultOperatorProcessorSQL, valueProcessor = defaultValueProcessorByRule, concatOperator = "||" } = opts;
  const wrapFieldName = (v) => getQuotedFieldName(v, { quoteFieldNamesWith, fieldIdentifierSeparator });
  const ruleField = wrapFieldName(rule.field);
  const matchEval = processMatchMode(rule);
  if (matchEval === false) {
    return;
  } else if (matchEval) {
    if ((opts == null ? void 0 : opts.preset) !== "postgresql") return "";
    const { mode, threshold } = matchEval;
    const arrayElementAlias = "elem_alias";
    const nestedArrayFilter = defaultRuleGroupProcessorSQL(transformQuery(rule.value, { ruleProcessor: (r) => __spreadProps27(__spreadValues28({}, r), { field: arrayElementAlias }) }), opts);
    switch (mode) {
      case "all":
        return `(select count(*) from unnest(${ruleField}) as ${wrapFieldName(arrayElementAlias)} where ${nestedArrayFilter}) = array_length(${ruleField}, 1)`;
      case "none":
        return `not exists (select 1 from unnest(${ruleField}) as ${wrapFieldName(arrayElementAlias)} where ${nestedArrayFilter})`;
      case "some":
        return `exists (select 1 from unnest(${ruleField}) as ${wrapFieldName(arrayElementAlias)} where ${nestedArrayFilter})`;
      case "atleast":
      case "atmost":
      case "exactly": {
        const op = mode === "atleast" ? ">=" : mode === "atmost" ? "<=" : "=";
        return `(select count(*)${threshold > 0 && threshold < 1 ? ` / array_length(${ruleField}, 1)` : ""} from unnest(${ruleField}) as ${wrapFieldName(arrayElementAlias)} where ${nestedArrayFilter}) ${op} ${threshold}`;
      }
    }
  }
  const value = valueProcessor(rule, __spreadProps27(__spreadValues28({}, opts), { quoteFieldNamesWith, fieldIdentifierSeparator, quoteValuesWith, concatOperator }));
  const operator = operatorProcessor(rule, opts);
  const operatorLowerCase = lc(operator);
  if ((operatorLowerCase === "in" || operatorLowerCase === "not in" || operatorLowerCase === "between" || operatorLowerCase === "not between") && !value) {
    return "";
  }
  return `${ruleField} ${operator} ${value}`.trim();
};

// src/utils/formatQuery/defaultRuleProcessorParameterized.ts
var __defProp29 = Object.defineProperty;
var __defProps28 = Object.defineProperties;
var __getOwnPropDescs28 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols29 = Object.getOwnPropertySymbols;
var __hasOwnProp29 = Object.prototype.hasOwnProperty;
var __propIsEnum29 = Object.prototype.propertyIsEnumerable;
var __defNormalProp29 = (obj, key, value) => key in obj ? __defProp29(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues29 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp29.call(b, prop)) __defNormalProp29(a, prop, b[prop]);
  if (__getOwnPropSymbols29) for (var prop of __getOwnPropSymbols29(b)) {
    if (__propIsEnum29.call(b, prop)) __defNormalProp29(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps28 = (a, b) => __defProps28(a, __getOwnPropDescs28(b));
var defaultRuleProcessorParameterized = (rule, opts, meta) => {
  const { fieldData, format, getNextNamedParam, parseNumbers, paramPrefix, paramsKeepPrefix, numberedParams, quoteFieldNamesWith = ["", ""], fieldIdentifierSeparator, concatOperator, operatorProcessor = defaultOperatorProcessorSQL, valueProcessor = defaultValueProcessorByRule } = opts != null ? opts : {};
  const { processedParams = [] } = meta != null ? meta : {};
  const parameterized = format === "parameterized";
  const params = [];
  const paramsNamed = {};
  const finalize = (sql) => parameterized ? { sql, params } : { sql, params: paramsNamed };
  const wrapFieldName = (v) => getQuotedFieldName(v, { quoteFieldNamesWith, fieldIdentifierSeparator });
  const ruleField = wrapFieldName(rule.field);
  const matchEval = processMatchMode(rule);
  if (matchEval === false) {
    return;
  } else if (matchEval) {
    if ((opts == null ? void 0 : opts.preset) !== "postgresql") return finalize("");
    const { mode, threshold } = matchEval;
    const arrayElementAlias = "elem_alias";
    const { sql: nestedSQL, params: nestedParams } = defaultRuleGroupProcessorParameterized(transformQuery(rule.value, { ruleProcessor: (r) => __spreadProps28(__spreadValues29({}, r), { field: arrayElementAlias }) }), __spreadProps28(__spreadValues29({}, opts), { fields: [] }));
    if (Array.isArray(nestedParams)) {
      params.push(...nestedParams);
    } else {
      Object.assign(paramsNamed, nestedParams);
    }
    switch (mode) {
      case "all":
        return finalize(`(select count(*) from unnest(${ruleField}) as ${wrapFieldName(arrayElementAlias)} where ${nestedSQL}) = array_length(${ruleField}, 1)`);
      case "none":
        return finalize(`not exists (select 1 from unnest(${ruleField}) as ${wrapFieldName(arrayElementAlias)} where ${nestedSQL})`);
      case "some":
        return finalize(`exists (select 1 from unnest(${ruleField}) as ${wrapFieldName(arrayElementAlias)} where ${nestedSQL})`);
      case "atleast":
      case "atmost":
      case "exactly": {
        const op = mode === "atleast" ? ">=" : mode === "atmost" ? "<=" : "=";
        return finalize(`(select count(*)${threshold > 0 && threshold < 1 ? ` / array_length(${ruleField}, 1)` : ""} from unnest(${ruleField}) as ${wrapFieldName(arrayElementAlias)} where ${nestedSQL}) ${op} ${threshold}`);
      }
    }
  }
  const value = valueProcessor(rule, { parseNumbers, quoteFieldNamesWith, concatOperator, fieldData, format });
  const sqlOperator = operatorProcessor(rule, opts);
  const sqlOperatorLowerCase = lc(sqlOperator);
  const [qPre, qPost] = quoteFieldNamesWith;
  if ((sqlOperatorLowerCase === "in" || sqlOperatorLowerCase === "not in" || sqlOperatorLowerCase === "between" || sqlOperatorLowerCase === "not between") && !value) {
    return finalize("");
  } else if (sqlOperatorLowerCase === "is null" || sqlOperatorLowerCase === "is not null") {
    return finalize(`${qPre}${rule.field}${qPost} ${sqlOperator}`);
  } else if (rule.valueSource === "field") {
    return finalize(`${qPre}${rule.field}${qPost} ${sqlOperator} ${value}`.trim());
  } else if (sqlOperatorLowerCase === "in" || sqlOperatorLowerCase === "not in") {
    const splitValue = toArray(rule.value);
    if (parameterized) {
      for (const v of splitValue) {
        params.push(shouldRenderAsNumber(v, parseNumbers) ? parseNumber(v, { parseNumbers }) : v);
      }
      return finalize(`${qPre}${rule.field}${qPost} ${sqlOperator} (${splitValue.map((_v, i) => numberedParams ? `${paramPrefix}${processedParams.length + 1 + splitValue.length - (splitValue.length - i)}` : "?").join(", ")})`);
    }
    const inParams = [];
    for (const v of splitValue) {
      const thisParamName = getNextNamedParam(rule.field);
      inParams.push(`${paramPrefix}${thisParamName}`);
      paramsNamed[`${paramsKeepPrefix ? paramPrefix : ""}${thisParamName}`] = shouldRenderAsNumber(v, parseNumbers) ? parseNumber(v, { parseNumbers }) : v;
    }
    return finalize(`${qPre}${rule.field}${qPost} ${sqlOperator} (${inParams.join(", ")})`);
  } else if (sqlOperatorLowerCase === "between" || sqlOperatorLowerCase === "not between") {
    const valueAsArray = toArray(rule.value, { retainEmptyStrings: true });
    const [first, second] = valueAsArray.slice(0, 2).map((v) => shouldRenderAsNumber(v, parseNumbers) ? parseNumber(v, { parseNumbers }) : v);
    if (parameterized) {
      params.push(first, second);
      return finalize(`${qPre}${rule.field}${qPost} ${sqlOperator} ${numberedParams ? `${paramPrefix}${processedParams.length + 1}` : "?"} and ${numberedParams ? `${paramPrefix}${processedParams.length + 2}` : "?"}`);
    }
    const firstParamName = getNextNamedParam(rule.field);
    const secondParamName = getNextNamedParam(rule.field);
    paramsNamed[`${paramsKeepPrefix ? paramPrefix : ""}${firstParamName}`] = first;
    paramsNamed[`${paramsKeepPrefix ? paramPrefix : ""}${secondParamName}`] = second;
    return finalize(`${qPre}${rule.field}${qPost} ${sqlOperator} ${paramPrefix}${firstParamName} and ${paramPrefix}${secondParamName}`);
  }
  let paramValue = rule.value;
  if (typeof rule.value === "string") {
    if (shouldRenderAsNumber(rule.value, parseNumbers)) {
      paramValue = parseNumber(rule.value, { parseNumbers });
    } else {
      paramValue = /^'.*'$/g.test(value) ? value.replaceAll(/(^'|'$)/g, "") : (
        /* istanbul ignore next */
        value
      );
    }
  }
  let paramName = "";
  if (parameterized) {
    params.push(paramValue);
  } else {
    paramName = getNextNamedParam(rule.field);
    paramsNamed[`${paramsKeepPrefix ? paramPrefix : ""}${paramName}`] = paramValue;
  }
  return finalize(`${qPre}${rule.field}${qPost} ${sqlOperator} ${parameterized ? numberedParams ? `${paramPrefix}${processedParams.length + 1}` : "?" : `${paramPrefix}${paramName}`}`.trim());
};

// src/utils/formatQuery/defaultRuleProcessorPrisma.ts
var processNumber2 = (value, fallback, parseNumbers) => shouldRenderAsNumber(value, !!parseNumbers || typeof value === "bigint") ? Number(parseNumber(value, { parseNumbers: !!parseNumbers })) : fallback;
var defaultRuleProcessorPrisma = function(rule) {
  let options = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  const { field, operator, value, valueSource } = rule;
  const { parseNumbers, preserveValueOrder } = options;
  if (valueSource === "field" || processMatchMode(rule)) return;
  const operatorLC = lc(operator);
  switch (operatorLC) {
    case "=":
      return { [field]: processNumber2(value, value, parseNumbers) };
    case "!=":
    case "<":
    case "<=":
    case ">":
    case ">=": {
      const prismaOperator = prismaOperators[operatorLC];
      return { [field]: { [prismaOperator]: processNumber2(value, value, parseNumbers) } };
    }
    case "contains":
      return { [field]: { contains: value } };
    case "beginswith":
      return { [field]: { startsWith: value } };
    case "endswith":
      return { [field]: { endsWith: value } };
    case "doesnotcontain":
      return { NOT: { [field]: { contains: value } } };
    case "doesnotbeginwith":
      return { NOT: { [field]: { startsWith: value } } };
    case "doesnotendwith":
      return { NOT: { [field]: { endsWith: value } } };
    case "null":
      return { [field]: null };
    case "notnull":
      return { [field]: { not: null } };
    case "in":
    case "notin": {
      const valueAsArray = toArray(value);
      return { [field]: { [prismaOperators[operatorLC]]: valueAsArray.map((val) => processNumber2(val, val, parseNumbers)) } };
    }
    case "between":
    case "notbetween": {
      const valueAsArray = toArray(value);
      if (valueAsArray.length >= 2 && isValidValue(valueAsArray[0]) && isValidValue(valueAsArray[1])) {
        const [first, second] = valueAsArray;
        const shouldParseNumbers = !(parseNumbers === false);
        const firstNum = shouldRenderAsNumber(first, shouldParseNumbers) ? parseNumber(first, { parseNumbers }) : Number.NaN;
        const secondNum = shouldRenderAsNumber(second, shouldParseNumbers) ? parseNumber(second, { parseNumbers }) : Number.NaN;
        let firstValue = Number.isNaN(firstNum) ? first : firstNum;
        let secondValue = Number.isNaN(secondNum) ? second : secondNum;
        if (!preserveValueOrder && firstValue === firstNum && secondValue === secondNum && secondNum < firstNum) {
          const tempNum = secondNum;
          secondValue = firstNum;
          firstValue = tempNum;
        }
        return operatorLC === "between" ? { [field]: { gte: firstValue, lte: secondValue } } : { OR: [{ [field]: { lt: firstValue } }, { [field]: { gt: secondValue } }] };
      } else {
        return "";
      }
    }
  }
  return "";
};

// src/utils/formatQuery/defaultRuleProcessorSequelize.ts
var defaultRuleProcessorSequelize = function(rule) {
  let { parseNumbers, preserveValueOrder, context = {} } = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  const { sequelizeOperators: Op, sequelizeCol: col, sequelizeFn: fn } = context;
  if (processMatchMode(rule)) return;
  const { field, operator, value, valueSource } = rule;
  const valueIsField = valueSource === "field";
  const operatorLC = lc(operator);
  if (
    // Bail out if we don't have the Op symbols
    !Op || // ...or valueSource is 'field' and we don't have the `col` function,
    valueIsField && (!col || // ...or valueSource is 'field' and we don't have the `fn` function
    // and the operator is one of the "doesNot*" ones
    !fn && ["doesnotcontain", "doesnotbeginwith", "doesnotendwith"].includes(operatorLC))
  ) {
    return;
  }
  switch (operatorLC) {
    case "=":
    case "!=":
    case "<":
    case "<=":
    case ">":
    case ">=": {
      const sequelizeOperator = { "=": Op.eq, "!=": Op.ne, "<": Op.lt, "<=": Op.lte, ">": Op.gt, ">=": Op.gte }[operatorLC];
      return { [field]: valueIsField && operatorLC === "=" ? { [Op.col]: value } : { [sequelizeOperator]: valueIsField ? col(value) : shouldRenderAsNumber(value, parseNumbers) ? parseNumber(value, { parseNumbers: "strict" }) : value } };
    }
    case "contains":
      return { [field]: { [Op.substring]: valueIsField ? col(value) : `${value}` } };
    case "beginswith":
      return { [field]: { [Op.startsWith]: valueIsField ? col(value) : `${value}` } };
    case "endswith":
      return { [field]: { [Op.endsWith]: valueIsField ? col(value) : `${value}` } };
    case "doesnotcontain":
      return { [field]: { [Op.notLike]: valueIsField ? fn("CONCAT", "%", col(value), "%") : `%${value}%` } };
    case "doesnotbeginwith":
      return { [field]: { [Op.notLike]: valueIsField ? fn("CONCAT", col(value), "%") : `${value}%` } };
    case "doesnotendwith":
      return { [field]: { [Op.notLike]: valueIsField ? fn("CONCAT", "%", col(value)) : `%${value}` } };
    case "null":
      return { [field]: { [Op.is]: null } };
    case "notnull":
      return { [field]: { [Op.not]: null } };
    case "in":
    case "notin": {
      const valueAsArray = toArray(value);
      return { [field]: { [operatorLC === "in" ? Op.in : Op.notIn]: valueAsArray.map((val) => valueIsField ? col(val) : shouldRenderAsNumber(val, parseNumbers) ? parseNumber(val, { parseNumbers: "strict" }) : val) } };
    }
    case "between":
    case "notbetween": {
      const valueAsArray = toArray(value, { retainEmptyStrings: true });
      if (valueAsArray.length < 2 || !isValidValue(valueAsArray[0]) || !isValidValue(valueAsArray[1])) {
        return;
      }
      const [first, second] = valueAsArray;
      const firstNum = shouldRenderAsNumber(first, parseNumbers) ? parseNumber(first, { parseNumbers: "strict" }) : Number.NaN;
      const secondNum = shouldRenderAsNumber(second, parseNumbers) ? parseNumber(second, { parseNumbers: "strict" }) : Number.NaN;
      const firstValue = Number.isNaN(firstNum) ? first : firstNum;
      const secondValue = Number.isNaN(secondNum) ? second : secondNum;
      const valsOneAndTwoOnly = [firstValue, secondValue];
      if (!preserveValueOrder && firstValue === firstNum && secondValue === secondNum && secondNum < firstNum) {
        valsOneAndTwoOnly[0] = secondNum;
        valsOneAndTwoOnly[1] = firstNum;
      }
      return { [field]: { [operatorLC === "between" ? Op.between : Op.notBetween]: valueIsField ? valsOneAndTwoOnly.map((v) => col(v)) : valsOneAndTwoOnly.every((v) => shouldRenderAsNumber(v, parseNumbers)) ? valsOneAndTwoOnly.map((v) => parseNumber(v, { parseNumbers: "strict" })) : valsOneAndTwoOnly } };
    }
  }
  return;
};

// src/utils/formatQuery/formatQuery.ts
import { produce as produce4 } from "immer";
var __defProp30 = Object.defineProperty;
var __defProps29 = Object.defineProperties;
var __getOwnPropDescs29 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols30 = Object.getOwnPropertySymbols;
var __hasOwnProp30 = Object.prototype.hasOwnProperty;
var __propIsEnum30 = Object.prototype.propertyIsEnumerable;
var __defNormalProp30 = (obj, key, value) => key in obj ? __defProp30(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues30 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp30.call(b, prop)) __defNormalProp30(a, prop, b[prop]);
  if (__getOwnPropSymbols30) for (var prop of __getOwnPropSymbols30(b)) {
    if (__propIsEnum30.call(b, prop)) __defNormalProp30(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps29 = (a, b) => __defProps29(a, __getOwnPropDescs29(b));
var sqlDialectPresets = {
  ansi: {},
  // This should always be empty
  sqlite: { paramsKeepPrefix: true },
  oracle: {},
  mssql: { concatOperator: "+", quoteFieldNamesWith: ["[", "]"], fieldIdentifierSeparator: ".", paramPrefix: "@" },
  mysql: { concatOperator: "CONCAT" },
  postgresql: { quoteFieldNamesWith: '"', numberedParams: true, paramPrefix: "$" }
};
var formatQueryOptionPresets = __spreadValues30({}, sqlDialectPresets);
var defaultRuleProcessors = { cel: defaultRuleProcessorCEL, drizzle: defaultRuleProcessorDrizzle, elasticsearch: defaultRuleProcessorElasticSearch, json_without_ids: defaultRuleProcessorSQL, json: defaultRuleProcessorSQL, jsonata: defaultRuleProcessorJSONata, jsonlogic: defaultRuleProcessorJsonLogic, ldap: defaultRuleProcessorLDAP, mongodb_query: defaultRuleProcessorMongoDBQuery, mongodb: defaultRuleProcessorMongoDB, natural_language: defaultRuleProcessorNL, parameterized_named: defaultRuleProcessorParameterized, parameterized: defaultRuleProcessorParameterized, prisma: defaultRuleProcessorPrisma, sequelize: defaultRuleProcessorSequelize, spel: defaultRuleProcessorSpEL, sql: defaultRuleProcessorSQL };
var defaultOperatorProcessor = (r) => r.operator;
var defaultOperatorProcessors = { cel: defaultOperatorProcessor, drizzle: defaultOperatorProcessor, elasticsearch: defaultOperatorProcessor, json_without_ids: defaultOperatorProcessor, json: defaultOperatorProcessor, jsonata: defaultOperatorProcessor, jsonlogic: defaultOperatorProcessor, ldap: defaultOperatorProcessor, mongodb_query: defaultOperatorProcessor, mongodb: defaultOperatorProcessor, natural_language: defaultOperatorProcessorNL, parameterized_named: defaultOperatorProcessorSQL, parameterized: defaultOperatorProcessorSQL, prisma: defaultOperatorProcessor, sequelize: defaultOperatorProcessor, spel: defaultOperatorProcessor, sql: defaultOperatorProcessorSQL };
var defaultFallbackExpressions = { cel: "1 == 1", ldap: "", mongodb: '"$and":[{"$expr":true}]', natural_language: "1 is 1", spel: "1 == 1", sql: "(1 = 1)" };
var defaultFormatQueryOptions = { format: "json", fields: [], quoteFieldNamesWith: ["", ""], fieldIdentifierSeparator: "", getOperators: () => [], paramPrefix: ":", paramsKeepPrefix: false, numberedParams: false, preserveValueOrder: false, placeholderFieldName: defaultPlaceholderFieldName, placeholderOperatorName: defaultPlaceholderOperatorName, quoteValuesWith: "'", concatOperator: "||", preset: "ansi", wordOrder: "SVO", translations: {}, operatorMap: {} };
var valueProcessorCanActAsRuleProcessor = /* @__PURE__ */ new Set(["cel", "drizzle", "elasticsearch", "jsonata", "jsonlogic", "ldap", "mongodb_query", "mongodb", "prisma", "sequelize", "spel"]);
var sqlFormats = /* @__PURE__ */ new Set(["sql", "parameterized", "parameterized_named", "drizzle", "prisma", "sequelize"]);
function formatQuery(ruleGroup) {
  let optionParam = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  var _a, _b, _c, _d, _e, _f, _g, _h, _i;
  const options = typeof optionParam === "string" ? { format: lc(optionParam) } : optionParam;
  const optObj = __spreadValues30(__spreadValues30(__spreadValues30(__spreadValues30({}, defaultFormatQueryOptions), !options.format || sqlFormats.has(options.format) ? (_b = sqlDialectPresets[(_a = options.preset) != null ? _a : "ansi"]) != null ? _b : null : null), options), !options.format && (Object.keys(sqlDialectPresets).includes((_c = options.preset) != null ? _c : "") ? { format: "sql" } : null));
  const format = lc(optObj.format);
  const { fallbackExpression: fallbackExpression_option, getOperators: getOperators_option, operatorProcessor: operatorProcessor_option, parseNumbers, quoteFieldNamesWith: quoteFieldNamesWith_option, ruleGroupProcessor: ruleGroupProcessor_option, ruleProcessor: ruleProcessor_option, validator, valueProcessor: valueProcessor_option, context } = optObj;
  const getParseNumberBoolean = (inputType) => {
    const parseNumberMethod = getParseNumberMethod({ parseNumbers, inputType });
    return typeof parseNumberMethod === "string" ? true : typeof parseNumbers === "boolean" ? parseNumbers : void 0;
  };
  const operatorProcessor = typeof operatorProcessor_option === "function" ? operatorProcessor_option : (_d = defaultOperatorProcessors[format]) != null ? _d : defaultOperatorProcessor;
  const valueProcessor = typeof valueProcessor_option === "function" ? isValueProcessorLegacy(valueProcessor_option) ? (r) => valueProcessor_option(r.field, r.operator, r.value, r.valueSource) : valueProcessor_option : format === "natural_language" ? defaultValueProcessorNL : valueProcessorCanActAsRuleProcessor.has(format) ? ruleProcessor_option != null ? ruleProcessor_option : defaultRuleProcessors[format] : defaultValueProcessorByRule;
  const ruleProcessor = (_g = (_f = (_e = typeof ruleProcessor_option === "function" ? ruleProcessor_option : null) != null ? _e : valueProcessorCanActAsRuleProcessor.has(format) && typeof ruleProcessor_option !== "function" && valueProcessor_option ? valueProcessor : null) != null ? _f : defaultRuleProcessors[format]) != null ? _g : defaultRuleProcessorSQL;
  const quoteFieldNamesWith = getQuoteFieldNamesWithArray(quoteFieldNamesWith_option);
  const fields = toFullOptionList(optObj.fields);
  const getOperators = (f, m) => {
    var _a2;
    return toFullOptionList((_a2 = getOperators_option(f, m)) != null ? _a2 : (
      /* istanbul ignore next */
      []
    ));
  };
  const fallbackExpression = (_h = fallbackExpression_option != null ? fallbackExpression_option : defaultFallbackExpressions[format]) != null ? _h : defaultFallbackExpressions.sql;
  let validationMap = {};
  if (typeof validator === "function") {
    const validationResult = validator(ruleGroup);
    if (typeof validationResult === "boolean") {
      if (validationResult === false) {
        return format === "parameterized" ? { sql: fallbackExpression, params: [] } : format === "parameterized_named" ? { sql: fallbackExpression, params: {} } : format === "mongodb" ? `{${fallbackExpression}}` : format === "mongodb_query" ? mongoDbFallback : format === "prisma" ? prismaFallback : format === "jsonlogic" ? false : format === "elasticsearch" ? {} : format === "drizzle" || format === "sequelize" ? void 0 : fallbackExpression;
      }
    } else {
      validationMap = validationResult;
    }
  }
  const validatorMap = {};
  const uniqueFields = toFlatOptionArray(fields);
  for (const f of uniqueFields) {
    if (typeof f.validator === "function") {
      validatorMap[(_i = f.value) != null ? _i : (
        /* istanbul ignore next */
        f.name
      )] = f.validator;
    }
  }
  const validateRule = (rule) => {
    let validationResult;
    let fieldValidator;
    if (rule.id) {
      validationResult = validationMap[rule.id];
    }
    if (uniqueFields.length > 0) {
      const fieldArr = uniqueFields.filter((f) => f.name === rule.field);
      if (fieldArr.length > 0) {
        const field = fieldArr[0];
        if (typeof field.validator === "function") {
          fieldValidator = field.validator;
        }
      }
    }
    return [validationResult, fieldValidator];
  };
  const finalOptions = __spreadProps29(__spreadValues30({}, optObj), { fallbackExpression, fields, format, getOperators, getParseNumberBoolean, quoteFieldNamesWith, operatorProcessor, ruleProcessor, valueProcessor, validateRule, validationMap, context });
  if (typeof ruleGroupProcessor_option === "function") {
    return ruleGroupProcessor_option(ruleGroup, finalOptions);
  }
  switch (format) {
    case "json":
    case "json_without_ids": {
      const rg = parseNumbers ? produce4(ruleGroup, (g) => numerifyValues(g, finalOptions)) : ruleGroup;
      if (format === "json_without_ids") {
        return JSON.stringify(rg, (key, value) => (
          // Remove `id` and `path` keys; leave everything else unchanged.
          key === "id" || key === "path" ? void 0 : bigIntJsonStringifyReplacer(key, value)
        ));
      }
      return JSON.stringify(rg, bigIntJsonStringifyReplacer, 2);
    }
    case "sql":
      return defaultRuleGroupProcessorSQL(ruleGroup, finalOptions);
    case "parameterized":
    case "parameterized_named":
      return defaultRuleGroupProcessorParameterized(ruleGroup, finalOptions);
    case "mongodb":
      return defaultRuleGroupProcessorMongoDB(ruleGroup, finalOptions);
    case "mongodb_query":
      return defaultRuleGroupProcessorMongoDBQuery(ruleGroup, finalOptions);
    case "cel":
      return defaultRuleGroupProcessorCEL(ruleGroup, finalOptions);
    case "spel":
      return defaultRuleGroupProcessorSpEL(ruleGroup, finalOptions);
    case "jsonata":
      return defaultRuleGroupProcessorJSONata(ruleGroup, finalOptions);
    case "jsonlogic":
      return defaultRuleGroupProcessorJsonLogic(ruleGroup, finalOptions);
    case "elasticsearch":
      return defaultRuleGroupProcessorElasticSearch(ruleGroup, finalOptions);
    case "natural_language":
      return defaultRuleGroupProcessorNL(ruleGroup, finalOptions);
    case "ldap":
      return defaultRuleGroupProcessorLDAP(ruleGroup, finalOptions);
    case "prisma":
      return defaultRuleGroupProcessorPrisma(ruleGroup, finalOptions);
    case "drizzle":
      return defaultRuleGroupProcessorDrizzle(ruleGroup, finalOptions);
    case "sequelize":
      return defaultRuleGroupProcessorSequelize(ruleGroup, finalOptions);
    default:
      return "";
  }
}

// src/utils/formatQuery/index.ts
var generateValueProcessor = (vpbr) => (field, operator, value, valueSource) => vpbr({ field, operator, value, valueSource }, { parseNumbers: false });
var defaultValueProcessor = generateValueProcessor(defaultValueProcessorByRule);
var defaultMongoDBValueProcessor = generateValueProcessor(defaultRuleProcessorMongoDB);
var defaultCELValueProcessor = generateValueProcessor(defaultRuleProcessorCEL);
var defaultSpELValueProcessor = generateValueProcessor(defaultRuleProcessorSpEL);
var defaultValueProcessorCELByRule = defaultRuleProcessorCEL;
var defaultValueProcessorMongoDBByRule = defaultRuleProcessorMongoDB;
var defaultValueProcessorSpELByRule = defaultRuleProcessorSpEL;

// src/utils/pathUtils.ts
var findPath = (path, query) => {
  let target = query;
  let level = 0;
  while (level < path.length && target && isRuleGroup(target)) {
    const t = target.rules[path[level]];
    target = typeof t === "string" ? null : t;
    level++;
  }
  return target;
};
var findID = (id, query) => {
  if (query.id === id) {
    return query;
  }
  for (const rule of query.rules) {
    if (typeof rule === "string") continue;
    if (rule.id === id) {
      return rule;
    } else if (isRuleGroup(rule)) {
      const subRule = findID(id, rule);
      if (subRule) {
        return subRule;
      }
    }
  }
  return null;
};
var getPathOfID = (id, query) => {
  if (query.id === id) return [];
  const idx = query.rules.findIndex((r) => !(typeof r === "string") && r.id === id);
  if (idx >= 0) {
    return [idx];
  }
  for (const [i, r] of Object.entries(query.rules)) {
    if (isRuleGroup(r)) {
      const subPath = getPathOfID(id, r);
      if (Array.isArray(subPath)) {
        return [Number.parseInt(i), ...subPath];
      }
    }
  }
  return null;
};
var getParentPath = (path) => path.slice(0, -1);
var pathsAreEqual = (path1, path2) => path1.length === path2.length && path1.every((val, idx) => val === path2[idx]);
var isAncestor = (maybeAncestor, path) => maybeAncestor.length < path.length && new RegExp(`^${maybeAncestor.join("-")}`).test(path.join("-"));
var getCommonAncestorPath = (path1, path2) => {
  const commonAncestorPath = [];
  const parentPath1 = getParentPath(path1);
  const parentPath2 = getParentPath(path2);
  let i = 0;
  while (i < parentPath1.length && i < parentPath2.length && parentPath1[i] === parentPath2[i]) {
    commonAncestorPath.push(parentPath2[i]);
    i++;
  }
  return commonAncestorPath;
};
var pathIsDisabled = (path, query) => {
  let disabled = !!query.disabled;
  let target = query;
  let level = 0;
  while (level < path.length && !disabled && isRuleGroup(target)) {
    const t = target.rules[path[level]];
    if (isPojo(t) && (isRuleGroup(t) || "field" in t && !!t.field)) {
      disabled = !!t.disabled;
      target = t;
    }
    level++;
  }
  return disabled;
};

// src/utils/generateAccessibleDescription.ts
var generateAccessibleDescription = (params) => pathsAreEqual([], params.path) ? `Query builder` : `Rule group at path ${params.path.join("-")}`;

// src/utils/generateID.ts
var cryptoModule = globalThis.crypto;
var generateID = () => "00-0-4-2-000".replaceAll(/[^-]/g, (s) => ((Math.random() + Math.trunc(s)) * 65536 >> Number.parseInt(s)).toString(16).padStart(4, "0"));
if (cryptoModule) {
  if (typeof cryptoModule.randomUUID === "function") {
    generateID = () => cryptoModule.randomUUID();
  } else if (typeof cryptoModule.getRandomValues === "function") {
    const position19vals = "89ab";
    const container = new Uint32Array(32);
    generateID = () => {
      cryptoModule.getRandomValues(container);
      let id = (container[0] % 16).toString(16);
      for (let i = 1; i < 32; i++) {
        if (i === 12) {
          id = `${id}${"4"}`;
        } else if (i === 16) {
          id = `${id}${position19vals[container[17] % 4]}`;
        } else {
          id = `${id}${(container[i] % 16).toString(16)}`;
        }
        if (i === 7 || i === 11 || i === 15 || i === 19) {
          id = `${id}${"-"}`;
        }
      }
      return id;
    };
  }
}

// src/hooks/usePreferProp.ts
var preferPropDefaultTrue = (prop, context) => prop === false ? false : prop ? true : !(context === false);
var preferPropDefaultFalse = (prop, context) => prop ? true : prop === false ? false : !!context;
var preferProp = (def, prop, context) => def ? preferPropDefaultTrue(prop, context) : preferPropDefaultFalse(prop, context);
var usePreferProp = (def, prop, context) => preferProp(def, prop, context);
var usePreferAnyProp = (def, prop, context) => prop !== void 0 && prop != null ? prop : context !== void 0 && context != null ? context : def;

// src/hooks/useMergedContext.ts
import { forwardRef as forwardRef2, useCallback as useCallback9, useContext as useContext2, useMemo as useMemo10 } from "react";

// ../../utils/react-compiler/react-compiler-runtime.ts
import * as React from "react";
var $empty = Symbol.for("react.memo_cache_sentinel");
function c(size) {
  return React.useMemo(
    () => {
      const $ = new Array(size);
      for (let ii = 0; ii < size; ii++) {
        $[ii] = $empty;
      }
      $[$empty] = true;
      return $;
    },
    // oxlint-disable-next-line exhaustive-deps
    []
  );
}

// src/components/MatchModeEditor.tsx
import { jsx as jsx6, jsxs } from "react/jsx-runtime";
import * as React2 from "react";
import { useCallback } from "react";

// src/utils/getCompatContextProvider.tsx
import { jsx as jsx4 } from "react/jsx-runtime";
var __defProp31 = Object.defineProperty;
var __defProps30 = Object.defineProperties;
var __getOwnPropDescs30 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols31 = Object.getOwnPropertySymbols;
var __hasOwnProp31 = Object.prototype.hasOwnProperty;
var __propIsEnum31 = Object.prototype.propertyIsEnumerable;
var __defNormalProp31 = (obj, key, value) => key in obj ? __defProp31(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues31 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp31.call(b, prop)) __defNormalProp31(a, prop, b[prop]);
  if (__getOwnPropSymbols31) for (var prop of __getOwnPropSymbols31(b)) {
    if (__propIsEnum31.call(b, prop)) __defNormalProp31(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps30 = (a, b) => __defProps30(a, __getOwnPropDescs30(b));
var getCompatContextProvider = (gccpProps) => {
  const QBContextWrapper = (props) => {
    const $ = c(3);
    const rqbContext = useMergedContext(gccpProps);
    let t0;
    if ($[0] !== props.children || $[1] !== rqbContext) {
      t0 = jsx4(QueryBuilderContext.Provider, { value: rqbContext, children: props.children });
      $[0] = props.children;
      $[1] = rqbContext;
      $[2] = t0;
    } else {
      t0 = $[2];
    }
    return t0;
  };
  const QBContextInner = (props) => {
    const $ = c(3);
    const rqbContext = useMergedContext(props);
    let t0;
    if ($[0] !== props.children || $[1] !== rqbContext) {
      t0 = jsx4(QueryBuilderContext.Provider, { value: rqbContext, children: props.children });
      $[0] = props.children;
      $[1] = rqbContext;
      $[2] = t0;
    } else {
      t0 = $[2];
    }
    return t0;
  };
  return (props) => {
    return /* @__PURE__ */ jsx4(QBContextWrapper, { children: /* @__PURE__ */ jsx4(QBContextInner, __spreadProps30(__spreadValues31({}, props), { children: props.children })) });
  };
};

// src/utils/getMatchModesUtil.ts
var dummyFD = { name: "name", value: "name", matchModes: null, label: "label" };
var getMatchModesUtil = (fieldData, getMatchModes) => {
  var _a, _b;
  const fd = fieldData ? toFullOption(fieldData) : (
    /* istanbul ignore next */
    dummyFD
  );
  let matchModes = (_a = fd.matchModes) != null ? _a : false;
  if (!matchModes && getMatchModes) {
    matchModes = getMatchModes(fd.value, { fieldData: fd });
  }
  if (matchModes === true) {
    return defaultMatchModes;
  } else if (matchModes === false) {
    return [];
  }
  if (isFlexibleOptionArray(matchModes)) {
    return toFullOptionList(matchModes);
  }
  return (_b = matchModes == null ? void 0 : matchModes.map((mm) => {
    var _a2;
    return (_a2 = defaultMatchModes.find((dmm) => dmm.value === lc(mm))) != null ? _a2 : { name: mm, value: mm, label: mm };
  })) != null ? _b : [];
};

// src/utils/getValidationClassNames.ts
var getValidationClassNames = (validationResult) => {
  const valid = typeof validationResult === "boolean" ? validationResult : typeof validationResult === "object" && validationResult !== null ? validationResult.valid : null;
  return typeof valid === "boolean" ? valid ? standardClassnames.valid : standardClassnames.invalid : "";
};

// src/utils/getValueSourcesUtil.ts
var defaultValueSourcesArray = [{ name: "value", value: "value", label: "value" }];
var dummyFD2 = { name: "name", value: "name", valueSources: null, label: "label" };
var getValueSourcesUtil = (fieldData, operator, getValueSources) => {
  var _a;
  const fd = fieldData ? toFullOption(fieldData) : (
    /* istanbul ignore else */
    dummyFD2
  );
  let valueSourcesNEW = (_a = fd.valueSources) != null ? _a : false;
  if (typeof valueSourcesNEW === "function") {
    valueSourcesNEW = valueSourcesNEW(operator);
  }
  if (!valueSourcesNEW && getValueSources) {
    valueSourcesNEW = getValueSources(fd.value, operator, { fieldData: fd });
  }
  if (!valueSourcesNEW) {
    return defaultValueSourcesArray;
  }
  if (isFlexibleOptionArray(valueSourcesNEW)) {
    return toFullOptionList(valueSourcesNEW);
  }
  return valueSourcesNEW.map((vs) => {
    var _a2;
    return (_a2 = defaultValueSourcesArray.find((dmm) => dmm.value === lc(vs))) != null ? _a2 : { name: vs, value: vs, label: vs };
  });
};

// src/utils/mergeClassnames.ts
var joinClassnamesByName = (name, args) => clsx(args.map((c2) => clsx(c2 == null ? void 0 : c2[name])));
var mergeClassnames = function() {
  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }
  return { queryBuilder: joinClassnamesByName("queryBuilder", args), ruleGroup: joinClassnamesByName("ruleGroup", args), header: joinClassnamesByName("header", args), body: joinClassnamesByName("body", args), combinators: joinClassnamesByName("combinators", args), addRule: joinClassnamesByName("addRule", args), addGroup: joinClassnamesByName("addGroup", args), cloneRule: joinClassnamesByName("cloneRule", args), cloneGroup: joinClassnamesByName("cloneGroup", args), removeGroup: joinClassnamesByName("removeGroup", args), rule: joinClassnamesByName("rule", args), fields: joinClassnamesByName("fields", args), operators: joinClassnamesByName("operators", args), value: joinClassnamesByName("value", args), removeRule: joinClassnamesByName("removeRule", args), notToggle: joinClassnamesByName("notToggle", args), shiftActions: joinClassnamesByName("shiftActions", args), dragHandle: joinClassnamesByName("dragHandle", args), lockRule: joinClassnamesByName("lockRule", args), lockGroup: joinClassnamesByName("lockGroup", args), valueSource: joinClassnamesByName("valueSource", args), actionElement: joinClassnamesByName("actionElement", args), valueSelector: joinClassnamesByName("valueSelector", args), betweenRules: joinClassnamesByName("betweenRules", args), valid: joinClassnamesByName("valid", args), invalid: joinClassnamesByName("invalid", args), dndDragging: joinClassnamesByName("dndDragging", args), dndOver: joinClassnamesByName("dndOver", args), dndCopy: joinClassnamesByName("dndCopy", args), dndGroup: joinClassnamesByName("dndGroup", args), disabled: joinClassnamesByName("disabled", args), valueListItem: joinClassnamesByName("valueListItem", args), matchMode: joinClassnamesByName("matchMode", args), matchThreshold: joinClassnamesByName("matchThreshold", args), branches: joinClassnamesByName("branches", args), hasSubQuery: joinClassnamesByName("hasSubQuery", args) };
};

// src/utils/mergeTranslations.ts
import { produce as produce5 } from "immer";

// src/utils/objectUtils.ts
var objectKeys = Object.keys;
var objectEntries = Object.entries;

// src/utils/mergeTranslations.ts
var mergeTranslations = function(base) {
  for (var _len = arguments.length, otherTranslations = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    otherTranslations[_key - 1] = arguments[_key];
  }
  return produce5(base, (draft) => {
    for (const translations of otherTranslations) {
      if (translations) {
        for (const t of objectKeys(translations)) {
          if (draft[t]) {
            Object.assign(draft[t], translations[t]);
          } else {
            Object.assign(draft, { [t]: translations[t] });
          }
        }
      }
    }
  });
};
var mergeTranslation = (el, keyPropContextMap, finalize) => {
  const finalKeys = objectEntries(keyPropContextMap).map((_ref) => {
    let [key, [pT, cT]] = _ref;
    var _a, _b;
    return [key, (_b = pT != null ? pT : cT) != null ? _b : (_a = (finalize ? defaultTranslations : {})[el]) == null ? void 0 : _a[key]];
  }).filter((k) => !!k[1]);
  return finalKeys.length > 0 ? { [el]: Object.fromEntries(finalKeys) } : void 0;
};

// src/utils/prepareQueryObjects.ts
import { produce as produce6 } from "immer";
var prepareRule = function(rule) {
  let { idGenerator = generateID } = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  return produce6(rule, (draft) => {
    if (!draft.id) {
      draft.id = idGenerator();
    }
  });
};
var prepareRuleGroup = function(queryObject) {
  let { idGenerator = generateID } = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  return produce6(queryObject, (draft) => {
    if (!draft.id) {
      draft.id = idGenerator();
    }
    draft.rules = draft.rules.map((r) => typeof r === "string" ? r : isRuleGroup(r) ? prepareRuleGroup(r, { idGenerator }) : prepareRule(r, { idGenerator }));
  });
};
var prepareRuleOrGroup = function(rg) {
  let { idGenerator = generateID } = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  return isRuleGroup(rg) ? prepareRuleGroup(rg, { idGenerator }) : prepareRule(rg, { idGenerator });
};

// src/utils/queryTools.ts
import { produce as produce7 } from "immer";

// src/utils/regenerateIDs.ts
var __defProp32 = Object.defineProperty;
var __defProps31 = Object.defineProperties;
var __getOwnPropDescs31 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols32 = Object.getOwnPropertySymbols;
var __hasOwnProp32 = Object.prototype.hasOwnProperty;
var __propIsEnum32 = Object.prototype.propertyIsEnumerable;
var __defNormalProp32 = (obj, key, value) => key in obj ? __defProp32(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues32 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp32.call(b, prop)) __defNormalProp32(a, prop, b[prop]);
  if (__getOwnPropSymbols32) for (var prop of __getOwnPropSymbols32(b)) {
    if (__propIsEnum32.call(b, prop)) __defNormalProp32(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps31 = (a, b) => __defProps31(a, __getOwnPropDescs31(b));
var regenerateID = function(rule) {
  let { idGenerator = generateID } = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  return structuredClone(__spreadProps31(__spreadValues32({}, rule), { id: idGenerator() }));
};
var regenerateIDs = function(ruleOrGroup) {
  let { idGenerator = generateID } = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  if (!isPojo(ruleOrGroup)) return ruleOrGroup;
  if (!isRuleGroup(ruleOrGroup)) {
    return structuredClone(__spreadProps31(__spreadValues32({}, ruleOrGroup), { id: idGenerator() }));
  }
  if (isRuleGroupType(ruleOrGroup)) {
    const rules2 = ruleOrGroup.rules.map((r) => isRuleGroup(r) ? regenerateIDs(r, { idGenerator }) : regenerateID(r, { idGenerator }));
    return __spreadProps31(__spreadValues32({}, ruleOrGroup), { id: idGenerator(), rules: rules2 });
  }
  const rules = ruleOrGroup.rules.map((r) => typeof r === "string" ? r : isRuleGroup(r) ? regenerateIDs(r, { idGenerator }) : regenerateID(r, { idGenerator }));
  return __spreadProps31(__spreadValues32({}, ruleOrGroup), { id: idGenerator(), rules });
};

// src/utils/queryTools.ts
var add = function(query, ruleOrGroup, parentPathOrID) {
  let { combinators = defaultCombinators, combinatorPreceding, idGenerator = generateID } = arguments.length > 3 && arguments[3] !== void 0 ? arguments[3] : {};
  return produce7(query, (draft) => {
    const parent = Array.isArray(parentPathOrID) ? findPath(parentPathOrID, draft) : findID(parentPathOrID, draft);
    if (!parent || !isRuleGroup(parent)) return;
    if (isRuleGroupTypeIC(parent) && parent.rules.length > 0) {
      const prevCombinator = parent.rules.at(-2);
      parent.rules.push(
        // @ts-expect-error This is technically a type violation until the next push
        // to the rules array, but that happens immediately and unconditionally so
        // there's no significant risk.
        combinatorPreceding != null ? combinatorPreceding : typeof prevCombinator === "string" ? prevCombinator : getFirstOption(combinators)
      );
    }
    parent.rules.push(prepareRuleOrGroup(ruleOrGroup, { idGenerator }));
  });
};
var update = function(query, prop, value, pathOrID) {
  let { resetOnFieldChange = true, resetOnOperatorChange = false, getRuleDefaultOperator = () => "=", getValueSources = () => ["value"], getRuleDefaultValue = () => "", getMatchModes = () => [] } = arguments.length > 4 && arguments[4] !== void 0 ? arguments[4] : {};
  return produce7(query, (draft) => {
    var _a;
    const path = Array.isArray(pathOrID) ? pathOrID : getPathOfID(pathOrID, draft);
    if (!path) return;
    if (prop === "combinator" && !isRuleGroupType(draft)) {
      const parentRules = findPath(getParentPath(path), draft).rules;
      if (path.at(-1) % 2 === 1) {
        parentRules[path.at(-1)] = value;
      }
      return;
    }
    const ruleOrGroup = findPath(path, draft);
    if (!ruleOrGroup) return;
    const isGroup = isRuleGroup(ruleOrGroup);
    if (ruleOrGroup[prop] === value) return;
    if (prop !== "valueSource") {
      ruleOrGroup[prop] = value;
    }
    if (isGroup) return;
    let resetValueSource = false;
    let resetValue = false;
    if (prop === "field" && !isGroup) {
      const fromFieldMatchModes = getMatchModes(ruleOrGroup.field);
      const toFieldMatchModes = getMatchModes(value);
      const nextMatchMode = ((_a = ruleOrGroup.match) == null ? void 0 : _a.mode) && toFieldMatchModes.length > 0 && getOption(toFieldMatchModes, ruleOrGroup.match.mode) ? null : getFirstOption(toFieldMatchModes);
      if (nextMatchMode) {
        ruleOrGroup.match = { mode: nextMatchMode, threshold: 1 };
      }
      if (fromFieldMatchModes.length > 0 || toFieldMatchModes.length > 0) {
        resetOnFieldChange = true;
      }
    }
    if (resetOnFieldChange && prop === "field") {
      ruleOrGroup.operator = getRuleDefaultOperator(value);
      resetValueSource = true;
      resetValue = true;
    }
    if (resetOnOperatorChange && prop === "operator") {
      resetValueSource = true;
      resetValue = true;
    }
    const valueSources = getValueSourcesUtil({ name: ruleOrGroup.field, value: ruleOrGroup.field, label: "" }, ruleOrGroup.operator, getValueSources);
    const defaultValueSource = getFirstOption(valueSources);
    if (resetValueSource && ruleOrGroup.valueSource && defaultValueSource !== ruleOrGroup.valueSource || prop === "valueSource" && value !== ruleOrGroup.valueSource) {
      resetValue = !!ruleOrGroup.valueSource || !ruleOrGroup.valueSource && value !== defaultValueSource;
      ruleOrGroup.valueSource = resetValueSource ? defaultValueSource : value;
    }
    if (resetValue) {
      ruleOrGroup.value = getRuleDefaultValue(ruleOrGroup);
    }
  });
};
var remove = (query, pathOrID) => {
  const path = Array.isArray(pathOrID) ? pathOrID : getPathOfID(pathOrID, query);
  if (!path) return query;
  if (
    // Can't remove the root group
    path.length === 0 || // Can't independently remove independent combinators
    !isRuleGroupType(query) && !findPath(path, query)
  ) {
    return query;
  }
  return produce7(query, (draft) => {
    const index = path.at(-1);
    const parent = findPath(getParentPath(path), draft);
    if (parent && isRuleGroup(parent)) {
      if (!isRuleGroupType(parent) && parent.rules.length > 1) {
        const idxStartDelete = index === 0 ? 0 : index - 1;
        parent.rules.splice(idxStartDelete, 2);
      } else {
        parent.rules.splice(index, 1);
      }
    }
  });
};
var getNextPath = (query, currentPath, newPathOrShiftDirection) => {
  if (Array.isArray(newPathOrShiftDirection)) {
    return newPathOrShiftDirection;
  }
  const ic = isRuleGroupTypeIC(query);
  if (newPathOrShiftDirection === "up") {
    if (pathsAreEqual(currentPath, [0])) {
      return currentPath;
    } else if (currentPath.at(-1) === 0) {
      const parentPath = getParentPath(currentPath);
      return [...getParentPath(parentPath), Math.max(0, parentPath.at(-1) - (ic ? 1 : 0))];
    } else {
      const evaluationPath = [...getParentPath(currentPath), Math.max(0, currentPath.at(-1) - (ic ? 2 : 1))];
      const entityAtTarget = findPath(evaluationPath, query);
      if (isRuleGroup(entityAtTarget)) {
        return [...evaluationPath, entityAtTarget.rules.length];
      } else {
        const targetPath = [...getParentPath(currentPath), Math.max(0, currentPath.at(-1) - (ic ? 3 : 1))];
        return targetPath;
      }
    }
  } else if (newPathOrShiftDirection === "down") {
    if (pathsAreEqual([query.rules.length - 1], currentPath)) {
      return currentPath;
    } else if (currentPath.at(-1) === findPath(getParentPath(currentPath), query).rules.length - 1) {
      const parentPath = getParentPath(currentPath);
      return [...getParentPath(parentPath), parentPath.at(-1) + 1];
    } else {
      const evaluationPath = [...getParentPath(currentPath), currentPath.at(-1) + (ic ? 2 : 1)];
      const entityToEvaluate = findPath(evaluationPath, query);
      if (isRuleGroup(entityToEvaluate)) {
        return [...evaluationPath, 0];
      } else {
        const targetPath = [...getParentPath(currentPath), currentPath.at(-1) + (ic ? 3 : 2)];
        return targetPath;
      }
    }
  }
  return currentPath;
};
var move = function(query, oldPathOrID, newPath) {
  let { clone = false, combinators = defaultCombinators, idGenerator = generateID } = arguments.length > 3 && arguments[3] !== void 0 ? arguments[3] : {};
  const oldPath = Array.isArray(oldPathOrID) ? oldPathOrID : getPathOfID(oldPathOrID, query);
  if (!oldPath) return query;
  const nextPath = getNextPath(query, oldPath, newPath);
  if (oldPath.length === 0 || pathsAreEqual(oldPath, nextPath) || !findPath(getParentPath(nextPath), query)) {
    return query;
  }
  const ruleOrGroupOriginal = findPath(oldPath, query);
  if (!ruleOrGroupOriginal) {
    return query;
  }
  const ruleOrGroup = clone ? regenerateIDs(ruleOrGroupOriginal, { idGenerator }) : ruleOrGroupOriginal;
  return produce7(query, (draft) => {
    var _a, _b, _c, _d;
    const independentCombinators = isRuleGroupTypeIC(draft);
    const parentOfRuleToRemove = findPath(getParentPath(oldPath), draft);
    const ruleToRemoveIndex = oldPath.at(-1);
    const oldPrevCombinator = independentCombinators && ruleToRemoveIndex > 0 ? parentOfRuleToRemove.rules[ruleToRemoveIndex - 1] : null;
    const oldNextCombinator = independentCombinators && ruleToRemoveIndex < parentOfRuleToRemove.rules.length - 1 ? parentOfRuleToRemove.rules[ruleToRemoveIndex + 1] : null;
    if (!clone) {
      const idxStartDelete = independentCombinators ? Math.max(0, ruleToRemoveIndex - 1) : ruleToRemoveIndex;
      const deleteLength = independentCombinators ? 2 : 1;
      parentOfRuleToRemove.rules.splice(idxStartDelete, deleteLength);
    }
    const newNewPath = [...nextPath];
    const commonAncestorPath = getCommonAncestorPath(oldPath, nextPath);
    if (!clone && oldPath.length === commonAncestorPath.length + 1 && nextPath[commonAncestorPath.length] > oldPath[commonAncestorPath.length]) {
      newNewPath[commonAncestorPath.length] -= independentCombinators ? 2 : 1;
    }
    const newNewParentPath = getParentPath(newNewPath);
    const parentToInsertInto = findPath(newNewParentPath, draft);
    const newIndex = newNewPath.at(-1);
    const insertRuleOrGroup = function() {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      return parentToInsertInto.rules.splice(newIndex, 0, ...args);
    };
    if (parentToInsertInto.rules.length === 0 || !independentCombinators) {
      insertRuleOrGroup(ruleOrGroup);
    } else {
      if (newIndex === 0) {
        if (ruleToRemoveIndex === 0 && oldNextCombinator) {
          insertRuleOrGroup(ruleOrGroup, oldNextCombinator);
        } else {
          const newNextCombinator = (_b = (_a = parentToInsertInto.rules[1]) != null ? _a : oldPrevCombinator) != null ? _b : getFirstOption(combinators);
          insertRuleOrGroup(ruleOrGroup, newNextCombinator);
        }
      } else {
        if (oldPrevCombinator) {
          insertRuleOrGroup(oldPrevCombinator, ruleOrGroup);
        } else {
          const newPrevCombinator = (_d = (_c = parentToInsertInto.rules[newIndex - 2]) != null ? _c : oldNextCombinator) != null ? _d : getFirstOption(combinators);
          insertRuleOrGroup(newPrevCombinator, ruleOrGroup);
        }
      }
    }
  });
};
var insert = function(query, ruleOrGroup, path) {
  let { combinators = defaultCombinators, combinatorPreceding, combinatorSucceeding, idGenerator = generateID, replace = false } = arguments.length > 3 && arguments[3] !== void 0 ? arguments[3] : {};
  return produce7(query, (draft) => {
    var _a, _b, _c, _d, _e;
    const parentToInsertInto = findPath(getParentPath(path), draft);
    if (!parentToInsertInto || !isRuleGroup(parentToInsertInto)) return;
    const rorg = regenerateIDs(ruleOrGroup, { idGenerator });
    const independentCombinators = isRuleGroupTypeIC(draft);
    const newIndex = path.at(-1);
    const insertRuleOrGroup = function(idx) {
      for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
        args[_key2 - 1] = arguments[_key2];
      }
      return parentToInsertInto.rules.splice(idx, replace ? args.length : 0, ...args);
    };
    if (parentToInsertInto.rules.length === 0 || !independentCombinators) {
      insertRuleOrGroup(newIndex, rorg);
    } else if (replace && independentCombinators) {
      insertRuleOrGroup(newIndex + newIndex % 2, rorg);
    } else {
      if (newIndex === 0) {
        if (((_a = rorg.path) == null ? void 0 : _a.at(-1)) === 0 && combinatorSucceeding) {
          insertRuleOrGroup(newIndex, rorg, combinatorSucceeding);
        } else {
          const newNextCombinator = (_c = (_b = parentToInsertInto.rules[1]) != null ? _b : combinatorPreceding) != null ? _c : getFirstOption(combinators);
          insertRuleOrGroup(newIndex, rorg, newNextCombinator);
        }
      } else {
        const normalizedNewIndex = newIndex % 2 === 0 ? newIndex - 1 : newIndex;
        if (combinatorPreceding) {
          insertRuleOrGroup(normalizedNewIndex, combinatorPreceding, rorg);
        } else {
          const newPrevCombinator = (_e = (_d = parentToInsertInto.rules[normalizedNewIndex - 2]) != null ? _d : combinatorSucceeding) != null ? _e : getFirstOption(combinators);
          insertRuleOrGroup(normalizedNewIndex, newPrevCombinator, rorg);
        }
      }
    }
  });
};
var group = function(query, sourcePathOrID, targetPathOrID) {
  let { clone = false, combinators = defaultCombinators, idGenerator = generateID } = arguments.length > 3 && arguments[3] !== void 0 ? arguments[3] : {};
  const sourcePath = Array.isArray(sourcePathOrID) ? sourcePathOrID : getPathOfID(sourcePathOrID, query);
  const targetPath = Array.isArray(targetPathOrID) ? targetPathOrID : getPathOfID(targetPathOrID, query);
  if (!sourcePath || !targetPath) return query;
  const nextPath = getNextPath(query, sourcePath, targetPath);
  if (sourcePath.length === 0 || pathsAreEqual(sourcePath, nextPath) || !findPath(getParentPath(nextPath), query)) {
    return query;
  }
  const sourceRuleOrGroupOriginal = findPath(sourcePath, query);
  const targetRuleOrGroup = findPath(targetPath, query);
  if (!sourceRuleOrGroupOriginal || !targetRuleOrGroup) {
    return query;
  }
  const sourceRuleOrGroup = clone ? regenerateIDs(sourceRuleOrGroupOriginal, { idGenerator }) : sourceRuleOrGroupOriginal;
  return produce7(query, (draft) => {
    const independentCombinators = isRuleGroupTypeIC(draft);
    const parentOfRuleToRemove = findPath(getParentPath(sourcePath), draft);
    const ruleToRemoveIndex = sourcePath.at(-1);
    if (!clone) {
      const idxStartDelete = independentCombinators ? Math.max(0, ruleToRemoveIndex - 1) : ruleToRemoveIndex;
      const deleteLength = independentCombinators ? 2 : 1;
      parentOfRuleToRemove.rules.splice(idxStartDelete, deleteLength);
    }
    const newNewPath = [...nextPath];
    const commonAncestorPath = getCommonAncestorPath(sourcePath, nextPath);
    if (!clone && sourcePath.length === commonAncestorPath.length + 1 && nextPath[commonAncestorPath.length] > sourcePath[commonAncestorPath.length]) {
      newNewPath[commonAncestorPath.length] -= independentCombinators ? 2 : 1;
    }
    const newNewParentPath = getParentPath(newNewPath);
    const parentOfTargetPath = findPath(newNewParentPath, draft);
    const targetPathIndex = newNewPath.at(-1);
    parentOfTargetPath.rules.splice(targetPathIndex, 1, prepareRuleOrGroup(
      independentCombinators ? { rules: [targetRuleOrGroup, getFirstOption(combinators), sourceRuleOrGroup] } : {
        combinator: getFirstOption(combinators),
        rules: [targetRuleOrGroup, sourceRuleOrGroup]
        // oxlint-disable-next-line typescript/no-explicit-any
      },
      { idGenerator }
      // oxlint-disable-next-line typescript/no-explicit-any
    ));
  });
};

// src/utils/toOptions.tsx
import { jsx as jsx5 } from "react/jsx-runtime";
var toOptions = (arr) => isOptionGroupArray(arr) ? arr.map((og) => /* @__PURE__ */ jsx5("optgroup", { label: og.label, children: og.options.map((opt) => /* @__PURE__ */ jsx5("option", { value: opt.name, disabled: opt.disabled, children: opt.label }, opt.name)) }, og.label)) : Array.isArray(arr) ? arr.map((opt) => /* @__PURE__ */ jsx5("option", { value: opt.name, disabled: opt.disabled, children: opt.label }, opt.name)) : null;

// src/components/MatchModeEditor.tsx
var __defProp33 = Object.defineProperty;
var __defProps32 = Object.defineProperties;
var __getOwnPropDescs32 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols33 = Object.getOwnPropertySymbols;
var __hasOwnProp33 = Object.prototype.hasOwnProperty;
var __propIsEnum33 = Object.prototype.propertyIsEnumerable;
var __defNormalProp33 = (obj, key, value) => key in obj ? __defProp33(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues33 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp33.call(b, prop)) __defNormalProp33(a, prop, b[prop]);
  if (__getOwnPropSymbols33) for (var prop of __getOwnPropSymbols33(b)) {
    if (__propIsEnum33.call(b, prop)) __defNormalProp33(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps32 = (a, b) => __defProps32(a, __getOwnPropDescs32(b));
var dummyFieldData = { name: "", value: "", label: "" };
var requiresThreshold = (mm) => {
  var _a;
  return ["atleast", "atmost", "exactly"].includes((_a = lc(mm)) != null ? _a : (
    /* istanbul ignore next */
    ""
  ));
};
var dummyPath = [];
var MatchModeEditor = (props) => {
  const { match, options, title, className, disabled, testID, schema, selectorComponent: SelectorComponent = props.schema.controls.valueSelector, numericEditorComponent: NumericEditorComponent = props.schema.controls.valueEditor } = props;
  const { thresholdNum, thresholdRule, thresholdSchema, handleChangeMode, handleChangeThreshold } = useMatchModeEditor(props);
  return /* @__PURE__ */ jsxs(React2.Fragment, { children: [/* @__PURE__ */ jsx6(SelectorComponent, { schema, testID, className, title, handleOnChange: handleChangeMode, disabled, value: match.mode, options, multiple: false, listsAsArrays: false, path: dummyPath, level: 0 }), requiresThreshold(match.mode) && /* @__PURE__ */ jsx6(NumericEditorComponent, { skipHook: true, testID, inputType: "number", title, className, disabled, handleOnChange: handleChangeThreshold, field: "", operator: "", value: thresholdNum, valueSource: "value", fieldData: dummyFieldData, schema: thresholdSchema, path: dummyPath, level: 0, rule: thresholdRule })] });
};
var useMatchModeEditor = (props) => {
  const $ = c(16);
  const { match, handleOnChange } = props;
  let t0;
  t0 = typeof match.threshold === "number" ? Math.max(0, match.threshold) : 1;
  const thresholdNum = t0;
  let t1;
  let t2;
  if ($[0] !== thresholdNum) {
    t2 = { field: "", operator: "=", value: thresholdNum };
    $[0] = thresholdNum;
    $[1] = t2;
  } else {
    t2 = $[1];
  }
  t1 = t2;
  const thresholdRule = t1;
  let t3;
  let t4;
  if ($[2] !== props.schema) {
    t4 = __spreadProps32(__spreadValues33({}, props.schema), { parseNumbers: true });
    $[2] = props.schema;
    $[3] = t4;
  } else {
    t4 = $[3];
  }
  t3 = t4;
  const thresholdSchema = t3;
  let t5;
  if ($[4] !== handleOnChange || $[5] !== match) {
    t5 = (mode) => {
      if (requiresThreshold(mode) && typeof match.threshold !== "number") {
        handleOnChange(__spreadProps32(__spreadValues33({}, match), { mode, threshold: 1 }));
      } else {
        handleOnChange(__spreadProps32(__spreadValues33({}, match), { mode }));
      }
    };
    $[4] = handleOnChange;
    $[5] = match;
    $[6] = t5;
  } else {
    t5 = $[6];
  }
  const handleChangeMode = t5;
  let t6;
  if ($[7] !== handleOnChange || $[8] !== match) {
    t6 = (threshold) => {
      handleOnChange(__spreadProps32(__spreadValues33({}, match), { threshold: parseNumber(threshold, { parseNumbers: true }) }));
    };
    $[7] = handleOnChange;
    $[8] = match;
    $[9] = t6;
  } else {
    t6 = $[9];
  }
  const handleChangeThreshold = t6;
  let t7;
  if ($[10] !== handleChangeMode || $[11] !== handleChangeThreshold || $[12] !== thresholdNum || $[13] !== thresholdRule || $[14] !== thresholdSchema) {
    t7 = { thresholdNum, thresholdRule, thresholdSchema, handleChangeMode, handleChangeThreshold };
    $[10] = handleChangeMode;
    $[11] = handleChangeThreshold;
    $[12] = thresholdNum;
    $[13] = thresholdRule;
    $[14] = thresholdSchema;
    $[15] = t7;
  } else {
    t7 = $[15];
  }
  return t7;
};

// src/components/NotToggle.tsx
import { jsx as jsx7, jsxs as jsxs2 } from "react/jsx-runtime";
import * as React3 from "react";
var NotToggle = (props) => {
  const $ = c(11);
  const id = React3.useId();
  let t0;
  if ($[0] !== props) {
    t0 = (e) => props.handleOnChange(e.target.checked);
    $[0] = props;
    $[1] = t0;
  } else {
    t0 = $[1];
  }
  const t1 = !!props.checked;
  let t2;
  if ($[2] !== id || $[3] !== props.className || $[4] !== props.disabled || $[5] !== props.label || $[6] !== props.testID || $[7] !== props.title || $[8] !== t0 || $[9] !== t1) {
    t2 = jsxs2("label", { "data-testid": props.testID, className: props.className, title: props.title, htmlFor: id, children: [jsx7("input", { id, type: "checkbox", onChange: t0, checked: t1, disabled: props.disabled }), props.label] });
    $[2] = id;
    $[3] = props.className;
    $[4] = props.disabled;
    $[5] = props.label;
    $[6] = props.testID;
    $[7] = props.title;
    $[8] = t0;
    $[9] = t1;
    $[10] = t2;
  } else {
    t2 = $[10];
  }
  return t2;
};

// src/components/Rule.tsx
import { jsx as jsx9, jsxs as jsxs4 } from "react/jsx-runtime";
import * as React7 from "react";
import { useCallback as useCallback6, useMemo as useMemo7 } from "react";

// src/messages.ts
var messages = { errorInvalidIndependentCombinatorsProp: "QueryBuilder was rendered with a truthy independentCombinators prop. This prop is deprecated and unnecessary. Furthermore, the initial query/defaultQuery prop was of type RuleGroupType instead of type RuleGroupIC. More info: https://react-querybuilder.js.org/docs/components/querybuilder#independent-combinators", errorUnnecessaryIndependentCombinatorsProp: "QueryBuilder was rendered with the deprecated and unnecessary independentCombinators prop. To use independent combinators, make sure the query/defaultQuery prop is of type RuleGroupIC when the component mounts. More info: https://react-querybuilder.js.org/docs/components/querybuilder#independent-combinators", errorDeprecatedRuleGroupProps: "A custom RuleGroup component has rendered a standard RuleGroup component with deprecated props. The combinator, not, and rules props should not be used. Instead, the full group object should be passed as the ruleGroup prop.", errorDeprecatedRuleProps: "A custom RuleGroup component has rendered a standard Rule component with deprecated props. The field, operator, value, and valueSource props should not be used. Instead, the full rule object should be passed as the rule prop.", errorBothQueryDefaultQuery: "QueryBuilder was rendered with both query and defaultQuery props. QueryBuilder must be either controlled or uncontrolled (specify either the query prop, or the defaultQuery prop, but not both). Decide between using a controlled or uncontrolled query builder and remove one of these props. More info: https://reactjs.org/link/controlled-components", errorUncontrolledToControlled: "QueryBuilder is changing from an uncontrolled component to be controlled. This is likely caused by the query changing from undefined to a defined value, which should not happen. Decide between using a controlled or uncontrolled query builder for the lifetime of the component. More info: https://reactjs.org/link/controlled-components", errorControlledToUncontrolled: "QueryBuilder is changing from a controlled component to be uncontrolled. This is likely caused by the query changing from defined to undefined, which should not happen. Decide between using a controlled or uncontrolled query builder for the lifetime of the component. More info: https://reactjs.org/link/controlled-components", errorEnabledDndWithoutReactDnD: "QueryBuilder was rendered with the enableDragAndDrop prop set to true, but either react-dnd was not detected or one of react-dnd-html5-backend or react-dnd-touch-backend was not detected. To enable drag-and-drop functionality, install react-dnd and one of the backend packages and wrap QueryBuilder in QueryBuilderDnD from @react-querybuilder/dnd." };

// src/redux/_internal.ts
import { createDispatchHook, createStoreHook } from "react-redux";

// src/redux/queriesSlice.ts
import { createSlice } from "@reduxjs/toolkit";
var initialState = {};
var queriesSlice = createSlice({ name: "queries", initialState, reducers: { setQueryState: (state, _ref) => {
  let { payload: { qbId, query } } = _ref;
  state[qbId] = query;
} }, selectors: { getQuerySelectorById: (state, qbId) => state[qbId] } });

// src/redux/QueryBuilderStateContext.ts
import * as React4 from "react";
var QueryBuilderStateContext = /* @__PURE__ */ React4.createContext(null);

// src/redux/warningsSlice.ts
import { createSlice as createSlice2 } from "@reduxjs/toolkit";
var initialState2 = { [messages.errorInvalidIndependentCombinatorsProp]: false, [messages.errorUnnecessaryIndependentCombinatorsProp]: false, [messages.errorDeprecatedRuleGroupProps]: false, [messages.errorDeprecatedRuleProps]: false, [messages.errorBothQueryDefaultQuery]: false, [messages.errorUncontrolledToControlled]: false, [messages.errorControlledToUncontrolled]: false, [messages.errorEnabledDndWithoutReactDnD]: false };
var warningsSlice = createSlice2({ name: "warnings", initialState: initialState2, reducers: { rqbWarn: (state, _ref) => {
  let { payload } = _ref;
  if (!state[payload]) {
    console.error(payload);
    state[payload] = true;
  }
} } });

// src/redux/_internal.ts
var _RQB_INTERNAL_dispatchThunk = (_ref) => {
  let { payload, onQueryChange } = _ref;
  return (dispatch) => {
    dispatch(queriesSlice.actions.setQueryState(payload));
    if (typeof onQueryChange === "function") {
      onQueryChange(
        payload.query
        /* ??? */
      );
    }
  };
};
var useRQB_INTERNAL_QueryBuilderDispatch = createDispatchHook(QueryBuilderStateContext);
var useRQB_INTERNAL_QueryBuilderStore = createStoreHook(QueryBuilderStateContext);
var { rqbWarn: _SYNC_rqbWarn } = warningsSlice.actions;
var rqbWarn = (msg) => (dispatch) => {
  setTimeout(() => dispatch(_SYNC_rqbWarn(msg)));
};
var preloadedState = { queries: queriesSlice.getInitialState(), warnings: warningsSlice.getInitialState() };
var storeCommon = { reducer: { queries: queriesSlice.reducer, warnings: warningsSlice.reducer }, preloadedState, middleware: (getDefaultMiddleware) => getDefaultMiddleware({
  // Ignore non-serializable values in setQueryState actions and rule `value`s
  // https://redux-toolkit.js.org/usage/usage-guide#working-with-non-serializable-data
  serializableCheck: { ignoredActions: [queriesSlice.actions.setQueryState.type], ignoredPaths: [/^queries\b.*\.rules\.\d+\.value$/] }
}) };

// src/hooks/usePrevious.ts
import { useRef } from "react";
var usePrevious = (value) => {
  const ref = useRef({ value, prev: null });
  const current = ref.current.value;
  if (value !== current) {
    ref.current = { value, prev: current };
  }
  return ref.current.prev;
};

// src/hooks/useControlledOrUncontrolled.ts
var useControlledOrUncontrolled = (params) => {
  const dispatch = useRQB_INTERNAL_QueryBuilderDispatch();
  const { defaultQuery, queryProp } = params;
  const prevQueryPresent = usePrevious(!!queryProp);
  if (process.env.NODE_ENV !== "production") {
    if (!!queryProp && !!defaultQuery) {
      dispatch(rqbWarn(messages.errorBothQueryDefaultQuery));
    } else {
      if (prevQueryPresent === true && !queryProp && !!defaultQuery) {
        dispatch(rqbWarn(messages.errorControlledToUncontrolled));
      } else {
        if (prevQueryPresent === false && !!queryProp && !defaultQuery) {
          dispatch(rqbWarn(messages.errorUncontrolledToControlled));
        }
      }
    }
  }
};

// src/hooks/useDeprecatedProps.ts
function useDeprecatedProps(type, logWarning, otherParams) {
  const dispatch = useRQB_INTERNAL_QueryBuilderDispatch();
  if (process.env.NODE_ENV !== "production" && logWarning) {
    if (type === "independentCombinators") {
      if (otherParams === "invalid") {
        dispatch(rqbWarn(messages.errorInvalidIndependentCombinatorsProp));
      }
      if (otherParams === "unnecessary") {
        dispatch(rqbWarn(messages.errorUnnecessaryIndependentCombinatorsProp));
      }
    }
    if (type === "rule") {
      dispatch(rqbWarn(messages.errorDeprecatedRuleProps));
    }
    if (type === "ruleGroup") {
      dispatch(rqbWarn(messages.errorDeprecatedRuleGroupProps));
    }
  }
}

// src/hooks/useFields.ts
import { useMemo as useMemo2 } from "react";
var __defProp34 = Object.defineProperty;
var __defProps33 = Object.defineProperties;
var __getOwnPropDescs33 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols34 = Object.getOwnPropertySymbols;
var __hasOwnProp34 = Object.prototype.hasOwnProperty;
var __propIsEnum34 = Object.prototype.propertyIsEnumerable;
var __defNormalProp34 = (obj, key, value) => key in obj ? __defProp34(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues34 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp34.call(b, prop)) __defNormalProp34(a, prop, b[prop]);
  if (__getOwnPropSymbols34) for (var prop of __getOwnPropSymbols34(b)) {
    if (__propIsEnum34.call(b, prop)) __defNormalProp34(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps33 = (a, b) => __defProps33(a, __getOwnPropDescs33(b));
var useFields = (props) => {
  const { fields: fieldsPropOriginal, baseField, translations, autoSelectField } = props;
  const defaultField = useMemo2(() => ({ id: translations.fields.placeholderName, name: translations.fields.placeholderName, value: translations.fields.placeholderName, label: translations.fields.placeholderLabel }), [translations.fields.placeholderLabel, translations.fields.placeholderName]);
  const fieldsProp = useMemo2(() => fieldsPropOriginal != null ? fieldsPropOriginal : [defaultField], [defaultField, fieldsPropOriginal]);
  const fields = useMemo2(() => {
    const flds = Array.isArray(fieldsProp) ? toFullOptionList(fieldsProp, baseField) : objectKeys(toFullOptionMap(fieldsProp, baseField)).map((fld) => __spreadProps33(__spreadValues34({}, fieldsProp[fld]), { name: fld, value: fld })).sort((a, b) => a.label.localeCompare(b.label));
    if (isFlexibleOptionGroupArray(flds)) {
      return autoSelectField ? uniqOptGroups(flds) : uniqOptGroups([{ label: translations.fields.placeholderGroupLabel, options: [defaultField] }, ...flds]);
    } else {
      return autoSelectField ? uniqByIdentifier(flds) : uniqByIdentifier([defaultField, ...flds]);
    }
  }, [autoSelectField, baseField, defaultField, fieldsProp, translations.fields.placeholderGroupLabel]);
  const fieldMap = useMemo2(() => {
    var _a, _b;
    if (!Array.isArray(fieldsProp)) {
      const fp = toFullOptionMap(fieldsProp, baseField);
      return autoSelectField ? fp : __spreadProps33(__spreadValues34({}, fp), { [translations.fields.placeholderName]: defaultField });
    }
    const fm = {};
    if (isFlexibleOptionGroupArray(fields)) {
      for (const f of fields) {
        for (const opt of f.options) {
          fm[(_a = opt.value) != null ? _a : (
            /* istanbul ignore next */
            opt.name
          )] = toFullOption(opt, baseField);
        }
      }
    } else {
      for (const f_0 of fields) {
        fm[(_b = f_0.value) != null ? _b : (
          /* istanbul ignore next */
          f_0.name
        )] = toFullOption(f_0, baseField);
      }
    }
    return fm;
  }, [autoSelectField, baseField, defaultField, fields, fieldsProp, translations.fields.placeholderName]);
  return { defaultField, fields, fieldMap };
};

// src/hooks/useReactDndWarning.ts
var didWarnEnabledDndWithoutReactDnD = false;
var useReactDndWarning = (enableDragAndDrop, dndRefs) => {
  if (process.env.NODE_ENV !== "production" && !didWarnEnabledDndWithoutReactDnD && enableDragAndDrop && !dndRefs) {
    console.error(messages.errorEnabledDndWithoutReactDnD);
    didWarnEnabledDndWithoutReactDnD = true;
  }
};

// src/hooks/useSelectElementChangeHandler.ts
import { useMemo as useMemo3 } from "react";
var useSelectElementChangeHandler = (params) => {
  const $ = c(3);
  const { multiple, onChange } = params;
  let t0;
  let t1;
  if ($[0] !== multiple || $[1] !== onChange) {
    t1 = multiple ? (e) => onChange(Array.from(e.target.selectedOptions).map(_temp)) : (e_0) => onChange(e_0.target.value);
    $[0] = multiple;
    $[1] = onChange;
    $[2] = t1;
  } else {
    t1 = $[2];
  }
  t0 = t1;
  const selectElementChangeHandler = t0;
  return selectElementChangeHandler;
};
function _temp(o) {
  return o.value;
}

// src/hooks/useStopEventPropagation.ts
import { useCallback as useCallback2 } from "react";
var useStopEventPropagation = (method) => {
  const $ = c(2);
  let t0;
  if ($[0] !== method) {
    t0 = (event, context) => {
      event == null ? void 0 : event.preventDefault();
      event == null ? void 0 : event.stopPropagation();
      method(event, context);
    };
    $[0] = method;
    $[1] = t0;
  } else {
    t0 = $[1];
  }
  return t0;
};

// src/components/QueryBuilder.useQueryBuilderSchema.ts
import { useCallback as useCallback3, useEffect, useMemo as useMemo4, useRef as useRef2, useState } from "react";

// src/redux/hooks.ts
import * as React5 from "react";
import { createSelectorHook } from "react-redux";

// src/redux/selectors.ts
var getQuerySelectorById = (qbId) => (state) => queriesSlice.selectors.getQuerySelectorById({ queries: state.queries }, qbId);

// src/redux/hooks.ts
var useRQB_INTERNAL_QueryBuilderSelector = createSelectorHook(QueryBuilderStateContext);
var useQueryBuilderSelector = (selector, other) => {
  const rqbContext = React5.useContext(QueryBuilderContext);
  const result = useRQB_INTERNAL_QueryBuilderSelector(selector, other);
  return result != null ? result : rqbContext == null ? void 0 : rqbContext.initialQuery;
};
var useQueryBuilderQuery = (props) => {
  var _a, _b, _c;
  const rqbContext = React5.useContext(QueryBuilderContext);
  return (_c = useRQB_INTERNAL_QueryBuilderSelector(getQuerySelectorById((_b = (_a = props == null ? void 0 : props.schema.qbId) != null ? _a : rqbContext.qbId) != null ? _b : (
    /* istanbul ignore next */
    ""
  )))) != null ? _c : rqbContext == null ? void 0 : rqbContext.initialQuery;
};

// src/components/QueryBuilder.useQueryBuilderSchema.ts
var __defProp35 = Object.defineProperty;
var __defProps34 = Object.defineProperties;
var __getOwnPropDescs34 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols35 = Object.getOwnPropertySymbols;
var __hasOwnProp35 = Object.prototype.hasOwnProperty;
var __propIsEnum35 = Object.prototype.propertyIsEnumerable;
var __defNormalProp35 = (obj, key, value) => key in obj ? __defProp35(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues35 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp35.call(b, prop)) __defNormalProp35(a, prop, b[prop]);
  if (__getOwnPropSymbols35) for (var prop of __getOwnPropSymbols35(b)) {
    if (__propIsEnum35.call(b, prop)) __defNormalProp35(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps34 = (a, b) => __defProps34(a, __getOwnPropDescs34(b));
var defaultValidationResult = {};
var defaultValidationMap = {};
var defaultDisabledPaths = [];
var icCombinatorPropObject = {};
var defaultGetValueEditorSeparator = () => null;
var defaultGetRuleOrGroupClassname = () => "";
var defaultOnAddMoveRemove = () => true;
var defaultOnLog = function() {
  console.log(...arguments);
};
function useQueryBuilderSchema(props, setup) {
  var _a, _b, _c, _d;
  const { query: queryProp, defaultQuery: defaultQueryProp, getValueEditorSeparator = defaultGetValueEditorSeparator, getRuleClassname = defaultGetRuleOrGroupClassname, getRuleGroupClassname = defaultGetRuleOrGroupClassname, onAddRule = defaultOnAddMoveRemove, onAddGroup = defaultOnAddMoveRemove, onMoveRule = defaultOnAddMoveRemove, onMoveGroup = defaultOnAddMoveRemove, onGroupRule = defaultOnAddMoveRemove, onGroupGroup = defaultOnAddMoveRemove, onRemove = defaultOnAddMoveRemove, onQueryChange, showCombinatorsBetweenRules: showCombinatorsBetweenRulesProp = false, showNotToggle: showNotToggleProp = false, showShiftActions: showShiftActionsProp = false, showCloneButtons: showCloneButtonsProp = false, showLockButtons: showLockButtonsProp = false, suppressStandardClassnames: suppressStandardClassnamesProp = false, resetOnFieldChange: resetOnFieldChangeProp = true, resetOnOperatorChange: resetOnOperatorChangeProp = false, autoSelectField: autoSelectFieldProp = true, autoSelectOperator: autoSelectOperatorProp = true, autoSelectValue: autoSelectValueProp = true, addRuleToNewGroups: addRuleToNewGroupsProp = false, listsAsArrays: listsAsArraysProp = false, parseNumbers = false, disabled = false, validator, onLog = defaultOnLog, idGenerator, accessibleDescriptionGenerator = generateAccessibleDescription } = props;
  const { qbId, rqbContext: incomingRqbContext, fields, fieldMap, combinators, getOperatorsMain, getMatchModesMain, getRuleDefaultOperator, getSubQueryBuilderPropsMain, getValueEditorTypeMain, getValueSourcesMain, getValuesMain, getRuleDefaultValue, getInputTypeMain, createRule, createRuleGroup } = setup;
  const { controlClassnames, controlElements: controls, debugMode, enableDragAndDrop, enableMountQueryChange, translations } = incomingRqbContext;
  const showCombinatorsBetweenRules = !!showCombinatorsBetweenRulesProp;
  const showNotToggle = !!showNotToggleProp;
  const showShiftActions = !!showShiftActionsProp;
  const showCloneButtons = !!showCloneButtonsProp;
  const showLockButtons = !!showLockButtonsProp;
  const resetOnFieldChange = !!resetOnFieldChangeProp;
  const resetOnOperatorChange = !!resetOnOperatorChangeProp;
  const autoSelectField = !!autoSelectFieldProp;
  const autoSelectOperator = !!autoSelectOperatorProp;
  const autoSelectValue = !!autoSelectValueProp;
  const addRuleToNewGroups = !!addRuleToNewGroupsProp;
  const listsAsArrays = !!listsAsArraysProp;
  const suppressStandardClassnames = !!suppressStandardClassnamesProp;
  const maxLevels = ((_a = props.maxLevels) != null ? _a : 0) > 0 ? Number(props.maxLevels) : Infinity;
  const log = useCallback3(function() {
    if (debugMode) {
      onLog(...arguments);
    }
  }, [debugMode, onLog]);
  useControlledOrUncontrolled({ defaultQuery: defaultQueryProp, queryProp });
  const queryBuilderStore = useRQB_INTERNAL_QueryBuilderStore();
  const queryBuilderDispatch = useRQB_INTERNAL_QueryBuilderDispatch();
  const querySelector = useMemo4(() => getQuerySelectorById(qbId), [qbId]);
  const storeQuery = useQueryBuilderSelector(querySelector);
  const getQuery = useCallback3(() => querySelector(queryBuilderStore.getState()), [queryBuilderStore, querySelector]);
  const fallbackQuery = useMemo4(() => createRuleGroup(), [createRuleGroup]);
  const candidateQuery = (_c = (_b = queryProp != null ? queryProp : storeQuery) != null ? _b : defaultQueryProp) != null ? _c : fallbackQuery;
  const rootGroup = candidateQuery.id ? candidateQuery : prepareRuleGroup(candidateQuery, { idGenerator });
  const [initialQuery] = useState(rootGroup);
  const rqbContext = useMemo4(() => __spreadProps34(__spreadValues35({}, incomingRqbContext), { initialQuery }), [incomingRqbContext, initialQuery]);
  useEffect(() => {
    if (!!queryProp && !Object.is(queryProp, storeQuery)) {
      queryBuilderDispatch(_RQB_INTERNAL_dispatchThunk({ payload: { qbId, query: queryProp }, onQueryChange: void 0 }));
    }
  }, [queryProp, qbId, storeQuery, queryBuilderDispatch]);
  const independentCombinators = useMemo4(() => isRuleGroupTypeIC(rootGroup), [rootGroup]);
  const invalidIC = !!props.independentCombinators && !independentCombinators;
  useDeprecatedProps("independentCombinators", invalidIC || !invalidIC && ((_d = props.independentCombinators) != null ? _d : "not present") !== "not present", invalidIC ? "invalid" : "unnecessary");
  const hasRunMountQueryChange = useRef2(false);
  useEffect(() => {
    if (hasRunMountQueryChange.current) return;
    hasRunMountQueryChange.current = true;
    queryBuilderDispatch(_RQB_INTERNAL_dispatchThunk({ payload: { qbId, query: rootGroup }, onQueryChange: (
      // Leave `onQueryChange` undefined if `enableMountQueryChange` is disabled
      enableMountQueryChange && typeof onQueryChange === "function" ? onQueryChange : void 0
    ) }));
  }, [enableMountQueryChange, onQueryChange, qbId, queryBuilderDispatch, rootGroup]);
  const dispatchQuery = useCallback3((newQuery) => {
    queryBuilderDispatch(_RQB_INTERNAL_dispatchThunk({ payload: { qbId, query: newQuery }, onQueryChange }));
  }, [onQueryChange, qbId, queryBuilderDispatch]);
  const disabledPaths = useMemo4(() => Array.isArray(disabled) && disabled || defaultDisabledPaths, [disabled]);
  const queryDisabled = disabled === true;
  const rootGroupDisabled = useMemo4(() => rootGroup.disabled || disabledPaths.some((p) => p.length === 0), [disabledPaths, rootGroup.disabled]);
  const onRuleAdd = useCallback3(
    // oxlint-disable-next-line typescript/no-explicit-any
    (rule, parentPath, context) => {
      var _a2;
      const queryLocal = getQuerySelectorById(qbId)(queryBuilderStore.getState());
      if (!queryLocal) return;
      if (pathIsDisabled(parentPath, queryLocal) || queryDisabled) {
        log({ qbId, type: LogType.parentPathDisabled, rule, parentPath, query: queryLocal });
        return;
      }
      const nextRule = onAddRule(rule, parentPath, queryLocal, context);
      if (!nextRule) {
        log({ qbId, type: LogType.onAddRuleFalse, rule, parentPath, query: queryLocal });
        return;
      }
      const newRule = nextRule === true ? rule : nextRule;
      const newQuery_0 = add(queryLocal, newRule, parentPath, { combinators, combinatorPreceding: (_a2 = newRule.combinatorPreceding) != null ? _a2 : void 0, idGenerator });
      log({ qbId, type: LogType.add, query: queryLocal, newQuery: newQuery_0, newRule, parentPath });
      dispatchQuery(newQuery_0);
    },
    [combinators, dispatchQuery, idGenerator, log, onAddRule, qbId, queryBuilderStore, queryDisabled]
  );
  const onGroupAdd = useCallback3(
    // oxlint-disable-next-line typescript/no-explicit-any
    (ruleGroup, parentPath_0, context_0) => {
      var _a2_0;
      if (parentPath_0.length >= maxLevels) return;
      const queryLocal_0 = getQuerySelectorById(qbId)(queryBuilderStore.getState());
      if (!queryLocal_0) return;
      if (pathIsDisabled(parentPath_0, queryLocal_0) || queryDisabled) {
        log({ qbId, type: LogType.parentPathDisabled, ruleGroup, parentPath: parentPath_0, query: queryLocal_0 });
        return;
      }
      const nextGroup = onAddGroup(ruleGroup, parentPath_0, queryLocal_0, context_0);
      if (!nextGroup) {
        log({ qbId, type: LogType.onAddGroupFalse, ruleGroup, parentPath: parentPath_0, query: queryLocal_0 });
        return;
      }
      const newGroup = nextGroup === true ? ruleGroup : nextGroup;
      const newQuery_1 = add(queryLocal_0, newGroup, parentPath_0, { combinators, combinatorPreceding: (_a2_0 = newGroup.combinatorPreceding) != null ? _a2_0 : void 0, idGenerator });
      log({ qbId, type: LogType.add, query: queryLocal_0, newQuery: newQuery_1, newGroup, parentPath: parentPath_0 });
      dispatchQuery(newQuery_1);
    },
    [combinators, dispatchQuery, idGenerator, log, maxLevels, onAddGroup, qbId, queryBuilderStore, queryDisabled]
  );
  const onPropChange = useCallback3(
    // oxlint-disable-next-line typescript/no-explicit-any
    (prop, value, path) => {
      const queryLocal_1 = getQuerySelectorById(qbId)(queryBuilderStore.getState());
      if (!queryLocal_1) return;
      if (pathIsDisabled(path, queryLocal_1) && prop !== "disabled" || queryDisabled) {
        log({ qbId, type: LogType.pathDisabled, path, prop, value, query: queryLocal_1 });
        return;
      }
      const newQuery_2 = update(queryLocal_1, prop, value, path, { resetOnFieldChange, resetOnOperatorChange, getRuleDefaultOperator, getValueSources: getValueSourcesMain, getRuleDefaultValue, getMatchModes: getMatchModesMain });
      log({ qbId, type: LogType.update, query: queryLocal_1, newQuery: newQuery_2, prop, value, path });
      dispatchQuery(newQuery_2);
    },
    [dispatchQuery, getMatchModesMain, getRuleDefaultOperator, getRuleDefaultValue, getValueSourcesMain, log, qbId, queryBuilderStore, queryDisabled, resetOnFieldChange, resetOnOperatorChange]
  );
  const onRuleOrGroupRemove = useCallback3(
    // oxlint-disable-next-line typescript/no-explicit-any
    (path_0, context_1) => {
      const queryLocal_2 = getQuerySelectorById(qbId)(queryBuilderStore.getState());
      if (!queryLocal_2) return;
      if (pathIsDisabled(path_0, queryLocal_2) || queryDisabled) {
        log({ qbId, type: LogType.pathDisabled, path: path_0, query: queryLocal_2 });
        return;
      }
      const ruleOrGroup = findPath(path_0, queryLocal_2);
      if (ruleOrGroup) {
        if (onRemove(ruleOrGroup, path_0, queryLocal_2, context_1)) {
          const newQuery_3 = remove(queryLocal_2, path_0);
          log({ qbId, type: LogType.remove, query: queryLocal_2, newQuery: newQuery_3, path: path_0, ruleOrGroup });
          dispatchQuery(newQuery_3);
        } else {
          log({ qbId, type: LogType.onRemoveFalse, ruleOrGroup, path: path_0, query: queryLocal_2 });
        }
      }
    },
    [dispatchQuery, log, onRemove, qbId, queryBuilderStore, queryDisabled]
  );
  const moveRule = useCallback3(
    // oxlint-disable-next-line typescript/no-explicit-any
    (oldPath, newPath, clone, context_2) => {
      const queryLocal_3 = getQuerySelectorById(qbId)(queryBuilderStore.getState());
      if (!queryLocal_3) return;
      if (pathIsDisabled(oldPath, queryLocal_3) || queryDisabled) {
        log({ qbId, type: LogType.pathDisabled, oldPath, newPath, query: queryLocal_3 });
        return;
      }
      const nextQuery = move(queryLocal_3, oldPath, newPath, { clone, combinators, idGenerator });
      const ruleOrGroup_0 = findPath(oldPath, queryLocal_3);
      const isGroup = isRuleGroup(ruleOrGroup_0);
      const callbackResult = (isGroup ? onMoveGroup : onMoveRule)(ruleOrGroup_0, oldPath, newPath, queryLocal_3, nextQuery, { clone, combinators }, context_2);
      if (!callbackResult) {
        log({ qbId, type: isGroup ? LogType.onMoveGroupFalse : LogType.onMoveRuleFalse, ruleOrGroup: ruleOrGroup_0, oldPath, newPath, clone, query: queryLocal_3, nextQuery });
        return;
      }
      const newQuery_4 = isRuleGroup(callbackResult) ? callbackResult : nextQuery;
      log({ qbId, type: LogType.move, query: queryLocal_3, newQuery: newQuery_4, oldPath, newPath, clone });
      dispatchQuery(newQuery_4);
    },
    [combinators, dispatchQuery, idGenerator, log, onMoveGroup, onMoveRule, qbId, queryBuilderStore, queryDisabled]
  );
  const groupRule = useCallback3(
    // oxlint-disable-next-line typescript/no-explicit-any
    (sourcePath, targetPath, clone_0, context_3) => {
      const queryLocal_4 = getQuerySelectorById(qbId)(queryBuilderStore.getState());
      if (!queryLocal_4) return;
      if (pathIsDisabled(sourcePath, queryLocal_4) || queryDisabled) {
        log({ qbId, type: LogType.pathDisabled, sourcePath, targetPath, query: queryLocal_4 });
        return;
      }
      const nextQuery_0 = group(queryLocal_4, sourcePath, targetPath, { clone: clone_0, combinators, idGenerator });
      const ruleOrGroup_1 = findPath(sourcePath, queryLocal_4);
      const isGroup_0 = isRuleGroup(ruleOrGroup_1);
      const callbackResult_0 = (isGroup_0 ? onGroupGroup : onGroupRule)(ruleOrGroup_1, sourcePath, targetPath, queryLocal_4, nextQuery_0, { clone: clone_0, combinators }, context_3);
      if (!callbackResult_0) {
        log({ qbId, type: isGroup_0 ? LogType.onGroupGroupFalse : LogType.onGroupRuleFalse, ruleOrGroup: ruleOrGroup_1, sourcePath, targetPath, clone: clone_0, query: queryLocal_4, nextQuery: nextQuery_0 });
        return;
      }
      const newQuery_5 = isRuleGroup(callbackResult_0) ? callbackResult_0 : nextQuery_0;
      log({ qbId, type: LogType.group, query: queryLocal_4, newQuery: newQuery_5, sourcePath, targetPath, clone: clone_0 });
      dispatchQuery(newQuery_5);
    },
    [combinators, dispatchQuery, idGenerator, log, onGroupGroup, onGroupRule, qbId, queryBuilderStore, queryDisabled]
  );
  const { validationResult, validationMap } = useMemo4(() => {
    const validationResult2 = typeof validator === "function" && rootGroup ? validator(rootGroup) : defaultValidationResult;
    const validationMap2 = typeof validationResult2 === "boolean" ? defaultValidationMap : validationResult2;
    return { validationResult: validationResult2, validationMap: validationMap2 };
  }, [rootGroup, validator]);
  const dndEnabledAttr = useMemo4(() => enableDragAndDrop ? "enabled" : "disabled", [enableDragAndDrop]);
  const inlineCombinatorsAttr = useMemo4(() => independentCombinators || showCombinatorsBetweenRules ? "enabled" : "disabled", [independentCombinators, showCombinatorsBetweenRules]);
  const combinatorPropObject = useMemo4(() => typeof rootGroup.combinator === "string" ? { combinator: rootGroup.combinator } : icCombinatorPropObject, [rootGroup.combinator]);
  const wrapperClassName = useMemo4(() => clsx(
    suppressStandardClassnames || standardClassnames.queryBuilder,
    clsx(controlClassnames.queryBuilder),
    // custom conditional classes
    queryDisabled && controlClassnames.disabled,
    typeof validationResult === "boolean" && validationResult && controlClassnames.valid,
    typeof validationResult === "boolean" && !validationResult && controlClassnames.invalid,
    // standard conditional classes
    suppressStandardClassnames || { [standardClassnames.disabled]: queryDisabled, [standardClassnames.valid]: typeof validationResult === "boolean" && validationResult, [standardClassnames.invalid]: typeof validationResult === "boolean" && !validationResult }
  ), [controlClassnames.disabled, controlClassnames.invalid, controlClassnames.queryBuilder, controlClassnames.valid, queryDisabled, suppressStandardClassnames, validationResult]);
  const createRuleGroupOverride = useCallback3((ic) => createRuleGroup(ic != null ? ic : independentCombinators), [createRuleGroup, independentCombinators]);
  const schema = useMemo4(() => ({ addRuleToNewGroups, accessibleDescriptionGenerator, autoSelectField, autoSelectOperator, autoSelectValue, classNames: controlClassnames, combinators, controls, createRule, createRuleGroup: createRuleGroupOverride, disabledPaths, enableDragAndDrop, fieldMap, fields, dispatchQuery, getQuery, getInputType: getInputTypeMain, getOperators: getOperatorsMain, getMatchModes: getMatchModesMain, getRuleClassname, getRuleGroupClassname, getSubQueryBuilderProps: getSubQueryBuilderPropsMain, getValueEditorSeparator, getValueEditorType: getValueEditorTypeMain, getValues: getValuesMain, getValueSources: getValueSourcesMain, independentCombinators, listsAsArrays, maxLevels, parseNumbers, qbId, showCloneButtons, showCombinatorsBetweenRules, showLockButtons, showNotToggle, showShiftActions, suppressStandardClassnames, validationMap }), [accessibleDescriptionGenerator, addRuleToNewGroups, autoSelectField, autoSelectOperator, autoSelectValue, combinators, controlClassnames, controls, createRule, createRuleGroupOverride, disabledPaths, dispatchQuery, enableDragAndDrop, fieldMap, fields, getInputTypeMain, getOperatorsMain, getMatchModesMain, getQuery, getRuleClassname, getRuleGroupClassname, getSubQueryBuilderPropsMain, getValueEditorSeparator, getValueEditorTypeMain, getValuesMain, getValueSourcesMain, independentCombinators, listsAsArrays, maxLevels, parseNumbers, qbId, showCloneButtons, showCombinatorsBetweenRules, showLockButtons, showNotToggle, showShiftActions, suppressStandardClassnames, validationMap]);
  const actions = useMemo4(() => ({ moveRule, onGroupAdd, onGroupRemove: onRuleOrGroupRemove, onPropChange, onRuleAdd, onRuleRemove: onRuleOrGroupRemove, groupRule }), [groupRule, moveRule, onGroupAdd, onPropChange, onRuleAdd, onRuleOrGroupRemove]);
  return { actions, rootGroup, rootGroupDisabled, queryDisabled, rqbContext, schema, translations, wrapperClassName, dndEnabledAttr, inlineCombinatorsAttr, combinatorPropObject };
}

// src/components/QueryBuilder.useQueryBuilderSetup.ts
import { useCallback as useCallback4, useMemo as useMemo5, useState as useState2 } from "react";
var __defProp36 = Object.defineProperty;
var __defProps35 = Object.defineProperties;
var __getOwnPropDescs35 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols36 = Object.getOwnPropertySymbols;
var __hasOwnProp36 = Object.prototype.hasOwnProperty;
var __propIsEnum36 = Object.prototype.propertyIsEnumerable;
var __defNormalProp36 = (obj, key, value) => key in obj ? __defProp36(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues36 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp36.call(b, prop)) __defNormalProp36(a, prop, b[prop]);
  if (__getOwnPropSymbols36) for (var prop of __getOwnPropSymbols36(b)) {
    if (__propIsEnum36.call(b, prop)) __defNormalProp36(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps35 = (a, b) => __defProps35(a, __getOwnPropDescs35(b));
var getFirstOptionsFrom = (opts, r, listsAsArrays) => {
  const firstOption = getFirstOption(opts);
  if (r.operator === "between" || r.operator === "notBetween") {
    const valueAsArray = [firstOption, firstOption];
    return listsAsArrays ? valueAsArray : joinWith(valueAsArray.map((v) => v != null ? v : (
      /* istanbul ignore next */
      ""
    )), ",");
  }
  return firstOption;
};
var useQueryBuilderSetup = (props) => {
  var _a;
  const [qbId] = useState2(generateID);
  const { fields: fieldsProp, baseField, operators: operatorsProp, baseOperator, combinators: combinatorsProp, baseCombinator, translations: translationsProp, enableMountQueryChange: enableMountQueryChangeProp = true, controlClassnames: controlClassnamesProp, controlElements: controlElementsProp, getDefaultField, getDefaultOperator, getDefaultValue, getMatchModes, getOperators, getSubQueryBuilderProps, getValueEditorType, getValueSources, getInputType, getValues, autoSelectField = true, autoSelectOperator = true, autoSelectValue = true, addRuleToNewGroups = false, enableDragAndDrop: enableDragAndDropProp, listsAsArrays = false, debugMode: debugModeProp = false, idGenerator = generateID } = props;
  const [initialQueryProp] = useState2((_a = props.query) != null ? _a : props.defaultQuery);
  const rqbContext = useMergedContext({ controlClassnames: controlClassnamesProp, controlElements: controlElementsProp, debugMode: debugModeProp, enableDragAndDrop: enableDragAndDropProp, enableMountQueryChange: enableMountQueryChangeProp, translations: translationsProp, initialQuery: initialQueryProp, qbId, finalize: true });
  const { translations } = rqbContext;
  const { fields, fieldMap } = useFields({ fields: fieldsProp, baseField, autoSelectField, translations });
  const combinators = useMemo5(() => toFullOptionList(combinatorsProp != null ? combinatorsProp : defaultCombinators, baseCombinator, defaultCombinatorLabelMap), [baseCombinator, combinatorsProp]);
  const operators = useMemo5(() => toFullOptionList(operatorsProp != null ? operatorsProp : defaultOperators, baseOperator, defaultOperatorLabelMap), [baseOperator, operatorsProp]);
  const defaultOperator = useMemo5(() => ({ id: translations.operators.placeholderName, name: translations.operators.placeholderName, value: translations.operators.placeholderName, label: translations.operators.placeholderLabel }), [translations.operators.placeholderLabel, translations.operators.placeholderName]);
  const getOperatorsMain = useCallback4((field, _ref) => {
    let { fieldData } = _ref;
    let opsFinal = operators;
    if (fieldData == null ? void 0 : fieldData.operators) {
      opsFinal = toFullOptionList(fieldData.operators, baseOperator, defaultOperatorLabelMap);
    } else if (getOperators) {
      const ops = getOperators(field, { fieldData });
      if (ops) {
        opsFinal = toFullOptionList(ops, baseOperator, defaultOperatorLabelMap);
      }
    }
    if (!autoSelectOperator) {
      opsFinal = isFlexibleOptionGroupArray(opsFinal) ? [{ label: translations.operators.placeholderGroupLabel, options: [defaultOperator] }, ...opsFinal] : [defaultOperator, ...opsFinal];
    }
    return uniqOptList(opsFinal);
  }, [autoSelectOperator, baseOperator, defaultOperator, getOperators, operators, translations.operators.placeholderGroupLabel]);
  const getRuleDefaultOperator = useCallback4((field_0) => {
    var _a2, _b;
    const fieldData_0 = fieldMap[field_0];
    if (fieldData_0 == null ? void 0 : fieldData_0.defaultOperator) {
      return fieldData_0.defaultOperator;
    }
    if (getDefaultOperator) {
      return typeof getDefaultOperator === "function" ? getDefaultOperator(field_0, { fieldData: fieldData_0 }) : getDefaultOperator;
    }
    const ops_0 = (_a2 = getOperatorsMain(field_0, { fieldData: fieldData_0 })) != null ? _a2 : (
      /* istanbul ignore next */
      []
    );
    return (_b = getFirstOption(ops_0)) != null ? _b : (
      /* istanbul ignore next */
      ""
    );
  }, [fieldMap, getDefaultOperator, getOperatorsMain]);
  const getValueEditorTypeMain = useCallback4((field_1, operator, _ref2) => {
    let { fieldData: fieldData_1 } = _ref2;
    var _a2_0;
    if (fieldData_1.valueEditorType) {
      if (typeof fieldData_1.valueEditorType === "function") {
        return fieldData_1.valueEditorType(operator);
      }
      return fieldData_1.valueEditorType;
    }
    return (_a2_0 = getValueEditorType == null ? void 0 : getValueEditorType(field_1, operator, { fieldData: fieldData_1 })) != null ? _a2_0 : "text";
  }, [getValueEditorType]);
  const getValueSourcesMain = useCallback4((field_2, operator_0, _misc) => getValueSourcesUtil(fieldMap[field_2], operator_0, getValueSources), [fieldMap, getValueSources]);
  const getMatchModesMain = useCallback4((field_3, _misc_0) => getMatchModesUtil(fieldMap[field_3], getMatchModes), [fieldMap, getMatchModes]);
  const getSubQueryBuilderPropsMain = useCallback4((field_4, misc) => {
    var _a2_1;
    return (
      // oxlint-disable-next-line typescript/no-explicit-any
      (_a2_1 = getSubQueryBuilderProps == null ? void 0 : getSubQueryBuilderProps(field_4, misc)) != null ? _a2_1 : {}
    );
  }, [getSubQueryBuilderProps]);
  const defaultValueOption = useMemo5(() => ({ id: translations.values.placeholderName, name: translations.values.placeholderName, value: translations.values.placeholderName, label: translations.values.placeholderLabel }), [translations.values.placeholderLabel, translations.values.placeholderName]);
  const getValuesMain = useCallback4((field_5, operator_1, _ref3) => {
    let { fieldData: fieldData_2 } = _ref3;
    let valsFinal = [];
    if (fieldData_2 == null ? void 0 : fieldData_2.values) {
      valsFinal = toFullOptionList(fieldData_2.values);
    }
    if (getValues) {
      valsFinal = toFullOptionList(getValues(field_5, operator_1, { fieldData: fieldData_2 }));
    }
    if (!autoSelectValue) {
      valsFinal = isFlexibleOptionGroupArray(valsFinal) ? [{ label: translations.values.placeholderGroupLabel, options: [defaultValueOption] }, ...valsFinal] : [defaultValueOption, ...valsFinal];
    }
    return valsFinal;
  }, [autoSelectValue, defaultValueOption, getValues, translations.values.placeholderGroupLabel]);
  const getRuleDefaultValue = useCallback4((r) => {
    var _a2_2;
    const fieldData_3 = (_a2_2 = fieldMap[r.field]) != null ? _a2_2 : {};
    if ((fieldData_3 == null ? void 0 : fieldData_3.defaultValue) !== void 0 && fieldData_3.defaultValue !== null) {
      return fieldData_3.defaultValue;
    } else if (getDefaultValue) {
      return getDefaultValue(r, { fieldData: fieldData_3 });
    }
    let value = "";
    const values = getValuesMain(r.field, r.operator, { fieldData: fieldData_3 });
    if (r.valueSource === "field") {
      const filteredFields = filterFieldsByComparator(fieldData_3, fields, r.operator);
      value = filteredFields.length > 0 ? getFirstOptionsFrom(filteredFields, r, listsAsArrays) : "";
    } else if (values.length > 0) {
      const editorType = getValueEditorTypeMain(r.field, r.operator, { fieldData: fieldData_3 });
      if (editorType === "multiselect") {
        value = listsAsArrays ? [] : "";
      } else if (editorType === "select" || editorType === "radio") {
        value = getFirstOptionsFrom(values, r, listsAsArrays);
      }
    } else {
      const editorType_0 = getValueEditorTypeMain(r.field, r.operator, { fieldData: fieldData_3 });
      if (editorType_0 === "checkbox") {
        value = false;
      }
    }
    return value;
  }, [fieldMap, fields, getDefaultValue, getValueEditorTypeMain, getValuesMain, listsAsArrays]);
  const getInputTypeMain = useCallback4((field_6, operator_2, _ref4) => {
    let { fieldData: fieldData_4 } = _ref4;
    if (getInputType) {
      const inputType = getInputType(field_6, operator_2, { fieldData: fieldData_4 });
      if (inputType) return inputType;
    }
    return "text";
  }, [getInputType]);
  const createRule = useCallback4(() => {
    var _a2_3;
    let field_7 = "";
    const flds = fields;
    if ((flds == null ? void 0 : flds.length) > 0 && flds[0]) {
      const fo = getFirstOption(flds);
      if (fo) field_7 = fo;
    }
    if (getDefaultField) {
      if (typeof getDefaultField === "function") {
        const df = getDefaultField(flds);
        if (df) field_7 = df;
      } else {
        field_7 = getDefaultField;
      }
    }
    const operator_3 = getRuleDefaultOperator(field_7);
    const valueSource = (_a2_3 = getFirstOption(getValueSourcesMain(field_7, operator_3, { fieldData: getOption(flds, field_7) }))) != null ? _a2_3 : "value";
    const matchMode = getFirstOption(getMatchModesMain(field_7, { fieldData: getOption(flds, field_7) }));
    const newRule = __spreadValues36({ id: idGenerator(), field: field_7, operator: operator_3, valueSource, value: "" }, matchMode ? { match: { mode: matchMode, threshold: 1 } } : null);
    const value_0 = getRuleDefaultValue(newRule);
    return __spreadProps35(__spreadValues36({}, newRule), { value: value_0 });
  }, [fields, getDefaultField, getMatchModesMain, getRuleDefaultOperator, getRuleDefaultValue, getValueSourcesMain, idGenerator]);
  const createRuleGroup = useCallback4((independentCombinators) => {
    var _a2_4;
    if (independentCombinators) {
      return { id: idGenerator(), rules: addRuleToNewGroups ? [createRule()] : [], not: false };
    }
    return { id: idGenerator(), rules: addRuleToNewGroups ? [createRule()] : [], combinator: (_a2_4 = getFirstOption(combinators)) != null ? _a2_4 : (
      /* istanbul ignore next */
      ""
    ), not: false };
  }, [addRuleToNewGroups, combinators, createRule, idGenerator]);
  return {
    qbId,
    rqbContext,
    // TODO: Why is a cast necessary here?
    fields,
    fieldMap,
    combinators,
    getMatchModesMain,
    getOperatorsMain,
    getRuleDefaultOperator,
    getSubQueryBuilderPropsMain,
    getValueEditorTypeMain,
    getValueSourcesMain,
    getValuesMain,
    getRuleDefaultValue,
    getInputTypeMain,
    createRule,
    createRuleGroup
  };
};

// src/components/QueryBuilder.useQueryBuilder.ts
var useQueryBuilder = (props) => {
  return useQueryBuilderSchema(props, useQueryBuilderSetup(props));
};

// src/components/RuleGroup.tsx
import { jsx as jsx8, jsxs as jsxs3 } from "react/jsx-runtime";
import * as React6 from "react";
import { Fragment as Fragment2, useCallback as useCallback5, useMemo as useMemo6 } from "react";
var __defProp37 = Object.defineProperty;
var __defProps36 = Object.defineProperties;
var __getOwnPropDescs36 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols37 = Object.getOwnPropertySymbols;
var __hasOwnProp37 = Object.prototype.hasOwnProperty;
var __propIsEnum37 = Object.prototype.propertyIsEnumerable;
var __defNormalProp37 = (obj, key, value) => key in obj ? __defProp37(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues37 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp37.call(b, prop)) __defNormalProp37(a, prop, b[prop]);
  if (__getOwnPropSymbols37) for (var prop of __getOwnPropSymbols37(b)) {
    if (__propIsEnum37.call(b, prop)) __defNormalProp37(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps36 = (a, b) => __defProps36(a, __getOwnPropDescs36(b));
var RuleGroup = /* @__PURE__ */ React6.memo(function RuleGroup2(props) {
  const rg = useRuleGroup(props);
  const { schema: { controls: { ruleGroupBodyElements: RuleGroupBodyElements, ruleGroupHeaderElements: RuleGroupHeaderElements } } } = rg;
  const addRule = useStopEventPropagation(rg.addRule);
  const addGroup = useStopEventPropagation(rg.addGroup);
  const cloneGroup = useStopEventPropagation(rg.cloneGroup);
  const toggleLockGroup = useStopEventPropagation(rg.toggleLockGroup);
  const removeGroup = useStopEventPropagation(rg.removeGroup);
  const shiftGroupUp = useStopEventPropagation(rg.shiftGroupUp);
  const shiftGroupDown = useStopEventPropagation(rg.shiftGroupDown);
  const actions = useMemo6(() => ({ addRule, addGroup, cloneGroup, toggleLockGroup, removeGroup, shiftGroupUp, shiftGroupDown }), [addRule, addGroup, cloneGroup, toggleLockGroup, removeGroup, shiftGroupUp, shiftGroupDown]);
  return /* @__PURE__ */ jsxs3("div", { ref: rg.previewRef, title: rg.accessibleDescription, className: rg.outerClassName, "data-testid": TestID.ruleGroup, "data-dragmonitorid": rg.dragMonitorId, "data-dropmonitorid": rg.dropMonitorId, "data-rule-group-id": rg.id, "data-level": rg.path.length, "data-path": JSON.stringify(rg.path), children: [/* @__PURE__ */ jsx8("div", { ref: rg.dropRef, className: rg.classNames.header, children: /* @__PURE__ */ jsx8(RuleGroupHeaderElements, __spreadValues37(__spreadValues37({}, rg), actions)) }), /* @__PURE__ */ jsx8("div", { className: rg.classNames.body, children: /* @__PURE__ */ jsx8(RuleGroupBodyElements, __spreadValues37(__spreadValues37({}, rg), actions)) })] });
});
var RuleGroupHeaderComponents = /* @__PURE__ */ React6.memo(function RuleGroupHeaderComponents2(rg) {
  const { schema: { controls: { shiftActions: ShiftActionsControlElement, dragHandle: DragHandleControlElement, combinatorSelector: CombinatorSelectorControlElement, notToggle: NotToggleControlElement, addRuleAction: AddRuleActionControlElement, addGroupAction: AddGroupActionControlElement, cloneGroupAction: CloneGroupActionControlElement, lockGroupAction: LockGroupActionControlElement, removeGroupAction: RemoveGroupActionControlElement } } } = rg;
  const commonSubcomponentProps = useMemo6(() => ({ level: rg.path.length, path: rg.path, disabled: rg.disabled, context: rg.context, validation: rg.validationResult, schema: rg.schema, ruleOrGroup: rg.ruleGroup, ruleGroup: rg.ruleGroup }), [rg.path, rg.disabled, rg.context, rg.validationResult, rg.schema, rg.ruleGroup]);
  const shiftTitles = useMemo6(() => rg.schema.showShiftActions ? { shiftUp: rg.translations.shiftActionUp.title, shiftDown: rg.translations.shiftActionDown.title } : void 0, [rg.schema.showShiftActions, rg.translations]);
  const shiftLabels = useMemo6(() => rg.schema.showShiftActions ? { shiftUp: rg.translations.shiftActionUp.label, shiftDown: rg.translations.shiftActionDown.label } : void 0, [rg.schema.showShiftActions, rg.translations]);
  return /* @__PURE__ */ jsxs3(Fragment2, { children: [rg.schema.showShiftActions && rg.path.length > 0 && /* @__PURE__ */ jsx8(ShiftActionsControlElement, __spreadProps36(__spreadValues37({}, commonSubcomponentProps), { testID: TestID.shiftActions, titles: shiftTitles, labels: shiftLabels, className: rg.classNames.shiftActions, shiftUp: rg.shiftGroupUp, shiftDown: rg.shiftGroupDown, shiftUpDisabled: rg.shiftUpDisabled, shiftDownDisabled: rg.shiftDownDisabled }), TestID.shiftActions), rg.path.length > 0 && rg.schema.enableDragAndDrop && /* @__PURE__ */ jsx8(DragHandleControlElement, __spreadProps36(__spreadValues37({}, commonSubcomponentProps), { testID: TestID.dragHandle, ref: rg.dragRef, title: rg.translations.dragHandle.title, label: rg.translations.dragHandle.label, className: rg.classNames.dragHandle }), TestID.dragHandle), !rg.schema.showCombinatorsBetweenRules && !rg.schema.independentCombinators && /* @__PURE__ */ jsx8(CombinatorSelectorControlElement, __spreadProps36(__spreadValues37({}, commonSubcomponentProps), { testID: TestID.combinators, options: rg.schema.combinators, value: rg.combinator, title: rg.translations.combinators.title, className: rg.classNames.combinators, handleOnChange: rg.onCombinatorChange, rules: rg.ruleGroup.rules }), TestID.combinators), rg.schema.showNotToggle && /* @__PURE__ */ jsx8(NotToggleControlElement, __spreadProps36(__spreadValues37({}, commonSubcomponentProps), { testID: TestID.notToggle, className: rg.classNames.notToggle, title: rg.translations.notToggle.title, label: rg.translations.notToggle.label, checked: rg.ruleGroup.not, handleOnChange: rg.onNotToggleChange }), TestID.notToggle), /* @__PURE__ */ jsx8(AddRuleActionControlElement, __spreadProps36(__spreadValues37({}, commonSubcomponentProps), { testID: TestID.addRule, label: rg.translations.addRule.label, title: rg.translations.addRule.title, className: rg.classNames.addRule, handleOnClick: rg.addRule, rules: rg.ruleGroup.rules }), TestID.addRule), rg.schema.maxLevels > rg.path.length && /* @__PURE__ */ jsx8(AddGroupActionControlElement, __spreadProps36(__spreadValues37({}, commonSubcomponentProps), { testID: TestID.addGroup, label: rg.translations.addGroup.label, title: rg.translations.addGroup.title, className: rg.classNames.addGroup, handleOnClick: rg.addGroup, rules: rg.ruleGroup.rules }), TestID.addGroup), rg.schema.showCloneButtons && rg.path.length > 0 && /* @__PURE__ */ jsx8(CloneGroupActionControlElement, __spreadProps36(__spreadValues37({}, commonSubcomponentProps), { testID: TestID.cloneGroup, label: rg.translations.cloneRuleGroup.label, title: rg.translations.cloneRuleGroup.title, className: rg.classNames.cloneGroup, handleOnClick: rg.cloneGroup, rules: rg.ruleGroup.rules }), TestID.cloneGroup), rg.schema.showLockButtons && /* @__PURE__ */ jsx8(LockGroupActionControlElement, __spreadProps36(__spreadValues37({}, commonSubcomponentProps), { testID: TestID.lockGroup, label: rg.translations.lockGroup.label, title: rg.translations.lockGroup.title, className: rg.classNames.lockGroup, handleOnClick: rg.toggleLockGroup, rules: rg.ruleGroup.rules, disabledTranslation: rg.parentDisabled ? void 0 : rg.translations.lockGroupDisabled }), TestID.lockGroup), rg.path.length > 0 && /* @__PURE__ */ jsx8(RemoveGroupActionControlElement, __spreadProps36(__spreadValues37({}, commonSubcomponentProps), { testID: TestID.removeGroup, label: rg.translations.removeGroup.label, title: rg.translations.removeGroup.title, className: rg.classNames.removeGroup, handleOnClick: rg.removeGroup, rules: rg.ruleGroup.rules }), TestID.removeGroup)] });
});
var RuleGroupBodyComponents = /* @__PURE__ */ React6.memo(function RuleGroupBodyComponents2(rg) {
  const { schema: { controls: { combinatorSelector: CombinatorSelectorControlElement, inlineCombinator: InlineCombinatorControlElement, ruleGroup: RuleGroupControlElement, rule: RuleControlElement } } } = rg;
  return /* @__PURE__ */ jsx8(Fragment2, { children: rg.ruleGroup.rules.map((r, idx, _ref) => {
    let { length: ruleArrayLength } = _ref;
    const thisPathMemo = rg.pathsMemo[idx];
    const thisPath = thisPathMemo.path;
    const thisPathDisabled = thisPathMemo.disabled || typeof r !== "string" && r.disabled;
    const shiftUpDisabled = pathsAreEqual([0], thisPath);
    const shiftDownDisabled = rg.path.length === 0 && idx === ruleArrayLength - 1;
    const key = typeof r === "string" ? [...thisPath, r].join("-") : r.id;
    return /* @__PURE__ */ jsxs3(Fragment2, { children: [idx > 0 && !rg.schema.independentCombinators && rg.schema.showCombinatorsBetweenRules && /* @__PURE__ */ jsx8(InlineCombinatorControlElement, { options: rg.schema.combinators, value: rg.combinator, title: rg.translations.combinators.title, className: rg.classNames.combinators, handleOnChange: rg.onCombinatorChange, rules: rg.ruleGroup.rules, level: rg.path.length, context: rg.context, validation: rg.validationResult, component: CombinatorSelectorControlElement, path: thisPath, disabled: rg.disabled, schema: rg.schema }, TestID.inlineCombinator), typeof r === "string" ? /* @__PURE__ */ jsx8(InlineCombinatorControlElement, { options: rg.schema.combinators, value: r, title: rg.translations.combinators.title, className: rg.classNames.combinators, handleOnChange: (val) => rg.onIndependentCombinatorChange(val, idx), rules: rg.ruleGroup.rules, level: rg.path.length, context: rg.context, validation: rg.validationResult, component: CombinatorSelectorControlElement, path: thisPath, disabled: thisPathDisabled, schema: rg.schema }, `${TestID.inlineCombinator}-independent`) : isRuleGroup(r) ? /* @__PURE__ */ jsx8(RuleGroupControlElement, { id: r.id, schema: rg.schema, actions: rg.actions, path: thisPath, translations: rg.translations, ruleGroup: r, rules: r.rules, combinator: isRuleGroupType(r) ? r.combinator : void 0, not: !!r.not, disabled: thisPathDisabled, parentDisabled: rg.parentDisabled || rg.disabled, shiftUpDisabled, shiftDownDisabled, context: rg.context }, TestID.ruleGroup) : /* @__PURE__ */ jsx8(RuleControlElement, { id: r.id, rule: r, field: r.field, operator: r.operator, value: r.value, valueSource: r.valueSource, schema: rg.schema, actions: rg.actions, path: thisPath, disabled: thisPathDisabled, parentDisabled: rg.parentDisabled || rg.disabled, translations: rg.translations, shiftUpDisabled, shiftDownDisabled, context: rg.context }, TestID.rule)] }, key);
  }) });
});
var useRuleGroup = (props) => {
  const {
    id,
    path,
    ruleGroup: ruleGroupProp,
    schema: { qbId, accessibleDescriptionGenerator, classNames: classNamesProp, combinators, createRule, createRuleGroup, disabledPaths, independentCombinators, validationMap, enableDragAndDrop, getRuleGroupClassname, suppressStandardClassnames },
    actions: { onGroupAdd, onGroupRemove, onPropChange, onRuleAdd, moveRule },
    disabled: disabledProp,
    parentDisabled,
    shiftUpDisabled,
    shiftDownDisabled,
    combinator: combinatorProp,
    rules: rulesProp,
    not: notProp,
    // Drag-and-drop
    dropEffect = "move",
    groupItems = false,
    dragMonitorId = "",
    dropMonitorId = "",
    previewRef = null,
    dragRef = null,
    dropRef = null,
    isDragging = false,
    isOver = false
  } = props;
  useDeprecatedProps("ruleGroup", !ruleGroupProp);
  useReactDndWarning(enableDragAndDrop, !!(dragMonitorId || dropMonitorId || previewRef || dragRef || dropRef));
  const disabled = !!parentDisabled || !!disabledProp;
  const combinator = useMemo6(() => ruleGroupProp && isRuleGroupType(ruleGroupProp) ? ruleGroupProp.combinator : ruleGroupProp ? getFirstOption(combinators) : combinatorProp != null ? combinatorProp : getFirstOption(combinators), [combinatorProp, combinators, ruleGroupProp]);
  const ruleGroup = useMemo6(() => {
    if (ruleGroupProp) {
      if (ruleGroupProp.combinator === combinator || independentCombinators) {
        return ruleGroupProp;
      }
      const newRG = structuredClone(ruleGroupProp);
      newRG.combinator = combinator;
      return newRG;
    }
    return { rules: rulesProp, not: notProp };
  }, [combinator, independentCombinators, notProp, ruleGroupProp, rulesProp]);
  const classNames = useMemo6(() => ({ header: clsx(suppressStandardClassnames || standardClassnames.header, classNamesProp.header, isOver && dropEffect === "copy" && classNamesProp.dndCopy, suppressStandardClassnames || { [standardClassnames.dndOver]: isOver, [standardClassnames.dndCopy]: isOver && dropEffect === "copy" }), shiftActions: clsx(suppressStandardClassnames || standardClassnames.shiftActions, classNamesProp.shiftActions), dragHandle: clsx(suppressStandardClassnames || standardClassnames.dragHandle, classNamesProp.dragHandle), combinators: clsx(suppressStandardClassnames || standardClassnames.combinators, classNamesProp.valueSelector, classNamesProp.combinators), notToggle: clsx(suppressStandardClassnames || standardClassnames.notToggle, classNamesProp.notToggle), addRule: clsx(suppressStandardClassnames || standardClassnames.addRule, classNamesProp.actionElement, classNamesProp.addRule), addGroup: clsx(suppressStandardClassnames || standardClassnames.addGroup, classNamesProp.actionElement, classNamesProp.addGroup), cloneGroup: clsx(suppressStandardClassnames || standardClassnames.cloneGroup, classNamesProp.actionElement, classNamesProp.cloneGroup), lockGroup: clsx(suppressStandardClassnames || standardClassnames.lockGroup, classNamesProp.actionElement, classNamesProp.lockGroup), removeGroup: clsx(suppressStandardClassnames || standardClassnames.removeGroup, classNamesProp.actionElement, classNamesProp.removeGroup), body: clsx(suppressStandardClassnames || standardClassnames.body, classNamesProp.body) }), [classNamesProp.actionElement, classNamesProp.addGroup, classNamesProp.addRule, classNamesProp.body, classNamesProp.cloneGroup, classNamesProp.combinators, classNamesProp.dndCopy, classNamesProp.dragHandle, classNamesProp.header, classNamesProp.lockGroup, classNamesProp.notToggle, classNamesProp.removeGroup, classNamesProp.shiftActions, classNamesProp.valueSelector, dropEffect, isOver, suppressStandardClassnames]);
  const onCombinatorChange = useCallback5((value) => {
    if (!disabled) {
      onPropChange("combinator", value, path);
    }
  }, [disabled, onPropChange, path]);
  const onIndependentCombinatorChange = useCallback5(
    // oxlint-disable-next-line typescript/no-explicit-any
    (value_0, index, _context) => {
      if (!disabled) {
        onPropChange("combinator", value_0, [...path, index]);
      }
    },
    [disabled, onPropChange, path]
  );
  const onNotToggleChange = useCallback5(
    // oxlint-disable-next-line typescript/no-explicit-any
    (checked, _context_0) => {
      if (!disabled) {
        onPropChange("not", checked, path);
      }
    },
    [disabled, onPropChange, path]
  );
  const addRule = useCallback5((_e, context) => {
    if (!disabled) {
      const newRule = createRule();
      onRuleAdd(newRule, path, context);
    }
  }, [createRule, disabled, onRuleAdd, path]);
  const addGroup = useCallback5((_e_0, context_0) => {
    if (!disabled) {
      const newGroup = createRuleGroup();
      onGroupAdd(newGroup, path, context_0);
    }
  }, [createRuleGroup, disabled, onGroupAdd, path]);
  const cloneGroup = useCallback5(() => {
    if (!disabled) {
      const newPath = [...getParentPath(path), path.at(-1) + 1];
      moveRule(path, newPath, true);
    }
  }, [disabled, moveRule, path]);
  const shiftGroupUp = useCallback5(
    // oxlint-disable-next-line typescript/no-explicit-any
    (event, _context_1) => {
      if (!disabled && !shiftUpDisabled) {
        moveRule(path, "up", event == null ? void 0 : event.altKey);
      }
    },
    [disabled, moveRule, path, shiftUpDisabled]
  );
  const shiftGroupDown = useCallback5(
    // oxlint-disable-next-line typescript/no-explicit-any
    (event_0, _context_2) => {
      if (!disabled && !shiftDownDisabled) {
        moveRule(path, "down", event_0 == null ? void 0 : event_0.altKey);
      }
    },
    [disabled, moveRule, path, shiftDownDisabled]
  );
  const toggleLockGroup = useCallback5(() => {
    onPropChange("disabled", !disabled, path);
  }, [disabled, onPropChange, path]);
  const removeGroup = useCallback5(() => {
    if (!disabled) {
      onGroupRemove(path);
    }
  }, [disabled, onGroupRemove, path]);
  const validationResult = useMemo6(() => validationMap[id != null ? id : (
    /* istanbul ignore next */
    ""
  )], [id, validationMap]);
  const validationClassName = useMemo6(() => getValidationClassNames(validationResult), [validationResult]);
  const combinatorBasedClassName = useMemo6(() => {
    var _a, _b;
    return independentCombinators ? null : (_b = (_a = getOption(combinators, combinator)) == null ? void 0 : _a.className) != null ? _b : "";
  }, [combinator, combinators, independentCombinators]);
  const ruleGroupClassname = useMemo6(() => getRuleGroupClassname(ruleGroup), [getRuleGroupClassname, ruleGroup]);
  const outerClassName = useMemo6(() => clsx(ruleGroupClassname, combinatorBasedClassName, suppressStandardClassnames || standardClassnames.ruleGroup, classNamesProp.ruleGroup, disabled && classNamesProp.disabled, isDragging && classNamesProp.dndDragging, isOver && groupItems && classNamesProp.dndGroup, suppressStandardClassnames || { [standardClassnames.disabled]: disabled, [standardClassnames.dndDragging]: isDragging, [standardClassnames.dndGroup]: isOver && groupItems }, validationClassName), [classNamesProp.disabled, classNamesProp.dndDragging, classNamesProp.dndGroup, classNamesProp.ruleGroup, combinatorBasedClassName, disabled, groupItems, isDragging, isOver, ruleGroupClassname, suppressStandardClassnames, validationClassName]);
  const pathsMemo = useMemo6(() => {
    const paths = [];
    for (let i = 0; i < ruleGroup.rules.length; i++) {
      const thisPath = [...path, i];
      paths[i] = { path: thisPath, disabled: disabled || disabledPaths.some((p) => pathsAreEqual(thisPath, p)) };
    }
    return paths;
  }, [disabled, path, ruleGroup.rules.length, disabledPaths]);
  const accessibleDescription = useMemo6(() => accessibleDescriptionGenerator({ path, qbId }), [accessibleDescriptionGenerator, path, qbId]);
  return __spreadProps36(__spreadValues37({}, props), { addGroup, addRule, accessibleDescription, classNames, cloneGroup, combinator, disabled, dragMonitorId, dragRef, dropMonitorId, dropRef, isDragging, isOver, onCombinatorChange, onGroupAdd, onIndependentCombinatorChange, onNotToggleChange, outerClassName, parentDisabled, pathsMemo, previewRef, removeGroup, ruleGroup, shiftGroupUp, shiftGroupDown, toggleLockGroup, validationClassName, validationResult });
};

// src/components/Rule.tsx
var __defProp38 = Object.defineProperty;
var __defProps37 = Object.defineProperties;
var __getOwnPropDescs37 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols38 = Object.getOwnPropertySymbols;
var __hasOwnProp38 = Object.prototype.hasOwnProperty;
var __propIsEnum38 = Object.prototype.propertyIsEnumerable;
var __defNormalProp38 = (obj, key, value) => key in obj ? __defProp38(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues38 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp38.call(b, prop)) __defNormalProp38(a, prop, b[prop]);
  if (__getOwnPropSymbols38) for (var prop of __getOwnPropSymbols38(b)) {
    if (__propIsEnum38.call(b, prop)) __defNormalProp38(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps37 = (a, b) => __defProps37(a, __getOwnPropDescs37(b));
var defaultMatch = { mode: "all" };
var defaultSubproperties = [{ name: "", value: "", label: "" }];
var Rule = /* @__PURE__ */ React7.memo(function Rule2(props) {
  const r = useRule(props);
  const cloneRule = useStopEventPropagation(r.cloneRule);
  const toggleLockRule = useStopEventPropagation(r.toggleLockRule);
  const removeRule = useStopEventPropagation(r.removeRule);
  const shiftRuleUp = useStopEventPropagation(r.shiftRuleUp);
  const shiftRuleDown = useStopEventPropagation(r.shiftRuleDown);
  const actions = useMemo7(() => ({ cloneRule, toggleLockRule, removeRule, shiftRuleUp, shiftRuleDown }), [cloneRule, removeRule, shiftRuleDown, shiftRuleUp, toggleLockRule]);
  return /* @__PURE__ */ jsx9("div", { ref: r.dndRef, "data-testid": TestID.rule, "data-dragmonitorid": r.dragMonitorId, "data-dropmonitorid": r.dropMonitorId, className: r.outerClassName, "data-rule-id": r.id, "data-level": r.path.length, "data-path": JSON.stringify(r.path), children: r.matchModes.length > 0 ? /* @__PURE__ */ jsx9(RuleComponentsWithSubQuery, __spreadValues38(__spreadValues38({}, r), actions)) : /* @__PURE__ */ jsx9(RuleComponents, __spreadValues38(__spreadValues38({}, r), actions)) });
});
var RuleComponents = /* @__PURE__ */ React7.memo(function RuleComponents2(r) {
  var _a, _b, _c;
  const { schema: { controls: { shiftActions: ShiftActionsControlElement, dragHandle: DragHandleControlElement, fieldSelector: FieldSelectorControlElement, matchModeEditor: MatchModeEditorControlElement, operatorSelector: OperatorSelectorControlElement, valueSourceSelector: ValueSourceSelectorControlElement, valueEditor: ValueEditorControlElement, cloneRuleAction: CloneRuleActionControlElement, lockRuleAction: LockRuleActionControlElement, removeRuleAction: RemoveRuleActionControlElement, ruleGroupBodyElements: RuleGroupBodyControlElements, ruleGroupHeaderElements: RuleGroupHeaderControlElements } }, groupComponentsWrapper: GroupComponentsWrapper = React7.Fragment } = r;
  const commonSubcomponentProps = useMemo7(() => ({ level: r.path.length, path: r.path, disabled: r.disabled, context: r.context, validation: r.validationResult, schema: r.schema, rule: r.rule }), [r.path, r.disabled, r.context, r.validationResult, r.schema, r.rule]);
  const showFieldSelector = useMemo7(() => !(r.schema.fields.length === 1 && isPojo(r.schema.fields[0]) && "value" in r.schema.fields[0] && r.schema.fields[0].value === ""), [r.schema.fields]);
  const shiftTitles = useMemo7(() => r.schema.showShiftActions ? { shiftUp: r.translations.shiftActionUp.title, shiftDown: r.translations.shiftActionDown.title } : void 0, [r.schema.showShiftActions, r.translations]);
  const shiftLabels = useMemo7(() => r.schema.showShiftActions ? { shiftUp: r.translations.shiftActionUp.label, shiftDown: r.translations.shiftActionDown.label } : void 0, [r.schema.showShiftActions, r.translations]);
  return /* @__PURE__ */ jsxs4(React7.Fragment, { children: [r.schema.showShiftActions && /* @__PURE__ */ jsx9(ShiftActionsControlElement, __spreadProps37(__spreadValues38({}, commonSubcomponentProps), { testID: TestID.shiftActions, titles: shiftTitles, labels: shiftLabels, className: r.classNames.shiftActions, ruleOrGroup: r.rule, shiftUp: r.shiftRuleUp, shiftDown: r.shiftRuleDown, shiftUpDisabled: r.shiftUpDisabled, shiftDownDisabled: r.shiftDownDisabled }), TestID.shiftActions), r.schema.enableDragAndDrop && /* @__PURE__ */ jsx9(DragHandleControlElement, __spreadProps37(__spreadValues38({}, commonSubcomponentProps), { testID: TestID.dragHandle, ref: r.dragRef, title: r.translations.dragHandle.title, label: r.translations.dragHandle.label, className: r.classNames.dragHandle, ruleOrGroup: r.rule }), TestID.dragHandle), showFieldSelector && /* @__PURE__ */ jsx9(FieldSelectorControlElement, __spreadProps37(__spreadValues38({}, commonSubcomponentProps), { testID: TestID.fields, options: r.schema.fields, title: r.translations.fields.title, value: r.rule.field, operator: r.rule.operator, className: r.classNames.fields, handleOnChange: r.onChangeField }), TestID.fields), (r.schema.autoSelectField || r.rule.field !== r.translations.fields.placeholderName) && (r.subQuery ? /* @__PURE__ */ jsx9(MatchModeEditorControlElement, __spreadProps37(__spreadValues38({}, commonSubcomponentProps), { testID: TestID.matchModeEditor, field: r.rule.field, fieldData: r.fieldData, title: r.translations.matchMode.title, options: r.matchModes, match: (_a = r.rule.match) != null ? _a : (
    /* istanbul ignore next */
    defaultMatch
  ), className: r.classNames.matchMode, classNames: r.classNames, handleOnChange: r.onChangeMatchMode }), TestID.matchModeEditor) : /* @__PURE__ */ jsxs4(React7.Fragment, { children: [/* @__PURE__ */ jsx9(OperatorSelectorControlElement, __spreadProps37(__spreadValues38({}, commonSubcomponentProps), { testID: TestID.operators, field: r.rule.field, fieldData: r.fieldData, title: r.translations.operators.title, options: r.operators, value: r.rule.operator, className: r.classNames.operators, handleOnChange: r.onChangeOperator }), TestID.operators), (r.schema.autoSelectOperator || r.rule.operator !== r.translations.operators.placeholderName) && !r.hideValueControls && /* @__PURE__ */ jsxs4(React7.Fragment, { children: [!["null", "notnull"].includes(lc(`${r.rule.operator}`)) && r.valueSources.length > 1 && /* @__PURE__ */ jsx9(ValueSourceSelectorControlElement, __spreadProps37(__spreadValues38({}, commonSubcomponentProps), { testID: TestID.valueSourceSelector, field: r.rule.field, fieldData: r.fieldData, title: r.translations.valueSourceSelector.title, options: r.valueSourceOptions, value: (_b = r.rule.valueSource) != null ? _b : "value", className: r.classNames.valueSource, handleOnChange: r.onChangeValueSource }), TestID.valueSourceSelector), /* @__PURE__ */ jsx9(ValueEditorControlElement, __spreadProps37(__spreadValues38({}, commonSubcomponentProps), { testID: TestID.valueEditor, field: r.rule.field, fieldData: r.fieldData, title: r.translations.value.title, operator: r.rule.operator, value: r.rule.value, valueSource: (_c = r.rule.valueSource) != null ? _c : "value", type: r.valueEditorType, inputType: r.inputType, values: r.values, listsAsArrays: r.schema.listsAsArrays, parseNumbers: r.schema.parseNumbers, separator: r.valueEditorSeparator, className: r.classNames.value, handleOnChange: r.onChangeValue }), TestID.valueEditor)] })] })), r.subQuery && /* @__PURE__ */ jsx9(GroupComponentsWrapper, { className: r.subQuery.classNames.header, children: /* @__PURE__ */ jsx9(RuleGroupHeaderControlElements, __spreadValues38({}, r.subQuery)) }), r.schema.showCloneButtons && /* @__PURE__ */ jsx9(CloneRuleActionControlElement, __spreadProps37(__spreadValues38({}, commonSubcomponentProps), { testID: TestID.cloneRule, label: r.translations.cloneRule.label, title: r.translations.cloneRule.title, className: r.classNames.cloneRule, ruleOrGroup: r.rule, handleOnClick: r.cloneRule }), TestID.cloneRule), r.schema.showLockButtons && /* @__PURE__ */ jsx9(LockRuleActionControlElement, __spreadProps37(__spreadValues38({}, commonSubcomponentProps), { testID: TestID.lockRule, label: r.translations.lockRule.label, title: r.translations.lockRule.title, className: r.classNames.lockRule, ruleOrGroup: r.rule, handleOnClick: r.toggleLockRule, disabledTranslation: r.parentDisabled ? void 0 : r.translations.lockRuleDisabled }), TestID.lockRule), /* @__PURE__ */ jsx9(RemoveRuleActionControlElement, __spreadProps37(__spreadValues38({}, commonSubcomponentProps), { testID: TestID.removeRule, label: r.translations.removeRule.label, title: r.translations.removeRule.title, className: r.classNames.removeRule, ruleOrGroup: r.rule, handleOnClick: r.removeRule }), TestID.removeRule), r.subQuery && /* @__PURE__ */ jsx9(GroupComponentsWrapper, { className: r.subQuery.classNames.body, children: /* @__PURE__ */ jsx9(RuleGroupBodyControlElements, __spreadValues38({}, r.subQuery)) })] });
});
var RuleWithSubQueryGroupComponentsWrapper = (props) => /* @__PURE__ */ jsx9("div", __spreadValues38({}, props));
var RuleComponentsWithSubQuery = /* @__PURE__ */ React7.memo(function RuleComponentsWithSubQuery2(r) {
  var _a;
  const initialQuery = useMemo7(() => r.schema.createRuleGroup(), [r.schema]);
  const subQB = useQueryBuilder(__spreadProps37(__spreadValues38({}, r.subQueryBuilderProps), {
    enableDragAndDrop: false,
    disabled: r.disabled,
    fields: r.subproperties.fields,
    // Update the value on first render if the value is not a valid rule group
    enableMountQueryChange: !isRuleGroup(r.rule.value) || !r.rule.value.id,
    query: isRuleGroup(r.rule.value) ? r.rule.value : initialQuery,
    onQueryChange: r.onChangeValue
  }));
  const subQuery = useRuleGroup(__spreadProps37(__spreadValues38({}, subQB), { ruleGroup: subQB.rootGroup, path: rootPath, disabled: r.disabled, parentDisabled: subQB.queryDisabled, id: subQB.rootGroup.id, shiftUpDisabled: true, shiftDownDisabled: true }));
  const addRule = useStopEventPropagation(subQuery.addRule);
  const addGroup = useStopEventPropagation(subQuery.addGroup);
  const cloneGroup = useStopEventPropagation(subQuery.cloneGroup);
  const toggleLockGroup = useStopEventPropagation(subQuery.toggleLockGroup);
  const removeGroup = useStopEventPropagation(subQuery.removeGroup);
  const shiftGroupUp = useStopEventPropagation(subQuery.shiftGroupUp);
  const shiftGroupDown = useStopEventPropagation(subQuery.shiftGroupDown);
  const memoizedSubQuery = useMemo7(() => __spreadProps37(__spreadValues38({}, subQuery), { addGroup, addRule, cloneGroup, removeGroup, shiftGroupDown, shiftGroupUp, toggleLockGroup }), [addGroup, addRule, cloneGroup, removeGroup, shiftGroupDown, shiftGroupUp, subQuery, toggleLockGroup]);
  return /* @__PURE__ */ jsx9(RuleComponents, __spreadProps37(__spreadValues38({}, r), { groupComponentsWrapper: (_a = r.groupComponentsWrapper) != null ? _a : RuleWithSubQueryGroupComponentsWrapper, subQuery: memoizedSubQuery }));
});
var useRule = (props) => {
  var _a, _b;
  const {
    id,
    path,
    rule: ruleProp,
    schema: { classNames: classNamesProp, fields, fieldMap, getInputType, getMatchModes, getOperators, getSubQueryBuilderProps, getValueEditorType, getValueEditorSeparator, getValueSources, getValues, validationMap, enableDragAndDrop, getRuleClassname, suppressStandardClassnames },
    actions: { moveRule, onPropChange, onRuleRemove },
    disabled: disabledProp,
    parentDisabled,
    shiftUpDisabled,
    shiftDownDisabled,
    field: fieldProp,
    operator: operatorProp,
    value: valueProp,
    valueSource: valueSourceProp,
    // Drag-and-drop
    dropEffect = "move",
    groupItems = false,
    dragMonitorId = "",
    dropMonitorId = "",
    dndRef = null,
    dragRef = null,
    isDragging = false,
    isOver = false
  } = props;
  useDeprecatedProps("rule", !ruleProp);
  useReactDndWarning(enableDragAndDrop, !!(dragMonitorId || dropMonitorId || dndRef || dragRef));
  const disabled = !!parentDisabled || !!disabledProp;
  const rule = useMemo7(() => ruleProp != null ? ruleProp : { id, field: fieldProp != null ? fieldProp : (
    /* istanbul ignore next */
    ""
  ), operator: operatorProp != null ? operatorProp : (
    /* istanbul ignore next */
    ""
  ), value: valueProp, valueSource: valueSourceProp }, [fieldProp, id, operatorProp, ruleProp, valueProp, valueSourceProp]);
  const classNames = useMemo7(() => ({ shiftActions: clsx(suppressStandardClassnames || standardClassnames.shiftActions, classNamesProp.shiftActions), dragHandle: clsx(suppressStandardClassnames || standardClassnames.dragHandle, classNamesProp.dragHandle), fields: clsx(suppressStandardClassnames || standardClassnames.fields, classNamesProp.valueSelector, classNamesProp.fields), matchMode: clsx(suppressStandardClassnames || standardClassnames.matchMode, classNamesProp.valueSelector, classNamesProp.matchMode), matchThreshold: clsx(suppressStandardClassnames || standardClassnames.matchThreshold, classNamesProp.valueSelector, classNamesProp.matchThreshold), operators: clsx(suppressStandardClassnames || standardClassnames.operators, classNamesProp.valueSelector, classNamesProp.operators), valueSource: clsx(suppressStandardClassnames || standardClassnames.valueSource, classNamesProp.valueSelector, classNamesProp.valueSource), value: clsx(suppressStandardClassnames || standardClassnames.value, classNamesProp.value), cloneRule: clsx(suppressStandardClassnames || standardClassnames.cloneRule, classNamesProp.actionElement, classNamesProp.cloneRule), lockRule: clsx(suppressStandardClassnames || standardClassnames.lockRule, classNamesProp.actionElement, classNamesProp.lockRule), removeRule: clsx(suppressStandardClassnames || standardClassnames.removeRule, classNamesProp.actionElement, classNamesProp.removeRule), valueListItem: clsx(suppressStandardClassnames || standardClassnames.valueListItem, classNamesProp.valueListItem) }), [classNamesProp.shiftActions, classNamesProp.dragHandle, classNamesProp.valueSelector, classNamesProp.fields, classNamesProp.matchMode, classNamesProp.matchThreshold, classNamesProp.operators, classNamesProp.valueSource, classNamesProp.value, classNamesProp.actionElement, classNamesProp.cloneRule, classNamesProp.lockRule, classNamesProp.removeRule, classNamesProp.valueListItem, suppressStandardClassnames]);
  const getChangeHandler = useCallback6(
    // oxlint-disable-next-line typescript/no-explicit-any
    (prop) => (value, context) => {
      if (!disabled) {
        onPropChange(prop, value, path, context);
      }
    },
    [disabled, onPropChange, path]
  );
  const onChangeField = useMemo7(() => getChangeHandler("field"), [getChangeHandler]);
  const onChangeOperator = useMemo7(() => getChangeHandler("operator"), [getChangeHandler]);
  const onChangeMatchMode = useMemo7(() => getChangeHandler("match"), [getChangeHandler]);
  const onChangeValueSource = useMemo7(() => getChangeHandler("valueSource"), [getChangeHandler]);
  const onChangeValue = useMemo7(() => getChangeHandler("value"), [getChangeHandler]);
  const cloneRule = useCallback6((_event, context_0) => {
    if (!disabled) {
      const newPath = [...getParentPath(path), path.at(-1) + 1];
      moveRule(path, newPath, true, context_0);
    }
  }, [disabled, moveRule, path]);
  const toggleLockRule = useCallback6((_event_0, context_1) => onPropChange("disabled", !disabled, path, context_1), [disabled, onPropChange, path]);
  const removeRule = useCallback6((_event_1, _context) => {
    if (!disabled) {
      onRuleRemove(path);
    }
  }, [disabled, onRuleRemove, path]);
  const shiftRuleUp = useCallback6((event, context_2) => {
    if (!disabled && !shiftUpDisabled) {
      moveRule(path, "up", event == null ? void 0 : event.altKey, context_2);
    }
  }, [disabled, moveRule, path, shiftUpDisabled]);
  const shiftRuleDown = useCallback6((event_0, context_3) => {
    if (!disabled && !shiftDownDisabled) {
      moveRule(path, "down", event_0 == null ? void 0 : event_0.altKey, context_3);
    }
  }, [disabled, moveRule, path, shiftDownDisabled]);
  const fieldData = useMemo7(() => {
    var _a2;
    return (_a2 = fieldMap == null ? void 0 : fieldMap[rule.field]) != null ? _a2 : { name: rule.field, value: rule.field, label: rule.field };
  }, [fieldMap, rule.field]);
  const inputType = useMemo7(() => {
    var _a2_0;
    return (_a2_0 = fieldData.inputType) != null ? _a2_0 : getInputType(rule.field, rule.operator, { fieldData });
  }, [fieldData, getInputType, rule.field, rule.operator]);
  const matchModes = useMemo7(() => getMatchModes(rule.field, { fieldData }), [fieldData, getMatchModes, rule.field]);
  const operators = useMemo7(() => getOperators(rule.field, { fieldData }), [fieldData, getOperators, rule.field]);
  const operatorObject = useMemo7(() => getOption(operators, rule.operator), [operators, rule.operator]);
  const arity = operatorObject == null ? void 0 : operatorObject.arity;
  const hideValueControls = typeof arity === "string" && arity === "unary" || typeof arity === "number" && arity < 2;
  const valueSourceOptions = useMemo7(() => {
    const configuredVSs = getValueSources(rule.field, rule.operator, { fieldData });
    if (rule.valueSource && !getOption(configuredVSs, rule.valueSource)) {
      return [...configuredVSs, { name: rule.valueSource, value: rule.valueSource, label: rule.valueSource }];
    }
    return configuredVSs;
  }, [fieldData, getValueSources, rule.field, rule.operator, rule.valueSource]);
  const valueSources = useMemo7(() => valueSourceOptions.map((_ref) => {
    let { value: value_0 } = _ref;
    return value_0;
  }), [valueSourceOptions]);
  const valueEditorType = useMemo7(() => rule.valueSource === "field" ? "select" : getValueEditorType(rule.field, rule.operator, { fieldData }), [fieldData, getValueEditorType, rule.field, rule.operator, rule.valueSource]);
  const valueEditorSeparator = useMemo7(() => getValueEditorSeparator(rule.field, rule.operator, { fieldData }), [fieldData, getValueEditorSeparator, rule.field, rule.operator]);
  const values = useMemo7(() => {
    const v = rule.valueSource === "field" ? filterFieldsByComparator(fieldData, fields, rule.operator) : getValues(rule.field, rule.operator, { fieldData });
    return isFlexibleOptionArray(v) || isFlexibleOptionGroupArray(v) ? toFullOptionList(v) : v;
  }, [fieldData, fields, getValues, rule.field, rule.operator, rule.valueSource]);
  const subQueryBuilderProps = useMemo7(() => getSubQueryBuilderProps(rule.field, { fieldData }), [fieldData, getSubQueryBuilderProps, rule.field]);
  const subproperties = useFields({ translations: props.translations, fields: (_b = (_a = fieldData.subproperties) != null ? _a : subQueryBuilderProps.fields) != null ? _b : defaultSubproperties, autoSelectField: props.schema.autoSelectField || !!fieldData.subproperties });
  const validationResult = useMemo7(() => {
    var _a2_1;
    return (_a2_1 = validationMap[id != null ? id : (
      /* istanbul ignore next */
      ""
    )]) != null ? _a2_1 : typeof fieldData.validator === "function" ? fieldData.validator(rule) : null;
  }, [fieldData, id, rule, validationMap]);
  const validationClassName = useMemo7(() => getValidationClassNames(validationResult), [validationResult]);
  const fieldBasedClassName = useMemo7(() => {
    var _a2_2;
    return (_a2_2 = fieldData == null ? void 0 : fieldData.className) != null ? _a2_2 : "";
  }, [fieldData == null ? void 0 : fieldData.className]);
  const operatorBasedClassName = useMemo7(() => {
    var _a2_3;
    return (_a2_3 = operatorObject == null ? void 0 : operatorObject.className) != null ? _a2_3 : "";
  }, [operatorObject == null ? void 0 : operatorObject.className]);
  const hasSubQuery = useMemo7(() => matchModes.length > 0, [matchModes.length]);
  const outerClassName = useMemo7(() => clsx(
    getRuleClassname(rule, { fieldData }),
    fieldBasedClassName,
    operatorBasedClassName,
    suppressStandardClassnames || standardClassnames.rule,
    classNamesProp.rule,
    // custom conditional classes
    disabled && classNamesProp.disabled,
    isDragging && classNamesProp.dndDragging,
    isOver && classNamesProp.dndOver,
    isOver && dropEffect === "copy" && classNamesProp.dndCopy,
    isOver && groupItems && classNamesProp.dndGroup,
    hasSubQuery && classNamesProp.hasSubQuery,
    // standard conditional classes
    suppressStandardClassnames || { [standardClassnames.disabled]: disabled, [standardClassnames.dndDragging]: isDragging, [standardClassnames.dndOver]: isOver, [standardClassnames.dndCopy]: isOver && dropEffect === "copy", [standardClassnames.dndGroup]: isOver && groupItems, [standardClassnames.hasSubQuery]: hasSubQuery },
    validationClassName
  ), [classNamesProp.disabled, classNamesProp.dndCopy, classNamesProp.dndDragging, classNamesProp.dndGroup, classNamesProp.dndOver, classNamesProp.hasSubQuery, classNamesProp.rule, disabled, dropEffect, fieldBasedClassName, fieldData, getRuleClassname, groupItems, hasSubQuery, isDragging, isOver, operatorBasedClassName, rule, suppressStandardClassnames, validationClassName]);
  return __spreadProps37(__spreadValues38({}, props), { classNames, cloneRule, disabled, dndRef, dragMonitorId, dragRef, dropMonitorId, fieldData, generateOnChangeHandler: getChangeHandler, onChangeField, onChangeMatchMode, onChangeOperator, onChangeValueSource, onChangeValue, hideValueControls, inputType, matchModes, operators, outerClassName, removeRule, rule, shiftRuleUp, shiftRuleDown, subproperties, subQueryBuilderProps, toggleLockRule, validationResult, valueEditorSeparator, valueEditorType, values, valueSourceOptions, valueSources });
};

// src/components/ShiftActions.tsx
import { jsx as jsx10, jsxs as jsxs5 } from "react/jsx-runtime";
var ShiftActions = (props) => {
  var _a, _b, _c, _d;
  return /* @__PURE__ */ jsxs5("div", { "data-testid": props.testID, className: props.className, children: [/* @__PURE__ */ jsx10("button", { disabled: props.disabled || props.shiftUpDisabled, onClick: props.shiftUp, title: (_a = props.titles) == null ? void 0 : _a.shiftUp, children: (_b = props.labels) == null ? void 0 : _b.shiftUp }), /* @__PURE__ */ jsx10("button", { disabled: props.disabled || props.shiftDownDisabled, onClick: props.shiftDown, title: (_c = props.titles) == null ? void 0 : _c.shiftDown, children: (_d = props.labels) == null ? void 0 : _d.shiftDown })] });
};

// src/components/ValueEditor.tsx
import { jsx as jsx11, jsxs as jsxs6 } from "react/jsx-runtime";
import { produce as produce8 } from "immer";
import * as React8 from "react";
import { useCallback as useCallback7, useEffect as useEffect2, useMemo as useMemo8 } from "react";
var __defProp39 = Object.defineProperty;
var __defProps38 = Object.defineProperties;
var __getOwnPropDescs38 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols39 = Object.getOwnPropertySymbols;
var __hasOwnProp39 = Object.prototype.hasOwnProperty;
var __propIsEnum39 = Object.prototype.propertyIsEnumerable;
var __defNormalProp39 = (obj, key, value) => key in obj ? __defProp39(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues39 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp39.call(b, prop)) __defNormalProp39(a, prop, b[prop]);
  if (__getOwnPropSymbols39) for (var prop of __getOwnPropSymbols39(b)) {
    if (__propIsEnum39.call(b, prop)) __defNormalProp39(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps38 = (a, b) => __defProps38(a, __getOwnPropDescs38(b));
var __objRest3 = (source, exclude) => {
  var target = {};
  for (var prop in source) if (__hasOwnProp39.call(source, prop) && exclude.indexOf(prop) < 0) target[prop] = source[prop];
  if (source != null && __getOwnPropSymbols39) for (var prop of __getOwnPropSymbols39(source)) {
    if (exclude.indexOf(prop) < 0 && __propIsEnum39.call(source, prop)) target[prop] = source[prop];
  }
  return target;
};
var RadioButton = (t0) => {
  const $ = c(9);
  const { name, disabled, checked, label, handleOnChange } = t0;
  const id = React8.useId();
  let t1;
  if ($[0] !== handleOnChange) {
    t1 = (e) => handleOnChange(e.target.value);
    $[0] = handleOnChange;
    $[1] = t1;
  } else {
    t1 = $[1];
  }
  let t2;
  if ($[2] !== checked || $[3] !== disabled || $[4] !== id || $[5] !== label || $[6] !== name || $[7] !== t1) {
    t2 = jsxs6("label", { htmlFor: id, children: [jsx11("input", { id, type: "radio", value: name, disabled, checked, onChange: t1 }), label] });
    $[2] = checked;
    $[3] = disabled;
    $[4] = id;
    $[5] = label;
    $[6] = name;
    $[7] = t1;
    $[8] = t2;
  } else {
    t2 = $[8];
  }
  return t2;
};
var ValueEditor = (allProps) => {
  var _b;
  const _a = allProps, { operator, value, handleOnChange, title, className, type = "text", inputType, values = [], listsAsArrays, fieldData, disabled, separator = null, testID, selectorComponent: SelectorComponent = allProps.schema.controls.valueSelector, parseNumbers: _parseNumbers, skipHook: _skipHook, valueSource: _valueSource } = _a, propsForValueSelector = __objRest3(_a, [
    "operator",
    "value",
    "handleOnChange",
    "title",
    "className",
    "type",
    "inputType",
    "values",
    "listsAsArrays",
    "fieldData",
    "disabled",
    "separator",
    "testID",
    "selectorComponent",
    // Some value selectors spread all extra props to the rendered component, so
    // we cherry pick these out of `propsForValueSelector` to keep them from being
    // assigned to DOM elements. (The props with mixed case are the only ones that
    // really matter. Props in all lowercase don't emit warnings.)
    "parseNumbers",
    "skipHook",
    "valueSource"
  ]);
  const { valueAsArray, multiValueHandler, bigIntValueHandler, parseNumberMethod, valueListItemClassName, inputTypeCoerced } = useValueEditor(allProps);
  if (operator === "null" || operator === "notNull") {
    return null;
  }
  const placeHolderText = (_b = fieldData == null ? void 0 : fieldData.placeholder) != null ? _b : "";
  if ((operator === "between" || operator === "notBetween") && (type === "select" || type === "text")) {
    const editors = ["from", "to"].map((key, i) => {
      var _a2, _b2;
      if (type === "text") {
        return /* @__PURE__ */ jsx11("input", { type: inputTypeCoerced, placeholder: placeHolderText, value: (_a2 = valueAsArray[i]) != null ? _a2 : "", className: valueListItemClassName, disabled, onChange: (e) => multiValueHandler(e.target.value, i) }, key);
      }
      return /* @__PURE__ */ jsx11(SelectorComponent, __spreadProps38(__spreadValues39({}, propsForValueSelector), { schema: allProps.schema, className: valueListItemClassName, handleOnChange: (v) => multiValueHandler(v, i), disabled, value: (_b2 = valueAsArray[i]) != null ? _b2 : getFirstOption(values), options: values, listsAsArrays }), key);
    });
    return /* @__PURE__ */ jsxs6("span", { "data-testid": testID, className, title, children: [editors[0], separator, editors[1]] });
  }
  switch (type) {
    case "select":
    case "multiselect":
      return /* @__PURE__ */ jsx11(SelectorComponent, __spreadProps38(__spreadValues39({}, propsForValueSelector), { schema: allProps.schema, testID, className, title, handleOnChange, disabled, value, options: values, multiple: type === "multiselect", listsAsArrays }));
    case "textarea":
      return /* @__PURE__ */ jsx11("textarea", { "data-testid": testID, placeholder: placeHolderText, value, title, className, disabled, onChange: (e_1) => handleOnChange(e_1.target.value) });
    case "switch":
    case "checkbox":
      return /* @__PURE__ */ jsx11("input", { "data-testid": testID, type: "checkbox", className, title, onChange: (e_0) => handleOnChange(e_0.target.checked), checked: !!value, disabled });
    case "radio":
      return /* @__PURE__ */ jsx11("span", { "data-testid": testID, className, title, children: values.map((v_0) => /* @__PURE__ */ jsx11(RadioButton, { name: v_0.name, disabled, checked: value === v_0.name, handleOnChange, label: v_0.label }, v_0.name)) });
  }
  if (inputType === "bigint") {
    return /* @__PURE__ */ jsx11("input", { "data-testid": testID, type: inputTypeCoerced, placeholder: placeHolderText, value: `${value}`, title, className, disabled, onChange: (e_2) => bigIntValueHandler(e_2.target.value) });
  }
  return /* @__PURE__ */ jsx11("input", { "data-testid": testID, type: inputTypeCoerced, placeholder: placeHolderText, value, title, className, disabled, onChange: (e_3) => handleOnChange(parseNumber(e_3.target.value, { parseNumbers: parseNumberMethod })) });
};
var useValueEditor = (props) => {
  const { handleOnChange, inputType, operator, value, listsAsArrays, parseNumbers, values, type, skipHook, schema: { classNames: classNamesProp, suppressStandardClassnames } } = props;
  useEffect2(() => {
    var _a;
    if (!skipHook && type !== "multiselect" && !["between", "notBetween", "in", "notIn"].includes(operator) && (Array.isArray(value) || inputType === "number" && typeof value === "string" && value.includes(","))) {
      handleOnChange((_a = toArray(value, { retainEmptyStrings: true })[0]) != null ? _a : "");
    }
  }, [handleOnChange, inputType, operator, skipHook, type, value]);
  const valueAsArray = useMemo8(() => toArray(value, { retainEmptyStrings: true }), [value]);
  const parseNumberMethod = useMemo8(() => getParseNumberMethod({ parseNumbers, inputType }), [inputType, parseNumbers]);
  const multiValueHandler = useCallback7(
    // oxlint-disable-next-line typescript/no-explicit-any
    (val, idx) => {
      const v = produce8(valueAsArray, (va) => {
        va[idx] = parseNumber(val, { parseNumbers: parseNumberMethod });
        if (idx === 0 && (operator === "between" || operator === "notBetween") && (va.length < 2 || va[1] === void 0)) {
          va[1] = getFirstOption(values);
        }
      });
      handleOnChange(listsAsArrays ? v : joinWith(v, ","));
    },
    [handleOnChange, listsAsArrays, operator, parseNumberMethod, valueAsArray, values]
  );
  const bigIntValueHandler = useCallback7((v_0) => {
    const valAsMaybeNumber = parseNumber(v_0, { parseNumbers: parseNumberMethod, bigIntOnOverflow: true });
    let bi;
    try {
      bi = BigInt(valAsMaybeNumber);
    } catch (e) {
      handleOnChange(valAsMaybeNumber);
      return;
    }
    handleOnChange(bi);
  }, [handleOnChange, parseNumberMethod]);
  const valueListItemClassName = clsx_default(
    suppressStandardClassnames || standardClassnames.valueListItem,
    // Optional chaining is necessary for QueryBuilderNative
    classNamesProp == null ? void 0 : classNamesProp.valueListItem
  );
  const inputTypeCoerced = inputType === "bigint" || operator === "in" || operator === "notIn" ? "text" : inputType || "text";
  return { valueAsArray, multiValueHandler, bigIntValueHandler, parseNumberMethod, valueListItemClassName, inputTypeCoerced };
};

// src/components/ValueSelector.tsx
import { jsx as jsx12 } from "react/jsx-runtime";
import { useCallback as useCallback8, useMemo as useMemo9 } from "react";
var ValueSelector = (props) => {
  const $ = c(12);
  const { onChange, val } = useValueSelector(props);
  let t0;
  if ($[0] !== onChange || $[1] !== props.multiple) {
    t0 = { multiple: props.multiple, onChange };
    $[0] = onChange;
    $[1] = props.multiple;
    $[2] = t0;
  } else {
    t0 = $[2];
  }
  const selectElementChangeHandler = useSelectElementChangeHandler(t0);
  const t1 = !!props.multiple;
  let t2;
  if ($[3] !== props.className || $[4] !== props.disabled || $[5] !== props.options || $[6] !== props.testID || $[7] !== props.title || $[8] !== selectElementChangeHandler || $[9] !== t1 || $[10] !== val) {
    t2 = jsx12("select", { "data-testid": props.testID, className: props.className, value: val, title: props.title, disabled: props.disabled, multiple: t1, onChange: selectElementChangeHandler, children: toOptions(props.options) });
    $[3] = props.className;
    $[4] = props.disabled;
    $[5] = props.options;
    $[6] = props.testID;
    $[7] = props.title;
    $[8] = selectElementChangeHandler;
    $[9] = t1;
    $[10] = val;
    $[11] = t2;
  } else {
    t2 = $[11];
  }
  return t2;
};
var useValueSelector = (props) => {
  const $ = c(10);
  const { handleOnChange, listsAsArrays: t0, multiple: t1, value } = props;
  const listsAsArrays = t0 === void 0 ? false : t0;
  const multiple = t1 === void 0 ? false : t1;
  let t2;
  if ($[0] !== handleOnChange || $[1] !== listsAsArrays || $[2] !== multiple) {
    t2 = (v) => {
      if (multiple) {
        const valueAsArray = toArray(v);
        handleOnChange(listsAsArrays ? valueAsArray : joinWith(valueAsArray, ","));
      } else {
        handleOnChange(v);
      }
    };
    $[0] = handleOnChange;
    $[1] = listsAsArrays;
    $[2] = multiple;
    $[3] = t2;
  } else {
    t2 = $[3];
  }
  const onChange = t2;
  let t3;
  let t4;
  if ($[4] !== multiple || $[5] !== value) {
    t4 = multiple ? toArray(value) : value;
    $[4] = multiple;
    $[5] = value;
    $[6] = t4;
  } else {
    t4 = $[6];
  }
  t3 = t4;
  const val = t3;
  let t5;
  if ($[7] !== onChange || $[8] !== val) {
    t5 = { onChange, val };
    $[7] = onChange;
    $[8] = val;
    $[9] = t5;
  } else {
    t5 = $[9];
  }
  return t5;
};

// src/defaultControlElements.ts
var defaultTranslations = { fields: { title: "Fields", placeholderName: defaultPlaceholderFieldName, placeholderLabel: defaultPlaceholderFieldLabel, placeholderGroupLabel: defaultPlaceholderFieldGroupLabel }, operators: { title: "Operators", placeholderName: defaultPlaceholderOperatorName, placeholderLabel: defaultPlaceholderOperatorLabel, placeholderGroupLabel: defaultPlaceholderOperatorGroupLabel }, values: { title: "Values", placeholderName: defaultPlaceholderValueName, placeholderLabel: defaultPlaceholderValueLabel, placeholderGroupLabel: defaultPlaceholderValueGroupLabel }, matchMode: { title: "Match mode" }, matchThreshold: { title: "Match threshold" }, value: { title: "Value" }, removeRule: { label: "\u2A2F", title: "Remove rule" }, removeGroup: { label: "\u2A2F", title: "Remove group" }, addRule: { label: "+ Rule", title: "Add rule" }, addGroup: { label: "+ Group", title: "Add group" }, combinators: { title: "Combinators" }, notToggle: { label: "Not", title: "Invert this group" }, cloneRule: { label: "\u29C9", title: "Clone rule" }, cloneRuleGroup: { label: "\u29C9", title: "Clone group" }, shiftActionUp: { label: "\u02C4", title: "Shift up" }, shiftActionDown: { label: "\u02C5", title: "Shift down" }, dragHandle: { label: "\u205E\u205E", title: "Drag handle" }, lockRule: { label: "\u{1F513}", title: "Lock rule" }, lockGroup: { label: "\u{1F513}", title: "Lock group" }, lockRuleDisabled: { label: "\u{1F512}", title: "Unlock rule" }, lockGroupDisabled: { label: "\u{1F512}", title: "Unlock group" }, valueSourceSelector: { title: "Value source" } };
var defaultControlElements = { actionElement: ActionElement, addGroupAction: ActionElement, addRuleAction: ActionElement, cloneGroupAction: ActionElement, cloneRuleAction: ActionElement, combinatorSelector: ValueSelector, dragHandle: DragHandle, fieldSelector: ValueSelector, inlineCombinator: InlineCombinator, lockGroupAction: ActionElement, lockRuleAction: ActionElement, matchModeEditor: MatchModeEditor, notToggle: NotToggle, operatorSelector: ValueSelector, removeGroupAction: ActionElement, removeRuleAction: ActionElement, rule: Rule, ruleGroup: RuleGroup, ruleGroupBodyElements: RuleGroupBodyComponents, ruleGroupHeaderElements: RuleGroupHeaderComponents, shiftActions: ShiftActions, valueEditor: ValueEditor, valueSelector: ValueSelector, valueSourceSelector: ValueSelector };

// src/hooks/useMergedContext.ts
var __getOwnPropSymbols40 = Object.getOwnPropertySymbols;
var __hasOwnProp40 = Object.prototype.hasOwnProperty;
var __propIsEnum40 = Object.prototype.propertyIsEnumerable;
var __objRest4 = (source, exclude) => {
  var target = {};
  for (var prop in source) if (__hasOwnProp40.call(source, prop) && exclude.indexOf(prop) < 0) target[prop] = source[prop];
  if (source != null && __getOwnPropSymbols40) for (var prop of __getOwnPropSymbols40(source)) {
    if (exclude.indexOf(prop) < 0 && __propIsEnum40.call(source, prop)) target[prop] = source[prop];
  }
  return target;
};
var nullComp = () => null;
var nullFwdComp = /* @__PURE__ */ forwardRef2(nullComp);
var emptyObject = {};
var useMergedContext = (_a) => {
  var _b = _a, { finalize } = _b, props = __objRest4(_b, ["finalize"]);
  var _a2, _b2, _c, _d, _e, _f, _g, _h, _i, _j, _k, _l, _m, _n, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z, _A, _B, _C, _D, _E, _F, _G, _H, _I, _J, _K, _L, _M, _N, _O, _P, _Q, _R, _S, _T, _U, _V, _W, _X, _Y, _Z, __, _$, _aa, _ba, _ca, _da, _ea, _fa, _ga, _ha, _ia, _ja, _ka, _la, _ma, _na, _oa, _pa, _qa, _ra, _sa, _ta, _ua, _va, _wa, _xa, _ya, _za, _Aa, _Ba, _Ca, _Da, _Ea, _Fa, _Ga, _Ha, _Ia, _Ja, _Ka, _La, _Ma;
  const rqbContext = useContext2(QueryBuilderContext);
  const debugModePreferred = usePreferProp(false, props.debugMode, rqbContext.debugMode);
  const debugMode = finalize ? debugModePreferred : (_a2 = props.debugMode) != null ? _a2 : rqbContext.debugMode;
  const enableMountQueryChangePreferred = usePreferProp(true, props.enableMountQueryChange, rqbContext.enableMountQueryChange);
  const enableMountQueryChange = finalize ? enableMountQueryChangePreferred : (_b2 = props.enableMountQueryChange) != null ? _b2 : rqbContext.enableMountQueryChange;
  const enableDragAndDropPreferred = usePreferProp(false, props.enableDragAndDrop, rqbContext.enableDragAndDrop) && rqbContext.enableDragAndDrop !== false;
  const enableDragAndDrop = finalize ? enableDragAndDropPreferred : (_c = props.enableDragAndDrop) != null ? _c : rqbContext.enableDragAndDrop;
  const cc = useMemo10(() => mergeClassnames(finalize ? Object.assign({}, defaultControlClassnames) : emptyObject, rqbContext.controlClassnames, props.controlClassnames), [rqbContext.controlClassnames, props.controlClassnames, finalize]);
  const controlClassnames = useMemo10(() => ({ actionElement: cc.actionElement, addGroup: cc.addGroup, addRule: cc.addRule, body: cc.body, cloneGroup: cc.cloneGroup, cloneRule: cc.cloneRule, combinators: cc.combinators, dragHandle: cc.dragHandle, fields: cc.fields, header: cc.header, lockGroup: cc.lockGroup, lockRule: cc.lockRule, notToggle: cc.notToggle, operators: cc.operators, queryBuilder: cc.queryBuilder, removeGroup: cc.removeGroup, removeRule: cc.removeRule, rule: cc.rule, ruleGroup: cc.ruleGroup, shiftActions: cc.shiftActions, value: cc.value, valueSelector: cc.valueSelector, valueSource: cc.valueSource, betweenRules: cc.betweenRules, valid: cc.valid, invalid: cc.invalid, dndDragging: cc.dndDragging, dndOver: cc.dndOver, dndCopy: cc.dndCopy, dndGroup: cc.dndGroup, disabled: cc.disabled, valueListItem: cc.valueListItem, matchMode: cc.matchMode, matchThreshold: cc.matchThreshold, branches: cc.branches, hasSubQuery: cc.hasSubQuery }), [cc.actionElement, cc.addGroup, cc.addRule, cc.betweenRules, cc.body, cc.branches, cc.cloneGroup, cc.cloneRule, cc.combinators, cc.disabled, cc.dndCopy, cc.dndGroup, cc.dndDragging, cc.dndOver, cc.dragHandle, cc.fields, cc.hasSubQuery, cc.header, cc.invalid, cc.lockGroup, cc.lockRule, cc.matchMode, cc.matchThreshold, cc.notToggle, cc.operators, cc.queryBuilder, cc.removeGroup, cc.removeRule, cc.rule, cc.ruleGroup, cc.shiftActions, cc.valid, cc.value, cc.valueListItem, cc.valueSelector, cc.valueSource]);
  const contextCE = (_d = rqbContext.controlElements) != null ? _d : emptyObject;
  const propsCE = (_e = props.controlElements) != null ? _e : emptyObject;
  const mergeControlElement = useCallback9((name, propComp, contextComp) => {
    var _a3, _b3, _c2;
    const nc = name === "dragHandle" ? nullFwdComp : nullComp;
    const propBulkOverride = (_a3 = name.endsWith("Action") && propsCE.actionElement ? propsCE.actionElement : void 0) != null ? _a3 : name.endsWith("Selector") && propsCE.valueSelector ? propsCE.valueSelector : void 0;
    const contextBulkOverride = (_b3 = name.endsWith("Action") && contextCE.actionElement ? contextCE.actionElement : void 0) != null ? _b3 : name.endsWith("Selector") && contextCE.valueSelector ? contextCE.valueSelector : void 0;
    const comp = propComp === null ? nc : (_c2 = propComp != null ? propComp : finalize ? propBulkOverride : void 0) != null ? _c2 : contextComp === null ? nc : contextComp != null ? contextComp : finalize ? contextBulkOverride : void 0;
    return comp ? { [name]: comp } : finalize ? { [name]: defaultControlElements[name] } : emptyObject;
  }, [contextCE.actionElement, contextCE.valueSelector, finalize, propsCE.actionElement, propsCE.valueSelector]);
  const controlElements = useMemo10(() => {
    var _a3_0, _b3_0, _c2_0, _d2, _e2, _f2;
    return (
      // For some reason TypeScript doesn't like the object spread syntax here.
      // Something about a union type being too complex to represent.
      Object.assign({}, mergeControlElement("addGroupAction", propsCE.addGroupAction, contextCE.addGroupAction), mergeControlElement("addRuleAction", propsCE.addRuleAction, contextCE.addRuleAction), mergeControlElement("cloneGroupAction", propsCE.cloneGroupAction, contextCE.cloneGroupAction), mergeControlElement("cloneRuleAction", propsCE.cloneRuleAction, contextCE.cloneRuleAction), mergeControlElement("combinatorSelector", propsCE.combinatorSelector, contextCE.combinatorSelector), mergeControlElement("dragHandle", propsCE.dragHandle, contextCE.dragHandle), mergeControlElement("fieldSelector", propsCE.fieldSelector, contextCE.fieldSelector), mergeControlElement("inlineCombinator", propsCE.inlineCombinator, contextCE.inlineCombinator), mergeControlElement("lockGroupAction", propsCE.lockGroupAction, contextCE.lockGroupAction), mergeControlElement("lockRuleAction", propsCE.lockRuleAction, contextCE.lockRuleAction), mergeControlElement("notToggle", propsCE.notToggle, contextCE.notToggle), mergeControlElement("operatorSelector", propsCE.operatorSelector, contextCE.operatorSelector), mergeControlElement("removeGroupAction", propsCE.removeGroupAction, contextCE.removeGroupAction), mergeControlElement("removeRuleAction", propsCE.removeRuleAction, contextCE.removeRuleAction), mergeControlElement("shiftActions", propsCE.shiftActions, contextCE.shiftActions), { valueEditor: propsCE.valueEditor === null ? nullComp : (_b3_0 = (_a3_0 = propsCE.valueEditor) != null ? _a3_0 : contextCE.valueEditor === null ? nullComp : contextCE.valueEditor) != null ? _b3_0 : defaultControlElements.valueEditor }, mergeControlElement("valueSourceSelector", propsCE.valueSourceSelector, contextCE.valueSourceSelector), mergeControlElement("matchModeEditor", propsCE.matchModeEditor, contextCE.matchModeEditor), mergeControlElement("rule", propsCE.rule, contextCE.rule), mergeControlElement("ruleGroup", propsCE.ruleGroup, contextCE.ruleGroup), mergeControlElement("ruleGroupBodyElements", propsCE.ruleGroupBodyElements, contextCE.ruleGroupBodyElements), mergeControlElement("ruleGroupHeaderElements", propsCE.ruleGroupHeaderElements, contextCE.ruleGroupHeaderElements), { actionElement: (_d2 = (_c2_0 = propsCE.actionElement) != null ? _c2_0 : contextCE.actionElement) != null ? _d2 : finalize ? defaultControlElements.actionElement : void 0 }, { valueSelector: (_f2 = (_e2 = propsCE.valueSelector) != null ? _e2 : contextCE.valueSelector) != null ? _f2 : finalize ? defaultControlElements.valueSelector : void 0 })
    );
  }, [contextCE.actionElement, contextCE.addGroupAction, contextCE.addRuleAction, contextCE.cloneGroupAction, contextCE.cloneRuleAction, contextCE.combinatorSelector, contextCE.dragHandle, contextCE.fieldSelector, contextCE.inlineCombinator, contextCE.lockGroupAction, contextCE.lockRuleAction, contextCE.matchModeEditor, contextCE.notToggle, contextCE.operatorSelector, contextCE.removeGroupAction, contextCE.removeRuleAction, contextCE.rule, contextCE.ruleGroup, contextCE.ruleGroupBodyElements, contextCE.ruleGroupHeaderElements, contextCE.shiftActions, contextCE.valueEditor, contextCE.valueSelector, contextCE.valueSourceSelector, mergeControlElement, finalize, propsCE.actionElement, propsCE.addGroupAction, propsCE.addRuleAction, propsCE.cloneGroupAction, propsCE.cloneRuleAction, propsCE.combinatorSelector, propsCE.dragHandle, propsCE.fieldSelector, propsCE.inlineCombinator, propsCE.lockGroupAction, propsCE.lockRuleAction, propsCE.matchModeEditor, propsCE.notToggle, propsCE.operatorSelector, propsCE.removeGroupAction, propsCE.removeRuleAction, propsCE.rule, propsCE.ruleGroup, propsCE.ruleGroupBodyElements, propsCE.ruleGroupHeaderElements, propsCE.shiftActions, propsCE.valueEditor, propsCE.valueSelector, propsCE.valueSourceSelector]);
  const propsT = (_f = props.translations) != null ? _f : emptyObject;
  const contextT = (_g = rqbContext.translations) != null ? _g : emptyObject;
  const translations = useMemo10(() => {
    var _a3_1, _b3_1, _c2_1, _d2_0, _e2_0, _f2_0, _g2, _h2, _i2, _j2, _k2, _l2, _m2, _n2, _o2, _p2, _q2, _r2, _s2, _t2, _u2, _v2, _w2, _x2, _y2, _z2, _A2, _B2, _C2, _D2, _E2, _F2, _G2, _H2, _I2, _J2, _K2, _L2, _M2, _N2, _O2, _P2, _Q2, _R2, _S2, _T2, _U2, _V2, _W2, _X2, _Y2, _Z2, __2, _$2, _aa2, _ba2, _ca2, _da2, _ea2, _fa2, _ga2, _ha2, _ia2, _ja2, _ka2, _la2, _ma2, _na2, _oa2, _pa2, _qa2, _ra2, _sa2, _ta2, _ua2, _va2, _wa2, _xa2, _ya2, _za2, _Aa2, _Ba2, _Ca2, _Da2, _Ea2, _Fa2;
    return Object.assign(finalize ? Object.assign({}, defaultTranslations) : {}, mergeTranslation("addGroup", { label: [(_a3_1 = propsT.addGroup) == null ? void 0 : _a3_1.label, (_b3_1 = contextT.addGroup) == null ? void 0 : _b3_1.label], title: [(_c2_1 = propsT.addGroup) == null ? void 0 : _c2_1.title, (_d2_0 = contextT.addGroup) == null ? void 0 : _d2_0.title] }, finalize), mergeTranslation("addRule", { label: [(_e2_0 = propsT.addRule) == null ? void 0 : _e2_0.label, (_f2_0 = contextT.addRule) == null ? void 0 : _f2_0.label], title: [(_g2 = propsT.addRule) == null ? void 0 : _g2.title, (_h2 = contextT.addRule) == null ? void 0 : _h2.title] }, finalize), mergeTranslation("cloneRule", { label: [(_i2 = propsT.cloneRule) == null ? void 0 : _i2.label, (_j2 = contextT.cloneRule) == null ? void 0 : _j2.label], title: [(_k2 = propsT.cloneRule) == null ? void 0 : _k2.title, (_l2 = contextT.cloneRule) == null ? void 0 : _l2.title] }, finalize), mergeTranslation("cloneRuleGroup", { label: [(_m2 = propsT.cloneRuleGroup) == null ? void 0 : _m2.label, (_n2 = contextT.cloneRuleGroup) == null ? void 0 : _n2.label], title: [(_o2 = propsT.cloneRuleGroup) == null ? void 0 : _o2.title, (_p2 = contextT.cloneRuleGroup) == null ? void 0 : _p2.title] }, finalize), mergeTranslation("combinators", { title: [(_q2 = propsT.combinators) == null ? void 0 : _q2.title, (_r2 = contextT.combinators) == null ? void 0 : _r2.title] }, finalize), mergeTranslation("dragHandle", { label: [(_s2 = propsT.dragHandle) == null ? void 0 : _s2.label, (_t2 = contextT.dragHandle) == null ? void 0 : _t2.label], title: [(_u2 = propsT.dragHandle) == null ? void 0 : _u2.title, (_v2 = contextT.dragHandle) == null ? void 0 : _v2.title] }, finalize), mergeTranslation("fields", { placeholderGroupLabel: [(_w2 = propsT.fields) == null ? void 0 : _w2.placeholderGroupLabel, (_x2 = contextT.fields) == null ? void 0 : _x2.placeholderGroupLabel], placeholderLabel: [(_y2 = propsT.fields) == null ? void 0 : _y2.placeholderLabel, (_z2 = contextT.fields) == null ? void 0 : _z2.placeholderLabel], placeholderName: [(_A2 = propsT.fields) == null ? void 0 : _A2.placeholderName, (_B2 = contextT.fields) == null ? void 0 : _B2.placeholderName], title: [(_C2 = propsT.fields) == null ? void 0 : _C2.title, (_D2 = contextT.fields) == null ? void 0 : _D2.title] }, finalize), mergeTranslation("lockGroup", { label: [(_E2 = propsT.lockGroup) == null ? void 0 : _E2.label, (_F2 = contextT.lockGroup) == null ? void 0 : _F2.label], title: [(_G2 = propsT.lockGroup) == null ? void 0 : _G2.title, (_H2 = contextT.lockGroup) == null ? void 0 : _H2.title] }, finalize), mergeTranslation("lockGroupDisabled", { label: [(_I2 = propsT.lockGroupDisabled) == null ? void 0 : _I2.label, (_J2 = contextT.lockGroupDisabled) == null ? void 0 : _J2.label], title: [(_K2 = propsT.lockGroupDisabled) == null ? void 0 : _K2.title, (_L2 = contextT.lockGroupDisabled) == null ? void 0 : _L2.title] }, finalize), mergeTranslation("lockRule", { label: [(_M2 = propsT.lockRule) == null ? void 0 : _M2.label, (_N2 = contextT.lockRule) == null ? void 0 : _N2.label], title: [(_O2 = propsT.lockRule) == null ? void 0 : _O2.title, (_P2 = contextT.lockRule) == null ? void 0 : _P2.title] }, finalize), mergeTranslation("lockRuleDisabled", { label: [(_Q2 = propsT.lockRuleDisabled) == null ? void 0 : _Q2.label, (_R2 = contextT.lockRuleDisabled) == null ? void 0 : _R2.label], title: [(_S2 = propsT.lockRuleDisabled) == null ? void 0 : _S2.title, (_T2 = contextT.lockRuleDisabled) == null ? void 0 : _T2.title] }, finalize), mergeTranslation("notToggle", { label: [(_U2 = propsT.notToggle) == null ? void 0 : _U2.label, (_V2 = contextT.notToggle) == null ? void 0 : _V2.label], title: [(_W2 = propsT.notToggle) == null ? void 0 : _W2.title, (_X2 = contextT.notToggle) == null ? void 0 : _X2.title] }, finalize), mergeTranslation("operators", { placeholderGroupLabel: [(_Y2 = propsT.operators) == null ? void 0 : _Y2.placeholderGroupLabel, (_Z2 = contextT.operators) == null ? void 0 : _Z2.placeholderGroupLabel], placeholderLabel: [(__2 = propsT.operators) == null ? void 0 : __2.placeholderLabel, (_$2 = contextT.operators) == null ? void 0 : _$2.placeholderLabel], placeholderName: [(_aa2 = propsT.operators) == null ? void 0 : _aa2.placeholderName, (_ba2 = contextT.operators) == null ? void 0 : _ba2.placeholderName], title: [(_ca2 = propsT.operators) == null ? void 0 : _ca2.title, (_da2 = contextT.operators) == null ? void 0 : _da2.title] }, finalize), mergeTranslation("values", { placeholderGroupLabel: [(_ea2 = propsT.values) == null ? void 0 : _ea2.placeholderGroupLabel, (_fa2 = contextT.values) == null ? void 0 : _fa2.placeholderGroupLabel], placeholderLabel: [(_ga2 = propsT.values) == null ? void 0 : _ga2.placeholderLabel, (_ha2 = contextT.values) == null ? void 0 : _ha2.placeholderLabel], placeholderName: [(_ia2 = propsT.values) == null ? void 0 : _ia2.placeholderName, (_ja2 = contextT.values) == null ? void 0 : _ja2.placeholderName], title: [(_ka2 = propsT.values) == null ? void 0 : _ka2.title, (_la2 = contextT.values) == null ? void 0 : _la2.title] }, finalize), mergeTranslation("removeGroup", { label: [(_ma2 = propsT.removeGroup) == null ? void 0 : _ma2.label, (_na2 = contextT.removeGroup) == null ? void 0 : _na2.label], title: [(_oa2 = propsT.removeGroup) == null ? void 0 : _oa2.title, (_pa2 = contextT.removeGroup) == null ? void 0 : _pa2.title] }, finalize), mergeTranslation("removeRule", { label: [(_qa2 = propsT.removeRule) == null ? void 0 : _qa2.label, (_ra2 = contextT.removeRule) == null ? void 0 : _ra2.label], title: [(_sa2 = propsT.removeRule) == null ? void 0 : _sa2.title, (_ta2 = contextT.removeRule) == null ? void 0 : _ta2.title] }, finalize), mergeTranslation("shiftActionDown", { label: [(_ua2 = propsT.shiftActionDown) == null ? void 0 : _ua2.label, (_va2 = contextT.shiftActionDown) == null ? void 0 : _va2.label], title: [(_wa2 = propsT.shiftActionDown) == null ? void 0 : _wa2.title, (_xa2 = contextT.shiftActionDown) == null ? void 0 : _xa2.title] }, finalize), mergeTranslation("shiftActionUp", { label: [(_ya2 = propsT.shiftActionUp) == null ? void 0 : _ya2.label, (_za2 = contextT.shiftActionUp) == null ? void 0 : _za2.label], title: [(_Aa2 = propsT.shiftActionUp) == null ? void 0 : _Aa2.title, (_Ba2 = contextT.shiftActionUp) == null ? void 0 : _Ba2.title] }, finalize), mergeTranslation("value", { title: [(_Ca2 = propsT.value) == null ? void 0 : _Ca2.title, (_Da2 = contextT.value) == null ? void 0 : _Da2.title] }, finalize), mergeTranslation("valueSourceSelector", { title: [(_Ea2 = propsT.valueSourceSelector) == null ? void 0 : _Ea2.title, (_Fa2 = contextT.valueSourceSelector) == null ? void 0 : _Fa2.title] }, finalize));
  }, [(_h = contextT.addGroup) == null ? void 0 : _h.label, (_i = contextT.addGroup) == null ? void 0 : _i.title, (_j = contextT.addRule) == null ? void 0 : _j.label, (_k = contextT.addRule) == null ? void 0 : _k.title, (_l = contextT.cloneRule) == null ? void 0 : _l.label, (_m = contextT.cloneRule) == null ? void 0 : _m.title, (_n = contextT.cloneRuleGroup) == null ? void 0 : _n.label, (_o = contextT.cloneRuleGroup) == null ? void 0 : _o.title, (_p = contextT.combinators) == null ? void 0 : _p.title, (_q = contextT.dragHandle) == null ? void 0 : _q.label, (_r = contextT.dragHandle) == null ? void 0 : _r.title, (_s = contextT.fields) == null ? void 0 : _s.placeholderGroupLabel, (_t = contextT.fields) == null ? void 0 : _t.placeholderLabel, (_u = contextT.fields) == null ? void 0 : _u.placeholderName, (_v = contextT.fields) == null ? void 0 : _v.title, (_w = contextT.lockGroup) == null ? void 0 : _w.label, (_x = contextT.lockGroup) == null ? void 0 : _x.title, (_y = contextT.lockGroupDisabled) == null ? void 0 : _y.label, (_z = contextT.lockGroupDisabled) == null ? void 0 : _z.title, (_A = contextT.lockRule) == null ? void 0 : _A.label, (_B = contextT.lockRule) == null ? void 0 : _B.title, (_C = contextT.lockRuleDisabled) == null ? void 0 : _C.label, (_D = contextT.lockRuleDisabled) == null ? void 0 : _D.title, (_E = contextT.notToggle) == null ? void 0 : _E.label, (_F = contextT.notToggle) == null ? void 0 : _F.title, (_G = contextT.operators) == null ? void 0 : _G.placeholderGroupLabel, (_H = contextT.operators) == null ? void 0 : _H.placeholderLabel, (_I = contextT.operators) == null ? void 0 : _I.placeholderName, (_J = contextT.operators) == null ? void 0 : _J.title, (_K = contextT.removeGroup) == null ? void 0 : _K.label, (_L = contextT.removeGroup) == null ? void 0 : _L.title, (_M = contextT.removeRule) == null ? void 0 : _M.label, (_N = contextT.removeRule) == null ? void 0 : _N.title, (_O = contextT.shiftActionDown) == null ? void 0 : _O.label, (_P = contextT.shiftActionDown) == null ? void 0 : _P.title, (_Q = contextT.shiftActionUp) == null ? void 0 : _Q.label, (_R = contextT.shiftActionUp) == null ? void 0 : _R.title, (_S = contextT.value) == null ? void 0 : _S.title, (_T = contextT.values) == null ? void 0 : _T.placeholderGroupLabel, (_U = contextT.values) == null ? void 0 : _U.placeholderLabel, (_V = contextT.values) == null ? void 0 : _V.placeholderName, (_W = contextT.values) == null ? void 0 : _W.title, (_X = contextT.valueSourceSelector) == null ? void 0 : _X.title, finalize, (_Y = propsT.addGroup) == null ? void 0 : _Y.label, (_Z = propsT.addGroup) == null ? void 0 : _Z.title, (__ = propsT.addRule) == null ? void 0 : __.label, (_$ = propsT.addRule) == null ? void 0 : _$.title, (_aa = propsT.cloneRule) == null ? void 0 : _aa.label, (_ba = propsT.cloneRule) == null ? void 0 : _ba.title, (_ca = propsT.cloneRuleGroup) == null ? void 0 : _ca.label, (_da = propsT.cloneRuleGroup) == null ? void 0 : _da.title, (_ea = propsT.combinators) == null ? void 0 : _ea.title, (_fa = propsT.dragHandle) == null ? void 0 : _fa.label, (_ga = propsT.dragHandle) == null ? void 0 : _ga.title, (_ha = propsT.fields) == null ? void 0 : _ha.placeholderGroupLabel, (_ia = propsT.fields) == null ? void 0 : _ia.placeholderLabel, (_ja = propsT.fields) == null ? void 0 : _ja.placeholderName, (_ka = propsT.fields) == null ? void 0 : _ka.title, (_la = propsT.lockGroup) == null ? void 0 : _la.label, (_ma = propsT.lockGroup) == null ? void 0 : _ma.title, (_na = propsT.lockGroupDisabled) == null ? void 0 : _na.label, (_oa = propsT.lockGroupDisabled) == null ? void 0 : _oa.title, (_pa = propsT.lockRule) == null ? void 0 : _pa.label, (_qa = propsT.lockRule) == null ? void 0 : _qa.title, (_ra = propsT.lockRuleDisabled) == null ? void 0 : _ra.label, (_sa = propsT.lockRuleDisabled) == null ? void 0 : _sa.title, (_ta = propsT.notToggle) == null ? void 0 : _ta.label, (_ua = propsT.notToggle) == null ? void 0 : _ua.title, (_va = propsT.operators) == null ? void 0 : _va.placeholderGroupLabel, (_wa = propsT.operators) == null ? void 0 : _wa.placeholderLabel, (_xa = propsT.operators) == null ? void 0 : _xa.placeholderName, (_ya = propsT.operators) == null ? void 0 : _ya.title, (_za = propsT.removeGroup) == null ? void 0 : _za.label, (_Aa = propsT.removeGroup) == null ? void 0 : _Aa.title, (_Ba = propsT.removeRule) == null ? void 0 : _Ba.label, (_Ca = propsT.removeRule) == null ? void 0 : _Ca.title, (_Da = propsT.shiftActionDown) == null ? void 0 : _Da.label, (_Ea = propsT.shiftActionDown) == null ? void 0 : _Ea.title, (_Fa = propsT.shiftActionUp) == null ? void 0 : _Fa.label, (_Ga = propsT.shiftActionUp) == null ? void 0 : _Ga.title, (_Ha = propsT.value) == null ? void 0 : _Ha.title, (_Ia = propsT.values) == null ? void 0 : _Ia.placeholderGroupLabel, (_Ja = propsT.values) == null ? void 0 : _Ja.placeholderLabel, (_Ka = propsT.values) == null ? void 0 : _Ka.placeholderName, (_La = propsT.values) == null ? void 0 : _La.title, (_Ma = propsT.valueSourceSelector) == null ? void 0 : _Ma.title]);
  return { controlClassnames, controlElements, debugMode, enableDragAndDrop, enableMountQueryChange, translations, initialQuery: props.initialQuery, qbId: props.qbId };
};

// src/components/QueryBuilderInternal.tsx
import { jsx as jsx13 } from "react/jsx-runtime";
var __defProp40 = Object.defineProperty;
var __defProps39 = Object.defineProperties;
var __getOwnPropDescs39 = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols41 = Object.getOwnPropertySymbols;
var __hasOwnProp41 = Object.prototype.hasOwnProperty;
var __propIsEnum41 = Object.prototype.propertyIsEnumerable;
var __defNormalProp40 = (obj, key, value) => key in obj ? __defProp40(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues40 = (a, b) => {
  for (var prop in b || (b = {})) if (__hasOwnProp41.call(b, prop)) __defNormalProp40(a, prop, b[prop]);
  if (__getOwnPropSymbols41) for (var prop of __getOwnPropSymbols41(b)) {
    if (__propIsEnum41.call(b, prop)) __defNormalProp40(a, prop, b[prop]);
  }
  return a;
};
var __spreadProps39 = (a, b) => __defProps39(a, __getOwnPropDescs39(b));
var QueryBuilderInternal = (t0) => {
  const $ = c(14);
  const { props } = t0;
  const qb = useQueryBuilder(props);
  const RuleGroupControlElement = qb.schema.controls.ruleGroup;
  let t1;
  if ($[0] !== RuleGroupControlElement || $[1] !== props.context || $[2] !== qb.actions || $[3] !== qb.combinatorPropObject || $[4] !== qb.dndEnabledAttr || $[5] !== qb.inlineCombinatorsAttr || $[6] !== qb.queryDisabled || $[7] !== qb.rootGroup || $[8] !== qb.rootGroupDisabled || $[9] !== qb.rqbContext || $[10] !== qb.schema || $[11] !== qb.translations || $[12] !== qb.wrapperClassName) {
    t1 = jsx13(QueryBuilderContext.Provider, { value: qb.rqbContext, children: jsx13("div", { role: "form", className: qb.wrapperClassName, "data-dnd": qb.dndEnabledAttr, "data-inlinecombinators": qb.inlineCombinatorsAttr, children: jsx13(RuleGroupControlElement, __spreadProps39(__spreadValues40({ translations: qb.translations, ruleGroup: qb.rootGroup, rules: qb.rootGroup.rules }, qb.combinatorPropObject), { not: !!qb.rootGroup.not, schema: qb.schema, actions: qb.actions, id: qb.rootGroup.id, path: rootPath, disabled: qb.rootGroupDisabled, shiftUpDisabled: true, shiftDownDisabled: true, parentDisabled: qb.queryDisabled, context: props.context })) }) }, qb.dndEnabledAttr);
    $[0] = RuleGroupControlElement;
    $[1] = props.context;
    $[2] = qb.actions;
    $[3] = qb.combinatorPropObject;
    $[4] = qb.dndEnabledAttr;
    $[5] = qb.inlineCombinatorsAttr;
    $[6] = qb.queryDisabled;
    $[7] = qb.rootGroup;
    $[8] = qb.rootGroupDisabled;
    $[9] = qb.rqbContext;
    $[10] = qb.schema;
    $[11] = qb.translations;
    $[12] = qb.wrapperClassName;
    $[13] = t1;
  } else {
    t1 = $[13];
  }
  return t1;
};

export {
  QueryBuilderContext,
  ActionElement,
  DragHandle,
  defaultPlaceholderName,
  defaultPlaceholderLabel,
  defaultPlaceholderFieldName,
  defaultPlaceholderFieldLabel,
  defaultPlaceholderFieldGroupLabel,
  defaultPlaceholderOperatorName,
  defaultPlaceholderOperatorLabel,
  defaultPlaceholderOperatorGroupLabel,
  defaultPlaceholderValueName,
  defaultPlaceholderValueLabel,
  defaultPlaceholderValueGroupLabel,
  defaultJoinChar,
  defaultOperatorLabelMap,
  defaultCombinatorLabelMap,
  defaultOperators,
  defaultOperatorNegationMap,
  defaultCombinators,
  defaultCombinatorsExtended,
  defaultMatchModes,
  standardClassnames,
  defaultControlClassnames,
  groupInvalidReasons,
  TestID,
  LogType,
  rootPath,
  InlineCombinator,
  splitBy,
  joinWith,
  trimIfString,
  toArray,
  nullFreeArray,
  lc,
  numericRegex,
  isPojo,
  nullOrUndefinedOrEmpty,
  isRuleGroup,
  isRuleGroupType,
  isRuleGroupTypeIC,
  convertFromIC,
  convertToIC,
  convertQuery,
  defaultValidator,
  toFullOption,
  toFullOptionList,
  toFullOptionMap,
  uniqByName,
  uniqByIdentifier,
  isOptionGroupArray,
  isFlexibleOptionArray,
  isFullOptionArray,
  isFlexibleOptionGroupArray,
  isFullOptionGroupArray,
  getOption,
  getFirstOption,
  toFlatOptionArray,
  uniqOptGroups,
  uniqOptList,
  filterFieldsByComparator,
  parseNumber,
  transformQuery,
  isValidationResult,
  isRuleOrGroupValid,
  getParseNumberMethod,
  mapSQLOperator,
  mongoOperators,
  prismaOperators,
  celCombinatorMap,
  jsonLogicAdditionalOperators,
  numerifyValues,
  isValidValue,
  shouldRenderAsNumber,
  isValueProcessorLegacy,
  getQuoteFieldNamesWithArray,
  getQuotedFieldName,
  normalizeConstituentWordOrder,
  defaultNLTranslations,
  getNLTranslataion,
  processMatchMode,
  bigIntJsonStringifyReplacer,
  bigIntJsonParseReviver,
  defaultRuleGroupProcessorCEL,
  defaultRuleProcessorCEL,
  mongoDbFallback,
  defaultRuleGroupProcessorMongoDBQuery,
  defaultRuleProcessorMongoDBQuery,
  defaultRuleProcessorMongoDB,
  defaultRuleGroupProcessorSpEL,
  defaultRuleProcessorSpEL,
  defaultValueProcessorByRule,
  defaultRuleProcessorDrizzle,
  defaultRuleGroupProcessorDrizzle,
  defaultRuleGroupProcessorElasticSearch,
  defaultRuleGroupProcessorJSONata,
  defaultRuleGroupProcessorJsonLogic,
  defaultRuleGroupProcessorLDAP,
  defaultRuleGroupProcessorMongoDB,
  defaultRuleGroupProcessorNL,
  defaultRuleGroupProcessorParameterized,
  prismaFallback,
  defaultRuleGroupProcessorPrisma,
  defaultRuleGroupProcessorSequelize,
  defaultRuleGroupProcessorSQL,
  defaultRuleProcessorElasticSearch,
  defaultRuleProcessorJSONata,
  defaultRuleProcessorJsonLogic,
  defaultRuleProcessorLDAP,
  defaultValueProcessorNL,
  defaultExportOperatorMap,
  defaultOperatorProcessorNL,
  defaultRuleProcessorNL,
  defaultOperatorProcessorSQL,
  defaultRuleProcessorSQL,
  defaultRuleProcessorParameterized,
  defaultRuleProcessorPrisma,
  defaultRuleProcessorSequelize,
  sqlDialectPresets,
  formatQueryOptionPresets,
  formatQuery,
  defaultValueProcessor,
  defaultMongoDBValueProcessor,
  defaultCELValueProcessor,
  defaultSpELValueProcessor,
  defaultValueProcessorCELByRule,
  defaultValueProcessorMongoDBByRule,
  defaultValueProcessorSpELByRule,
  findPath,
  findID,
  getPathOfID,
  getParentPath,
  pathsAreEqual,
  isAncestor,
  getCommonAncestorPath,
  pathIsDisabled,
  generateAccessibleDescription,
  generateID,
  preferProp,
  usePreferProp,
  usePreferAnyProp,
  useMergedContext,
  getCompatContextProvider,
  getMatchModesUtil,
  getValidationClassNames,
  getValueSourcesUtil,
  mergeClassnames,
  objectKeys,
  objectEntries,
  mergeTranslations,
  mergeTranslation,
  prepareRule,
  prepareRuleGroup,
  prepareRuleOrGroup,
  regenerateID,
  regenerateIDs,
  add,
  update,
  remove,
  move,
  insert,
  group,
  toOptions,
  MatchModeEditor,
  useMatchModeEditor,
  NotToggle,
  messages,
  QueryBuilderStateContext,
  storeCommon,
  usePrevious,
  useControlledOrUncontrolled,
  useDeprecatedProps,
  useFields,
  useReactDndWarning,
  useSelectElementChangeHandler,
  useStopEventPropagation,
  getQuerySelectorById,
  useQueryBuilderSelector,
  useQueryBuilderQuery,
  useQueryBuilderSchema,
  useQueryBuilderSetup,
  useQueryBuilder,
  RuleGroup,
  RuleGroupHeaderComponents,
  RuleGroupBodyComponents,
  useRuleGroup,
  Rule,
  RuleComponents,
  RuleWithSubQueryGroupComponentsWrapper,
  RuleComponentsWithSubQuery,
  useRule,
  ShiftActions,
  ValueEditor,
  useValueEditor,
  ValueSelector,
  useValueSelector,
  defaultTranslations,
  defaultControlElements,
  QueryBuilderInternal
};
//# sourceMappingURL=chunk-WL5NAARS.js.map